@@ -10,12 +10,15 @@ import (
 
 	"github.com/pkg/errors"
 	log "github.com/sirupsen/logrus"
-	bind9config "isc.org/stork/appcfg/bind9"
+	bind9config "isc.org/stork/daemoncfg/bind9"
+	"isc.org/stork/datamodel/daemonname"
+	"isc.org/stork/datamodel/protocoltype"
 	storkutil "isc.org/stork/util"
 )
 
 var (
-	_ App             = (*Bind9App)(nil)
+	_ Daemon          = (*Bind9Daemon)(nil)
+	_ dnsDaemon       = (*Bind9Daemon)(nil)
 	_ bind9FileParser = (*bind9config.Parser)(nil)
 
 	// Patterns for detecting named process.
@@ -26,56 +29,19 @@ var (
 // An interface for parsing BIND 9 configuration files.
 // It is mocked in the tests.
 type bind9FileParser interface {
-	ParseFile(path string) (*bind9config.Config, error)
-}
-
-// Represents the BIND 9 process metadata.
-type Bind9Daemon struct {
-	Pid     int32
-	Name    string
-	Version string
-	Active  bool
-}
-
-// Represents the state of BIND 9.
-type Bind9State struct {
-	Version string
-	Active  bool
-	Daemon  Bind9Daemon
+	ParseFile(path string, chrootDir string) (*bind9config.Config, error)
 }
 
 // It holds common and BIND 9 specific runtime information.
-type Bind9App struct {
-	BaseApp
-	RndcClient    *RndcClient // to communicate with BIND 9 via rndc
-	zoneInventory *zoneInventory
+type Bind9Daemon struct {
+	dnsDaemonImpl
+	rndcClient    *RndcClient // to communicate with BIND 9 via rndc
+	pid           int32       // PID of the named process
+	bind9Config   *bind9config.Config
+	rndcKeyConfig *bind9config.Config
 }
 
-// Get base information about BIND 9 app.
-func (ba *Bind9App) GetBaseApp() *BaseApp {
-	return &ba.BaseApp
-}
-
-// Detect allowed logs provided by BIND 9.
-// TODO: currently it is not implemented and not used,
-// it returns always empty list and no error.
-func (ba *Bind9App) DetectAllowedLogs() ([]string, error) {
-	return nil, nil
-}
-
-// Stops the zone inventory.
-func (ba *Bind9App) StopZoneInventory() {
-	if ba.zoneInventory != nil {
-		ba.zoneInventory.stop()
-	}
-}
-
-// Returns the zone inventory instance associated with the BIND 9 app.
-func (ba *Bind9App) GetZoneInventory() *zoneInventory {
-	return ba.zoneInventory
-}
-
-// List of BIND 9 executables used during app detection.
+// List of BIND 9 executables used during daemon detection.
 const (
 	namedCheckconfExec = "named-checkconf"
 	rndcExec           = "rndc"
@@ -217,7 +183,7 @@ func parseNamedDefaultPath(output []byte) string {
 	return namedConfMatch[1]
 }
 
-// Detect the BIND 9 application by parsing the named process command line.
+// Detect the BIND 9 daemon by parsing the named process command line.
 // If the path to the configuration file is relative and chroot directory is
 // not specified, the path is resolved against the current working directory of
 // the process. If the chroot directory is specified, the path is resolved
@@ -239,13 +205,13 @@ func parseNamedDefaultPath(output []byte) string {
 // port, and secret key (if configured). The returned instance lacks information
 // about the active daemons. It must be detected separately.
 //
-// It returns the BIND 9 app instance or an error if the BIND 9 is not
+// It returns the BIND 9 daemon instance or an error if the BIND 9 is not
 // recognized or any error occurs.
 //
 // ToDo: Enable the linter check after splitting this function in #1991.
 //
 //nolint:gocyclo
-func detectBind9App(p supportedProcess, executor storkutil.CommandExecutor, explicitConfigPath string, parser bind9FileParser) (App, error) {
+func detectBind9Daemon(p supportedProcess, executor storkutil.CommandExecutor, explicitConfigPath string, parser bind9FileParser) (Daemon, error) {
 	cmdline, err := p.getCmdline()
 	if err != nil {
 		return nil, err
@@ -266,18 +232,18 @@ func detectBind9App(p supportedProcess, executor storkutil.CommandExecutor, expl
 	namedDir := match[1]
 	bind9Params := match[2]
 	// Path to actual chroot (empty if not used).
-	rootPrefix := ""
+	chrootDir := ""
 	// Absolute path from the actual root or chroot.
 	bind9ConfPath := ""
 
 	// Look for the chroot directory.
 	m := bind9ChrootPattern.FindStringSubmatch(bind9Params)
 	if m != nil {
-		rootPrefix = strings.TrimRight(m[1], "/")
+		chrootDir = strings.TrimRight(m[1], "/")
 
 		// The cwd path is already prefixed with the chroot directory
 		// because the /proc/(pid)/cwd is absolute.
-		cwd = strings.TrimPrefix(cwd, rootPrefix)
+		cwd = strings.TrimPrefix(cwd, chrootDir)
 	}
 
 	// Look for config file in cmd params.
@@ -302,11 +268,11 @@ func detectBind9App(p supportedProcess, executor storkutil.CommandExecutor, expl
 		if explicitConfigPath != "" {
 			log.Debugf("Looking for BIND 9 config in %s as explicitly specified in settings.", explicitConfigPath)
 			switch {
-			case !strings.HasPrefix(explicitConfigPath, rootPrefix):
-				log.Errorf("The explicitly specified config path must be inside the chroot directory: %s, got: %s", rootPrefix, explicitConfigPath)
+			case !strings.HasPrefix(explicitConfigPath, chrootDir):
+				log.Errorf("The explicitly specified config path must be inside the chroot directory: %s, got: %s", chrootDir, explicitConfigPath)
 			case executor.IsFileExist(explicitConfigPath):
-				// Trim the root prefix.
-				bind9ConfPath = explicitConfigPath[len(rootPrefix):]
+				// Trim the chroot directory.
+				bind9ConfPath = explicitConfigPath[len(chrootDir):]
 			default:
 				log.Errorf("File explicitly specified in settings (%s) not found or unreadable.", explicitConfigPath)
 			}
@@ -342,7 +308,7 @@ func detectBind9App(p supportedProcess, executor storkutil.CommandExecutor, expl
 		// config path not found in cmdline params so try to guess its location
 		for _, f := range getPotentialNamedConfLocations() {
 			// Concat with root or chroot.
-			fullPath := path.Join(rootPrefix, f, "named.conf")
+			fullPath := path.Join(chrootDir, f, "named.conf")
 			log.Debugf("Looking for BIND 9 config file in %s", fullPath)
 			if executor.IsFileExist(fullPath) {
 				bind9ConfPath = f
@@ -355,18 +321,17 @@ func detectBind9App(p supportedProcess, executor storkutil.CommandExecutor, expl
 	if bind9ConfPath == "" {
 		return nil, errors.Errorf("cannot find config file for BIND 9")
 	}
-	prefixedBind9ConfPath := path.Join(rootPrefix, bind9ConfPath)
 
 	// Parse the BIND 9 config file.
-	bind9Config, err := parser.ParseFile(prefixedBind9ConfPath)
+	bind9Config, err := parser.ParseFile(bind9ConfPath, chrootDir)
 	if err != nil {
-		return nil, errors.Wrapf(err, "failed to parse BIND 9 config file %s", prefixedBind9ConfPath)
+		return nil, errors.WithMessage(err, "failed to parse BIND 9 config file")
 	}
 
 	// Resolve include statements.
-	bind9Config, err = bind9Config.Expand(filepath.Dir(prefixedBind9ConfPath))
+	bind9Config, err = bind9Config.Expand()
 	if err != nil {
-		return nil, errors.Wrapf(err, "failed to resolve include statements in BIND 9 config file %s", prefixedBind9ConfPath)
+		return nil, errors.WithMessage(err, "failed to resolve include statements in BIND 9 config file")
 	}
 
 	if bind9Config.HasNoParse() {
@@ -377,21 +342,21 @@ func detectBind9App(p supportedProcess, executor storkutil.CommandExecutor, expl
 
 	// rndc.key file typically contains keys to be used for rndc authentication.
 	var rndcConfig *bind9config.Config
-	prefixedRndcKeyPath := filepath.Join(filepath.Dir(prefixedBind9ConfPath), RndcKeyFile)
-	if executor.IsFileExist(prefixedRndcKeyPath) {
-		rndcConfig, err = parser.ParseFile(prefixedRndcKeyPath)
+	rndcKeyPath := filepath.Join(filepath.Dir(bind9ConfPath), RndcKeyFile)
+	if executor.IsFileExist(path.Join(chrootDir, rndcKeyPath)) {
+		rndcConfig, err = parser.ParseFile(rndcKeyPath, chrootDir)
 		if err != nil {
-			return nil, errors.Wrapf(err, "failed to parse BIND 9 rndc key file %s", prefixedRndcKeyPath)
+			return nil, errors.WithMessage(err, "failed to parse BIND 9 rndc key file")
 		}
 	}
 
 	// look for control address in config
 	ctrlAddress, ctrlPort, ctrlKey, enabled, err := bind9Config.GetRndcConnParams(rndcConfig)
 	if err != nil {
-		return nil, errors.Wrapf(err, "failed to get BIND 9 rndc credentials")
+		return nil, errors.WithMessage(err, "failed to get BIND 9 rndc credentials")
 	}
 	if !enabled {
-		return nil, errors.Errorf("found BIND 9 config file (%s) but rndc support was disabled (empty `controls` clause)", prefixedBind9ConfPath)
+		return nil, errors.Errorf("found BIND 9 config file (%s) but rndc support was disabled (empty `controls` clause)", path.Join(chrootDir, bind9ConfPath))
 	}
 
 	rndcKey := ""
@@ -400,26 +365,28 @@ func detectBind9App(p supportedProcess, executor storkutil.CommandExecutor, expl
 		if err != nil {
 			return nil, err
 		}
-		rndcKey = fmt.Sprintf("%s:%s:%s", ctrlKey.Name, *algorithm, *secret)
+		rndcKey = fmt.Sprintf("%s:%s:%s", ctrlKey.Name, algorithm, secret)
 	}
 
 	accessPoints := []AccessPoint{
 		{
-			Type:    AccessPointControl,
-			Address: *ctrlAddress,
-			Port:    *ctrlPort,
-			Key:     rndcKey,
+			Type:     AccessPointControl,
+			Address:  *ctrlAddress,
+			Port:     *ctrlPort,
+			Key:      rndcKey,
+			Protocol: protocoltype.RNDC,
 		},
 	}
 
 	// look for statistics channel address in config
-	var inventory *zoneInventory
+	var inventory zoneInventory
 	address, port, enabled := bind9Config.GetStatisticsChannelConnParams()
 	if enabled {
 		accessPoints = append(accessPoints, AccessPoint{
-			Type:    AccessPointStatistics,
-			Address: *address,
-			Port:    *port,
+			Type:     AccessPointStatistics,
+			Address:  *address,
+			Port:     *port,
+			Protocol: protocoltype.HTTP,
 		})
 		client := NewBind9StatsClient()
 		// For larger deployments, it may take several minutes to retrieve the
@@ -437,7 +404,7 @@ func detectBind9App(p supportedProcess, executor storkutil.CommandExecutor, expl
 	err = rndcClient.DetermineDetails(
 		baseNamedDir,
 		// rndc client doesn't support chroot.
-		path.Dir(prefixedBind9ConfPath),
+		path.Join(chrootDir, path.Dir(bind9ConfPath)),
 		*ctrlAddress,
 		*ctrlPort,
 		ctrlKey,
@@ -446,20 +413,25 @@ func detectBind9App(p supportedProcess, executor storkutil.CommandExecutor, expl
 		return nil, errors.Wrapf(err, "failed to determine BIND 9 rndc details")
 	}
 
-	// prepare final BIND 9 app
-	bind9App := &Bind9App{
-		BaseApp: BaseApp{
-			Type:         AppTypeBind9,
-			AccessPoints: accessPoints,
+	// prepare final BIND 9 daemon
+	daemon := &Bind9Daemon{
+		dnsDaemonImpl: dnsDaemonImpl{
+			daemon: daemon{
+				Name:         daemonname.Bind9,
+				AccessPoints: accessPoints,
+			},
+			zoneInventory: inventory,
 		},
-		RndcClient:    rndcClient,
-		zoneInventory: inventory,
+		rndcClient:    rndcClient,
+		pid:           p.getPid(),
+		bind9Config:   bind9Config,
+		rndcKeyConfig: rndcConfig,
 	}
 
-	return bind9App, nil
+	return daemon, nil
 }
 
 // Send a command to named using rndc client.
-func (ba *Bind9App) sendCommand(command []string) (output []byte, err error) {
-	return ba.RndcClient.SendCommand(command)
+func (ba *Bind9Daemon) sendRNDCCommand(command []string) (output []byte, err error) {
+	return ba.rndcClient.SendCommand(command)
 }
