@@ -9,12 +9,14 @@ import (
 
 	"github.com/pkg/errors"
 	log "github.com/sirupsen/logrus"
-	pdnsconfig "isc.org/stork/appcfg/pdns"
+	pdnsconfig "isc.org/stork/daemoncfg/pdns"
+	"isc.org/stork/datamodel/daemonname"
 	storkutil "isc.org/stork/util"
 )
 
 var (
-	_ App              = (*PDNSApp)(nil)
+	_ Daemon           = (*pdnsDaemon)(nil)
+	_ dnsDaemon        = (*pdnsDaemon)(nil)
 	_ pdnsConfigParser = (*pdnsconfig.Parser)(nil)
 
 	// Pattern for detecting PowerDNS process.
@@ -37,42 +39,34 @@ type pdnsConfigParser interface {
 	ParseFile(path string) (*pdnsconfig.Config, error)
 }
 
-// PDNSApp implements the App interface for PowerDNS.
-type PDNSApp struct {
-	BaseApp
-	zoneInventory *zoneInventory
+// Implements the Daemon interface for PowerDNS.
+type pdnsDaemon struct {
+	dnsDaemonImpl
 }
 
-// Returns the base app.
-func (pa *PDNSApp) GetBaseApp() *BaseApp {
-	return &pa.BaseApp
-}
-
-// Returns the allowed logs. Always returns nil.
-func (pa *PDNSApp) DetectAllowedLogs() ([]string, error) {
-	return nil, nil
-}
-
-// Waits for the zone inventory to complete background tasks.
-func (pa *PDNSApp) AwaitBackgroundTasks() {
-	if pa.zoneInventory != nil {
-		pa.zoneInventory.awaitBackgroundTasks()
+// It returns the PowerDNS daemon instance or an error if the PowerDNS is not
+// recognized or any error occurs.
+func detectPowerDNSDaemon(p supportedProcess, commander storkutil.CommandExecutor, parser pdnsConfigParser, explicitConfigPath string) (Daemon, error) {
+	// PowerDNS server configuration location detection.
+	configPath, err := detectPowerDNSConfigPath(p, commander, explicitConfigPath)
+	if err != nil {
+		err = errors.WithMessage(err, "failed to detect PowerDNS server config path")
+		return nil, err
 	}
-}
+	log.WithFields(log.Fields{
+		"path": *configPath,
+	}).Debug("PowerDNS server config path detected")
 
-// Returns the zone inventory.
-func (pa *PDNSApp) GetZoneInventory() *zoneInventory {
-	return pa.zoneInventory
-}
-
-// Stops the zone inventory.
-func (pa *PDNSApp) StopZoneInventory() {
-	if pa.zoneInventory != nil {
-		pa.zoneInventory.stop()
+	// Parse and interpret the PowerDNS server configuration.
+	daemon, err := configurePowerDNSDaemon(*configPath, parser)
+	if err != nil {
+		err = errors.WithMessage(err, "PowerDNS server configuration is invalid")
+		return nil, err
 	}
+	return daemon, nil
 }
 
-// Detects the PowerDNS application config path using the following algorithm:
+// Detects the PowerDNS daemon config path using the following algorithm:
 //
 // STEP 1: Parse the command line arguments. If the config-dir is specified
 // in the process command line it points to the directory with the config file
@@ -89,7 +83,7 @@ func (pa *PDNSApp) StopZoneInventory() {
 // STEP 3: Try to find the config file in the common locations:
 // - Use the locations returned by getPotentialPDNSConfLocations() function.
 // - Prepend the chroot directory if it is set.
-func detectPowerDNSAppConfigPath(p supportedProcess, executor storkutil.CommandExecutor, explicitConfigPath string) (*string, error) {
+func detectPowerDNSConfigPath(p supportedProcess, executor storkutil.CommandExecutor, explicitConfigPath string) (*string, error) {
 	// We can't proceed without the command line.
 	cmdline, err := p.getCmdline()
 	if err != nil {
@@ -105,7 +99,7 @@ func detectPowerDNSAppConfigPath(p supportedProcess, executor storkutil.CommandE
 	// STEP 1: Let's try to parse --chroot, --config-dir and --config-name parameters passed to pdns_server.
 	log.Debug("Looking for PowerDNS config file in --config-dir and --config-name parameters of a running process.")
 
-	var configDir, configName, configPath, rootPrefix string
+	var configDir, configName, configPath, chrootDir string
 	if len(match) >= 3 {
 		// The command line contains parameters. Check if they specify config
 		// directory or config name.
@@ -118,7 +112,7 @@ func detectPowerDNSAppConfigPath(p supportedProcess, executor storkutil.CommandE
 			}
 			switch key {
 			case "--chroot":
-				rootPrefix = strings.TrimRight(value, "/")
+				chrootDir = strings.TrimRight(value, "/")
 			case "--config-dir":
 				configDir = strings.TrimRight(value, "/")
 			case "--config-name":
@@ -129,10 +123,10 @@ func detectPowerDNSAppConfigPath(p supportedProcess, executor storkutil.CommandE
 	log.WithFields(log.Fields{
 		"config-dir":  configDir,
 		"config-name": configName,
-		"chroot":      rootPrefix,
+		"chroot":      chrootDir,
 	}).Debug("PowerDNS was started with the following command line arguments")
 
-	if rootPrefix != "" && !filepath.IsAbs(rootPrefix) {
+	if chrootDir != "" && !filepath.IsAbs(chrootDir) {
 		// If the chroot directory is relative, we can use cwd as chroot because
 		// cwd is absolute and points to the chroot directory.
 		cwd, err := p.getCwd()
@@ -141,8 +135,8 @@ func detectPowerDNSAppConfigPath(p supportedProcess, executor storkutil.CommandE
 			// We can't reliably proceed.
 			return nil, errors.Wrapf(err, "failed to get PowerDNS current working directory to determine absolute chroot path")
 		}
-		log.Debugf("The PowerDNS chroot directory (%s) is relative. Using current working directory (%s) as chroot", rootPrefix, cwd)
-		rootPrefix = cwd
+		log.Debugf("The PowerDNS chroot directory (%s) is relative. Using current working directory (%s) as chroot", chrootDir, cwd)
+		chrootDir = cwd
 	}
 
 	// The default config file name is pdns.conf. The name can be overridden
@@ -159,7 +153,7 @@ func detectPowerDNSAppConfigPath(p supportedProcess, executor storkutil.CommandE
 	if configDir != "" {
 		if !filepath.IsAbs(configDir) {
 			// The config directory location is relative.
-			if rootPrefix != "" {
+			if chrootDir != "" {
 				// Using a relative config directory with chroot is not supported. Suppose
 				// we start the server like this:
 				// pdns_server --chroot=/home/frank/chroot --config-dir=frank/chroot/etc.
@@ -169,7 +163,7 @@ func detectPowerDNSAppConfigPath(p supportedProcess, executor storkutil.CommandE
 				// rather point me to the chroot directory, not the directory from which the
 				// process was started. We still can use other methods to detect the config file
 				// but let's log the issue.
-				log.Warnf("Config directory (%s) is relative while chroot is set (%s)", configDir, rootPrefix)
+				log.Warnf("Config directory (%s) is relative while chroot is set (%s)", configDir, chrootDir)
 				log.Warn("Unable to match relative config directory against chroot directory. Falling back to other possible locations")
 			} else {
 				// The config directory location is relative but the chroot is not set.
@@ -199,14 +193,14 @@ func detectPowerDNSAppConfigPath(p supportedProcess, executor storkutil.CommandE
 	if explicitConfigPath != "" {
 		var candidatePath string
 		log.Debugf("Looking for PowerDNS config in the location explicitly specified in settings: %s", explicitConfigPath)
-		if rootPrefix != "" {
-			rel, err := filepath.Rel(rootPrefix, explicitConfigPath)
+		if chrootDir != "" {
+			rel, err := filepath.Rel(chrootDir, explicitConfigPath)
 			if err != nil || strings.HasPrefix(rel, "..") {
 				// The explicit config path does not belong to the chroot directory when
 				// it is impossible to build a relative path between the two (error case).
 				// If the explicit path is a parent of the chroot directory, it is also
 				// wrong (the double dot case).
-				log.Errorf("The explicitly specified config path must be inside the chroot directory: %s, got: %s", rootPrefix, explicitConfigPath)
+				log.Errorf("The explicitly specified config path must be inside the chroot directory: %s, got: %s", chrootDir, explicitConfigPath)
 			} else {
 				candidatePath = explicitConfigPath
 			}
@@ -232,7 +226,7 @@ func detectPowerDNSAppConfigPath(p supportedProcess, executor storkutil.CommandE
 	log.Debugf("Looking for PowerDNS config file in typical locations")
 	for _, location := range getPotentialPDNSConfLocations() {
 		// Concat with root or chroot.
-		path := filepath.Join(rootPrefix, location, configFileName)
+		path := filepath.Join(chrootDir, location, configFileName)
 		log.Debugf("Checking if config file exists: %s", path)
 		if executor.IsFileExist(path) {
 			return &path, nil
@@ -245,7 +239,7 @@ func detectPowerDNSAppConfigPath(p supportedProcess, executor storkutil.CommandE
 // the webserver configuration and the API key. If the webserver is disabled or the
 // API key does not exist it returns an error. Otherwise it instantiates the
 // PowerDNS app and the zone inventory.
-func configurePowerDNSApp(configPath string, parser pdnsConfigParser) (App, error) {
+func configurePowerDNSDaemon(configPath string, parser pdnsConfigParser) (*pdnsDaemon, error) {
 	// Parse the configuration file.
 	parsedConfig, err := parser.ParseFile(configPath)
 	if err != nil {
@@ -271,19 +265,21 @@ func configurePowerDNSApp(configPath string, parser pdnsConfigParser) (App, erro
 	inventory := newZoneInventory(newZoneInventoryStorageMemory(), parsedConfig, client, *webserverAddress, *webserverPort)
 
 	// Create the PowerDNS app.
-	pdnsApp := &PDNSApp{
-		BaseApp: BaseApp{
-			Type: AppTypePowerDNS,
-			AccessPoints: []AccessPoint{
-				{
-					Type:    AccessPointControl,
-					Address: *webserverAddress,
-					Port:    *webserverPort,
-					Key:     *key,
+	daemon := &pdnsDaemon{
+		dnsDaemonImpl: dnsDaemonImpl{
+			daemon: daemon{
+				Name: daemonname.PDNS,
+				AccessPoints: []AccessPoint{
+					{
+						Type:    AccessPointControl,
+						Address: *webserverAddress,
+						Port:    *webserverPort,
+						Key:     *key,
+					},
 				},
 			},
+			zoneInventory: inventory,
 		},
-		zoneInventory: inventory,
 	}
-	return pdnsApp, nil
+	return daemon, nil
 }
