@@ -2,8 +2,6 @@ package agent
 
 import (
 	"context"
-	"errors"
-	"fmt"
 	"math"
 	"net"
 	"net/http"
@@ -12,7 +10,7 @@ import (
 	"strings"
 	"time"
 
-	pkgerrors "github.com/pkg/errors"
+	"github.com/pkg/errors"
 	"github.com/prometheus/client_golang/prometheus"
 	"github.com/prometheus/client_golang/prometheus/collectors"
 	versioncollector "github.com/prometheus/client_golang/prometheus/collectors/version"
@@ -21,6 +19,8 @@ import (
 	log "github.com/sirupsen/logrus"
 
 	"isc.org/stork"
+	"isc.org/stork/datamodel/daemonname"
+	storkutil "isc.org/stork/util"
 )
 
 const (
@@ -56,7 +56,7 @@ type PromBind9ExporterStats struct {
 }
 
 // Main structure for Prometheus BIND 9 Exporter. It holds its config,
-// references to app monitor, HTTP client, HTTP server, and mappings
+// references to daemon monitor, HTTP client, HTTP server, and mappings
 // between BIND 9 stats names to prometheus stats.
 type PromBind9Exporter struct {
 	Host string
@@ -64,12 +64,11 @@ type PromBind9Exporter struct {
 
 	StartTime time.Time
 
-	AppMonitor AppMonitor
+	Monitor    Monitor
 	HTTPClient *bind9StatsClient
 	HTTPServer *http.Server
 
 	up               int
-	procID           int32
 	procExporter     prometheus.Collector
 	Registry         *prometheus.Registry
 	serverStatsDesc  map[string]*prometheus.Desc
@@ -80,12 +79,12 @@ type PromBind9Exporter struct {
 }
 
 // Create new Prometheus BIND 9 Exporter.
-func NewPromBind9Exporter(host string, port int, appMonitor AppMonitor, httpClient *bind9StatsClient) *PromBind9Exporter {
+func NewPromBind9Exporter(host string, port int, monitor Monitor, httpClient *bind9StatsClient) *PromBind9Exporter {
 	pbe := &PromBind9Exporter{
 		Host:       host,
 		Port:       port,
 		StartTime:  time.Now(),
-		AppMonitor: appMonitor,
+		Monitor:    monitor,
 		HTTPClient: httpClient,
 		Registry:   prometheus.NewRegistry(),
 	}
@@ -487,7 +486,7 @@ func (pbe *PromBind9Exporter) qryRTTHistogram(stats map[string]float64) (uint64,
 				rtt := strings.TrimPrefix(statName, qryRTT)
 				bucket, err = strconv.ParseFloat(rtt, 64)
 				if err != nil {
-					return 0, math.NaN(), buckets, pkgerrors.Errorf("could not parse RTT: %s", rtt)
+					return 0, math.NaN(), buckets, errors.Errorf("could not parse RTT: %s", rtt)
 				}
 			}
 			buckets[bucket/1000] = uint64(statValue)
@@ -540,7 +539,7 @@ func (pbe *PromBind9Exporter) trafficSizesHistogram(stats map[string]float64) (c
 			}
 			bucket, err = strconv.ParseFloat(sizes[1], 64)
 			if err != nil {
-				return 0, math.NaN(), buckets, pkgerrors.Errorf("could not parse size: %s", sizes[1])
+				return 0, math.NaN(), buckets, errors.Errorf("could not parse size: %s", sizes[1])
 			}
 		}
 		buckets[bucket] = uint64(statValue)
@@ -589,10 +588,9 @@ func (pbe *PromBind9Exporter) collectResolverLabelStat(statName, view string, vi
 // Collect fetches the stats from configured location and delivers them
 // as Prometheus metrics. It implements prometheus.Collector.
 func (pbe *PromBind9Exporter) Collect(ch chan<- prometheus.Metric) {
-	var err error
-	pbe.procID, err = pbe.collectStats()
-	if pbe.procID == 0 {
-		return
+	err := pbe.collectStats()
+	if err != nil {
+		log.WithError(err).Error("Some error occurred while collecting BIND 9 stats")
 	}
 
 	// Collect basic metrics (uptime, up status, time metrics)
@@ -885,8 +883,7 @@ func (pbe *PromBind9Exporter) collectViewMetrics(ch chan<- prometheus.Metric) {
 // exposing them to Prometheus.
 func (pbe *PromBind9Exporter) Start() {
 	// initial collect
-	var err error
-	pbe.procID, err = pbe.collectStats()
+	err := pbe.collectStats()
 	if err != nil {
 		log.WithError(err).Error("Some errors were encountered while collecting stats from BIND 9")
 	}
@@ -897,7 +894,15 @@ func (pbe *PromBind9Exporter) Start() {
 	pbe.procExporter = collectors.NewProcessCollector(
 		collectors.ProcessCollectorOpts{
 			PidFn: func() (int, error) {
-				return int(pbe.procID), nil
+				for _, d := range pbe.Monitor.GetDaemons() {
+					if d.GetName() == daemonname.Bind9 {
+						daemonBIND9 := d.(*Bind9Daemon)
+						pid := daemonBIND9.pid
+						return int(pid), nil
+					}
+				}
+
+				return 0, errors.New("BIND 9 daemon not found")
 			},
 			Namespace: namespace,
 		})
@@ -930,12 +935,12 @@ func (pbe *PromBind9Exporter) Shutdown() {
 
 		pbe.HTTPServer.SetKeepAlivesEnabled(false)
 		if err := pbe.HTTPServer.Shutdown(ctx); err != nil {
-			log.Warnf("Could not gracefully shut down the BIND 9 exporter: %v\n", err)
+			log.WithError(err).Warn("Could not gracefully shut down the BIND 9 exporter")
 		}
 	}
 
 	// unregister bind9 counters from prometheus framework
-	if pbe.procID > 0 {
+	if pbe.procExporter != nil {
 		pbe.Registry.Unregister(pbe.procExporter)
 	}
 	pbe.Registry.Unregister(pbe)
@@ -961,7 +966,7 @@ func (pbe *PromBind9Exporter) scrapeServerStat(statMap map[string]interface{}, s
 	if statIfc != nil {
 		stats, ok := statIfc.(map[string]interface{})
 		if !ok {
-			return nil, pkgerrors.Errorf("problem casting '%s' interface", statName)
+			return nil, errors.Errorf("problem casting '%s' interface", statName)
 		}
 		for labelName, labelValueIfc := range stats {
 			// get value
@@ -985,21 +990,21 @@ func (pbe *PromBind9Exporter) scrapeTimeStats(statMap map[string]interface{}) (e
 	timeStr = getStat(statMap, "boot-time").(string)
 	timeVal, err = time.Parse(time.RFC3339, timeStr)
 	if err != nil {
-		return pkgerrors.Wrapf(err, "problem parsing time %+s", timeStr)
+		return errors.Wrapf(err, "problem parsing time %+s", timeStr)
 	}
 	pbe.stats.BootTime = timeVal
 	// config_time_seconds
 	timeStr = getStat(statMap, "config-time").(string)
 	timeVal, err = time.Parse(time.RFC3339, timeStr)
 	if err != nil {
-		return pkgerrors.Wrapf(err, "problem parsing time %+s", timeStr)
+		return errors.Wrapf(err, "problem parsing time %+s", timeStr)
 	}
 	pbe.stats.ConfigTime = timeVal
 	// current_time_seconds
 	timeStr = getStat(statMap, "current-time").(string)
 	timeVal, err = time.Parse(time.RFC3339, timeStr)
 	if err != nil {
-		return pkgerrors.Wrapf(err, "problem parsing time %+s", timeStr)
+		return errors.Wrapf(err, "problem parsing time %+s", timeStr)
 	}
 	pbe.stats.CurrentTime = timeVal
 
@@ -1167,12 +1172,12 @@ func (pbe *PromBind9Exporter) setDaemonStats(stats map[string]any) (ret error) {
 	// incoming_queries_total
 	pbe.stats.IncomingQueries, err = pbe.scrapeServerStat(stats, "qtypes")
 	if err != nil {
-		return pkgerrors.Wrapf(err, "problem parsing 'qtypes'")
+		return errors.WithMessagef(err, "problem parsing 'qtypes'")
 	}
 	// incoming_requests_total
 	pbe.stats.IncomingRequests, err = pbe.scrapeServerStat(stats, "opcodes")
 	if err != nil {
-		return pkgerrors.Wrapf(err, "problem parsing 'opcodes'")
+		return errors.WithMessagef(err, "problem parsing 'opcodes'")
 	}
 
 	// query_duplicates_total
@@ -1183,7 +1188,7 @@ func (pbe *PromBind9Exporter) setDaemonStats(stats map[string]any) (ret error) {
 	// zone_transfer_requests_done
 	pbe.stats.NsStats, err = pbe.scrapeServerStat(stats, "nsstats")
 	if err != nil {
-		return pkgerrors.Wrapf(err, "problem parsing 'nsstats'")
+		return errors.WithMessagef(err, "problem parsing 'nsstats'")
 	}
 
 	// zone_transfer_success_total
@@ -1194,31 +1199,31 @@ func (pbe *PromBind9Exporter) setDaemonStats(stats map[string]any) (ret error) {
 	// zone_transfer_incremental_requests_ipv6
 	pbe.stats.ZoneStats, err = pbe.scrapeServerStat(stats, "zonestats")
 	if err != nil {
-		return pkgerrors.Wrapf(err, "problem parsing 'zonestats'")
+		return errors.WithMessagef(err, "problem parsing 'zonestats'")
 	}
 
 	// tasks_running
 	// worker_threads
 	pbe.stats.TaskMgr, err = pbe.scrapeServerStat(stats, "taskmgr")
 	if err != nil {
-		return pkgerrors.Wrapf(err, "problem parsing 'taskmgr'")
+		return errors.WithMessagef(err, "problem parsing 'taskmgr'")
 	}
 
 	// Parse traffic stats.
 	trafficIfc, ok := stats["traffic"]
 	if !ok {
-		return pkgerrors.Errorf("no 'traffic' in response: %+v", stats)
+		return errors.Errorf("no 'traffic' in response: %+v", stats)
 	}
 	traffic, ok := trafficIfc.(map[string]interface{})
 	if !ok {
-		return pkgerrors.Errorf("problem casting trafficIfc: %+v", trafficIfc)
+		return errors.Errorf("problem casting trafficIfc: %+v", trafficIfc)
 	}
 	trafficMap := make(map[string]PromBind9TrafficStats)
 	for trafficName, trafficStatsIfc := range traffic {
 		sizeCounts := make(map[string]float64)
 		trafficStats, ok := trafficStatsIfc.(map[string]interface{})
 		if !ok {
-			return pkgerrors.Errorf("problem casting '%s' interface", trafficName)
+			return errors.Errorf("problem casting '%s' interface", trafficName)
 		}
 		for labelName, labelValueIfc := range trafficStats {
 			// get value
@@ -1238,12 +1243,12 @@ func (pbe *PromBind9Exporter) setDaemonStats(stats map[string]any) (ret error) {
 	// Parse views.
 	viewsIfc, ok := stats["views"]
 	if !ok {
-		return pkgerrors.Errorf("no 'views' in response: %+v", stats)
+		return errors.Errorf("no 'views' in response: %+v", stats)
 	}
 
 	views := viewsIfc.(map[string]interface{})
 	if !ok {
-		return pkgerrors.Errorf("problem casting viewsIfc: %+v", viewsIfc)
+		return errors.Errorf("problem casting viewsIfc: %+v", viewsIfc)
 	}
 
 	for viewName, viewStatsIfc := range views {
@@ -1252,58 +1257,55 @@ func (pbe *PromBind9Exporter) setDaemonStats(stats map[string]any) (ret error) {
 	return nil
 }
 
-// collectStats collects stats from all bind9 apps.
-func (pbe *PromBind9Exporter) collectStats() (bind9Pid int32, lastErr error) {
+// collectStats collects stats from all bind9 daemons.
+func (pbe *PromBind9Exporter) collectStats() error {
+	var errs []error
 	pbe.up = 0
 
-	// go through all bind9 apps discovered by monitor and query them for stats
-	apps := pbe.AppMonitor.GetApps()
-APP_LOOP:
-	for _, app := range apps {
-		// ignore non-bind9 apps
-		if app.GetBaseApp().Type != AppTypeBind9 {
+	// go through all bind9 daemons discovered by monitor and query them for stats
+	daemons := pbe.Monitor.GetDaemons()
+DAEMON_LOOP:
+	for _, daemon := range daemons {
+		if daemon.GetName() != daemonname.Bind9 {
+			// ignore non-bind9 daemons
 			continue
 		}
-		bind9Pid = app.GetBaseApp().Pid
 
 		// get stats from named
-		sap, err := getAccessPoint(app, AccessPointStatistics)
-		if err != nil {
-			lastErr = err
-			log.WithError(err).Error("Problem getting stats from BIND 9, bad access statistics point")
+		sap := daemon.GetAccessPoint(AccessPointStatistics)
+		if sap == nil {
+			err := errors.Errorf("missing statistics access point for daemon: %s", daemon)
+			errs = append(errs, err)
 			continue
 		}
 		responses, stats := pbe.HTTPClient.getServerAndTrafficStats(sap.Address, sap.Port)
 		for response, err := range responses {
 			if err != nil {
-				lastErr = err
-				log.WithError(err).Error("Problem getting stats from BIND 9")
-				continue APP_LOOP
+				err = errors.WithMessagef(err, "problem getting stats from daemon: %s", daemon)
+				errs = append(errs, err)
+				continue DAEMON_LOOP
 			}
 
 			// Error HTTP response received.
 			if response.IsError() {
-				errorText := fmt.Sprintf("BIND9 stats returned error status code with message: %s", response.String())
-				lastErr = pkgerrors.New(errorText)
-				log.WithFields(log.Fields{
-					"StatusCode": response.StatusCode(),
-				}).Error(errorText)
-				continue APP_LOOP
+				err = errors.Errorf("BIND9 stats returned error status code: %d with message: %s for daemon: %s", response.StatusCode(), response.String(), daemon)
+				errs = append(errs, err)
+				continue DAEMON_LOOP
 			}
 		}
 
 		// parse response
-		err = pbe.setDaemonStats(stats)
+		err := pbe.setDaemonStats(stats)
 		if err != nil {
-			lastErr = err
-			log.WithError(err).Error("Cannot get stat from daemon")
+			err = errors.WithMessage(err, "cannot get stat from daemon")
+			errs = append(errs, err)
 			continue
 		}
 
 		pbe.up = 1
 	}
 
-	return bind9Pid, lastErr
+	return storkutil.CombineErrors("some errors were encountered while collecting stats", errs)
 }
 
 // initViewStats initializes the maps for storing metrics.
