@@ -8,6 +8,7 @@ import (
 	"iter"
 	"os"
 	"path"
+	"path/filepath"
 	"runtime"
 	"slices"
 	"strings"
@@ -17,9 +18,9 @@ import (
 	"github.com/miekg/dns"
 	"github.com/pkg/errors"
 	log "github.com/sirupsen/logrus"
-	bind9config "isc.org/stork/appcfg/bind9"
-	pdnsconfig "isc.org/stork/appcfg/pdns"
-	"isc.org/stork/appdata/bind9stats"
+	bind9config "isc.org/stork/daemoncfg/bind9"
+	pdnsconfig "isc.org/stork/daemoncfg/pdns"
+	"isc.org/stork/daemondata/bind9stats"
 	storkutil "isc.org/stork/util"
 )
 
@@ -42,7 +43,7 @@ const zoneInventoryMetaFileName = "zone-inventory.json"
 // It returns the configuration elements required by the zone inventory. For example,
 // it returns the elements required to perform zone transfers.
 type dnsConfigAccessor interface {
-	GetAXFRCredentials(viewName string, zoneName string) (address *string, keyName *string, algorithm *string, secret *string, err error)
+	GetAXFRCredentials(viewName string, zoneName string) (address string, keyName string, algorithm string, secret string, err error)
 	GetAPIKey() string
 	IsRPZ(viewName string, zoneName string) bool
 }
@@ -357,19 +358,23 @@ func (storage *zoneInventoryStorageDisk) removeMeta() error {
 }
 
 // Saves the inventory metadata.
-func (storage *zoneInventoryStorageDisk) saveMeta(meta *ZoneInventoryMeta) error {
+func (storage *zoneInventoryStorageDisk) saveMeta(meta *ZoneInventoryMeta) (err error) {
 	metaFileName := path.Join(storage.location, zoneInventoryMetaFileName)
-	metaFile, err := os.OpenFile(metaFileName, os.O_RDWR|os.O_CREATE|os.O_TRUNC, 0o640)
+	metaFile, err := os.OpenFile(metaFileName, os.O_WRONLY|os.O_CREATE|os.O_TRUNC, 0o640)
 	if err != nil {
-		return errors.Wrapf(err, "failed to save inventory metadata file %s", metaFileName)
+		err = errors.Wrapf(err, "failed to save inventory metadata file %s", metaFileName)
+		return
 	}
-	defer metaFile.Close()
+	defer func() {
+		if closeErr := metaFile.Close(); closeErr != nil && err == nil {
+			err = errors.Wrapf(closeErr, "failed to close the inventory metadata file %s", metaFileName)
+		}
+	}()
 	encoder := json.NewEncoder(metaFile)
-	err = encoder.Encode(meta)
-	if err != nil {
-		return errors.Wrapf(err, "failed to encode inventory metadata into file %s", metaFileName)
+	if err = encoder.Encode(meta); err != nil {
+		err = errors.Wrapf(err, "failed to encode inventory metadata into file %s", metaFileName)
 	}
-	return nil
+	return
 }
 
 // Reads the inventory metadata.
@@ -593,9 +598,9 @@ func (state zoneInventoryState) isErred() bool {
 // from the DNS server.
 type zoneInventoryAXFRRequest struct {
 	zoneName  string
-	keyName   *string
-	algorithm *string
-	secret    *string
+	keyName   string
+	algorithm string
+	secret    string
 	address   string
 	respChan  chan *zoneInventoryAXFRResponse
 	closeOnce sync.Once
@@ -603,7 +608,7 @@ type zoneInventoryAXFRRequest struct {
 
 // Instantiates a new AXFR request. It ensures that the zone name, key name
 // and algorithm are fully qualified, as required by the dns package.
-func newZoneInventoryAXFRRequest(zoneName string, keyName *string, algorithm *string, secret *string, address string) *zoneInventoryAXFRRequest {
+func newZoneInventoryAXFRRequest(zoneName string, keyName string, algorithm string, secret string, address string) *zoneInventoryAXFRRequest {
 	return &zoneInventoryAXFRRequest{
 		zoneName:  zoneName,
 		keyName:   keyName,
@@ -658,6 +663,32 @@ type ZoneInventoryMeta struct {
 	PopulatedAt time.Time
 }
 
+// Interface specifying operations that can be performed on the zone inventory.
+type zoneInventory interface {
+	// Returns current inventory state. It is safe for concurrent use.
+	getCurrentState() *zoneInventoryState
+	// Contacts a DNS server to fetch views and zones. Then, it processes the received
+	// data to group them into collections that are stored in memory and/or on disk.
+	// It returns a channel to which the caller can subscribe to receive a notification
+	// about completion of populating the inventory. The block parameter indicates whether
+	// or not the caller will wait for the completion notification.
+	populate(block bool) (chan zoneInventoryAsyncNotify, error)
+	// Returns the channel used to receive all zones and the corresponding
+	// views. This function is typically called in the implementation of the
+	// streaming gRPC response used to transfer the zone information to the
+	// server.
+	receiveZones(ctx context.Context, filter *bind9stats.ZoneFilter) (chan zoneInventoryReceiveZoneResult, error)
+	// Requests an AXFR (zone transfer) for the specified zone and view.
+	// It returns a channel to which the caller must subscribe to receive
+	// the AXFR results. The channel is closed by the zone inventory when
+	// the transfer is complete.
+	requestAXFR(zoneName, viewName string) (chan *zoneInventoryAXFRResponse, error)
+	// Starts the zone inventory workers.
+	start()
+	// Stops the zone inventory and waits for the background tasks to complete.
+	stop()
+}
+
 // Zone inventory.
 //
 // It coordinates fetching the zone information from the monitored DNS servers,
@@ -674,7 +705,7 @@ type ZoneInventoryMeta struct {
 // Specifically, it returns the RRs to the Stork server via the gRPC stream.
 // The zone contents may be cached in the zone inventory to avoid excessive
 // DNS requests.
-type zoneInventory struct {
+type zoneInventoryImpl struct {
 	// The storage used to save the zone information.
 	storage zoneInventoryStorage
 	// Common interface to access DNS configuration from different DNS implementations.
@@ -708,6 +739,8 @@ type zoneInventory struct {
 	axfrWorkersActive bool
 }
 
+var _ zoneInventory = (*zoneInventoryImpl)(nil)
+
 // A message sent over the channels to notify that the long lasting
 // operation has completed. If the operation failed the err field
 // contains the error.
@@ -726,10 +759,9 @@ type zoneInventoryReceiveZoneResult struct {
 // Instantiates the inventory. If the specified storage saves the zone information on
 // disk this function prepares required directory structures. An error is returned if
 // creating these structures fails.
-func newZoneInventory(storage zoneInventoryStorage, config dnsConfigAccessor, client zoneFetcher, host string, port int64) *zoneInventory {
-	ctx, cancel := context.WithCancel(context.Background())
+func newZoneInventory(storage zoneInventoryStorage, config dnsConfigAccessor, client zoneFetcher, host string, port int64) *zoneInventoryImpl {
 	state := newZoneInventoryStateInitial()
-	inventory := &zoneInventory{
+	inventory := &zoneInventoryImpl{
 		storage: storage,
 		config:  config,
 		client:  client,
@@ -742,20 +774,18 @@ func newZoneInventory(storage zoneInventoryStorage, config dnsConfigAccessor, cl
 		mutex:             sync.RWMutex{},
 		wg:                sync.WaitGroup{},
 		axfrReqChan:       make(chan *zoneInventoryAXFRRequest),
-		axfrReqCancel:     cancel,
+		axfrReqCancel:     nil, // Nothing to cancel yet.
 		axfrExecutor:      &zoneInventoryAXFRExecutorImpl{},
-		axfrPool:          storkutil.NewPausablePool(runtime.GOMAXPROCS(0) * 2),
+		axfrPool:          nil,
 		axfrWorkersActive: false,
 	}
-	// Start the workers performing AXFR requests.
-	inventory.startAXFRWorkers(ctx)
 	return inventory
 }
 
 // Transitions the inventory to a new state. It returns a zoneInventoryBusyError if the
 // current state does not permit such a transition. A call to this function must be
 // protected by mutex.
-func (inventory *zoneInventory) transitionUnsafe(newState *zoneInventoryState) error {
+func (inventory *zoneInventoryImpl) transitionUnsafe(newState *zoneInventoryState) error {
 	state := inventory.getCurrentStateUnsafe()
 	if inventory.getCurrentStateUnsafe().isLongLasting() && newState.isLongLasting() {
 		return newZoneInventoryBusyError(state, newState)
@@ -766,7 +796,7 @@ func (inventory *zoneInventory) transitionUnsafe(newState *zoneInventoryState) e
 }
 
 // Transitions the inventory to a new state. It is safe for concurrent use.
-func (inventory *zoneInventory) transition(newState *zoneInventoryState) error {
+func (inventory *zoneInventoryImpl) transition(newState *zoneInventoryState) error {
 	inventory.mutex.Lock()
 	defer inventory.mutex.Unlock()
 	return inventory.transitionUnsafe(newState)
@@ -774,12 +804,12 @@ func (inventory *zoneInventory) transition(newState *zoneInventoryState) error {
 
 // Returns current inventory state. A cal to this function must be protected
 // by a mutex.
-func (inventory *zoneInventory) getCurrentStateUnsafe() *zoneInventoryState {
+func (inventory *zoneInventoryImpl) getCurrentStateUnsafe() *zoneInventoryState {
 	return inventory.visitedStates[inventory.state]
 }
 
 // Returns current inventory state. It is safe for concurrent use.
-func (inventory *zoneInventory) getCurrentState() *zoneInventoryState {
+func (inventory *zoneInventoryImpl) getCurrentState() *zoneInventoryState {
 	inventory.mutex.RLock()
 	defer inventory.mutex.RUnlock()
 	return inventory.getCurrentStateUnsafe()
@@ -787,7 +817,7 @@ func (inventory *zoneInventory) getCurrentState() *zoneInventoryState {
 
 // Returns visited inventory state by name. It returns nil if the state
 // with this name has not been visited.
-func (inventory *zoneInventory) getVisitedState(name zoneInventoryStateName) *zoneInventoryState {
+func (inventory *zoneInventoryImpl) getVisitedState(name zoneInventoryStateName) *zoneInventoryState {
 	inventory.mutex.RLock()
 	defer inventory.mutex.RUnlock()
 	return inventory.visitedStates[name]
@@ -798,14 +828,20 @@ func (inventory *zoneInventory) getVisitedState(name zoneInventoryStateName) *zo
 // It returns a channel to which the caller can subscribe to receive a notification
 // about completion of populating the inventory. The block parameter indicates whether
 // or not the caller will wait for the completion notification.
-func (inventory *zoneInventory) populate(block bool) (chan zoneInventoryAsyncNotify, error) {
+func (inventory *zoneInventoryImpl) populate(block bool) (chan zoneInventoryAsyncNotify, error) {
 	// Start populating the zones. This transition may fail if
 	// there is another long lasting operation running.
 	if err := inventory.transition(newZoneInventoryStatePopulating()); err != nil {
 		return nil, err
 	}
 	// No zone transfers during zones population.
+	if inventory.axfrPool == nil {
+		err := errors.New("zone inventory is not started")
+		_ = inventory.transition(newZoneInventoryStatePopulatingErred(err))
+		return nil, err
+	}
 	if err := inventory.axfrPool.Pause(); err != nil {
+		_ = inventory.transition(newZoneInventoryStatePopulatingErred(err))
 		return nil, err
 	}
 
@@ -853,7 +889,7 @@ func (inventory *zoneInventory) populate(block bool) (chan zoneInventoryAsyncNot
 // to which the caller can subscribe to receive a notification about completion of
 // loading the inventory. The block parameter indicates whether or not the caller
 // will wait for the completion notification.
-func (inventory *zoneInventory) load(block bool) (chan zoneInventoryAsyncNotify, error) {
+func (inventory *zoneInventoryImpl) load(block bool) (chan zoneInventoryAsyncNotify, error) {
 	if _, ok := inventory.storage.(*zoneInventoryStorageMemory); ok {
 		// Disk storage is required because we're going to load the data
 		// from disk to memory.
@@ -903,7 +939,7 @@ func (inventory *zoneInventory) load(block bool) (chan zoneInventoryAsyncNotify,
 // views. This function is typically called in the implementation of the
 // streaming gRPC response used to transfer the zone information to the
 // server.
-func (inventory *zoneInventory) receiveZones(ctx context.Context, filter *bind9stats.ZoneFilter) (chan zoneInventoryReceiveZoneResult, error) {
+func (inventory *zoneInventoryImpl) receiveZones(ctx context.Context, filter *bind9stats.ZoneFilter) (chan zoneInventoryReceiveZoneResult, error) {
 	var err error
 	inventory.mutex.Lock()
 	state := inventory.getCurrentStateUnsafe()
@@ -978,7 +1014,7 @@ func (inventory *zoneInventory) receiveZones(ctx context.Context, filter *bind9s
 // Attempts to find zone information in the specified view. Depending on the
 // inventory storage it finds the zone information in memory or reads it from
 // disk.
-func (inventory *zoneInventory) getZoneInView(viewName, zoneName string) (*bind9stats.Zone, error) {
+func (inventory *zoneInventoryImpl) getZoneInView(viewName, zoneName string) (*bind9stats.Zone, error) {
 	state := inventory.getCurrentState()
 	if state.isInitial() || state.isErred() {
 		return nil, newZoneInventoryNotInitedError()
@@ -990,14 +1026,14 @@ func (inventory *zoneInventory) getZoneInView(viewName, zoneName string) (*bind9
 // It returns a channel to which the caller must subscribe to receive
 // the AXFR results. The channel is closed by the zone inventory when
 // the transfer is complete.
-func (inventory *zoneInventory) requestAXFR(zoneName, viewName string) (chan *zoneInventoryAXFRResponse, error) {
+func (inventory *zoneInventoryImpl) requestAXFR(zoneName, viewName string) (chan *zoneInventoryAXFRResponse, error) {
 	address, keyName, algorithm, secret, err := inventory.config.GetAXFRCredentials(viewName, zoneName)
 	if err != nil {
 		return nil, err
 	}
 	// Create and queue the request. The request is picked by one of the
 	// workers and executed.
-	request := newZoneInventoryAXFRRequest(zoneName, keyName, algorithm, secret, *address)
+	request := newZoneInventoryAXFRRequest(zoneName, keyName, algorithm, secret, address)
 	inventory.axfrReqChan <- request
 	return request.respChan, nil
 }
@@ -1005,7 +1041,7 @@ func (inventory *zoneInventory) requestAXFR(zoneName, viewName string) (chan *zo
 // Performs the zone transfer from the DNS server. The request contains
 // the response channel to which the results are sent. The caller must read
 // from the channel until it is closed.
-func (inventory *zoneInventory) runAXFR(request *zoneInventoryAXFRRequest) {
+func (inventory *zoneInventoryImpl) runAXFR(request *zoneInventoryAXFRRequest) {
 	// Close the response channel when transfer done.
 	defer request.safeClose()
 
@@ -1035,16 +1071,16 @@ func (inventory *zoneInventory) runAXFR(request *zoneInventoryAXFRRequest) {
 
 	// Set TSIG if provided. In some cases the TSIG is not used. Typically when
 	// there is only a default view.
-	if request.keyName != nil && request.secret != nil {
+	if request.keyName != "" && request.secret != "" {
 		transfer.TsigSecret = map[string]string{
-			storkutil.FullyQualifyName(*request.keyName): *request.secret,
+			storkutil.FullyQualifyName(request.keyName): request.secret,
 		}
 	}
 	message.SetAxfr(storkutil.FullyQualifyName(request.zoneName))
 
 	// Again, set TSIG if provided.
-	if request.keyName != nil && request.algorithm != nil {
-		message.SetTsig(storkutil.FullyQualifyName(*request.keyName), storkutil.FullyQualifyName(*request.algorithm), 300, time.Now().Unix())
+	if request.keyName != "" && request.algorithm != "" {
+		message.SetTsig(storkutil.FullyQualifyName(request.keyName), storkutil.FullyQualifyName(request.algorithm), 300, time.Now().Unix())
 	}
 
 	// Perform the zone transfer.
@@ -1066,7 +1102,7 @@ func (inventory *zoneInventory) runAXFR(request *zoneInventoryAXFRRequest) {
 // Starts a pool of workers for running AXFR requests. This function must be
 // called only once. It is called internally during the zone inventory
 // initialization.
-func (inventory *zoneInventory) startAXFRWorkers(ctx context.Context) {
+func (inventory *zoneInventoryImpl) startAXFRWorkers(ctx context.Context) {
 	inventory.axfrWorkersActive = true
 	go func() {
 		defer func() {
@@ -1113,27 +1149,38 @@ func (inventory *zoneInventory) startAXFRWorkers(ctx context.Context) {
 }
 
 // Stops the AXFR workers and waits for them to complete their tasks.
-func (inventory *zoneInventory) stopAXFRWorkers() {
+func (inventory *zoneInventoryImpl) stopAXFRWorkers() {
 	// Stop the pool first to ensure that no new tasks are accepted.
-	inventory.axfrPool.Stop()
-	// Cancel the context to unblock the pending tasks.
-	inventory.axfrReqCancel()
+	if inventory.axfrPool != nil {
+		inventory.axfrPool.Stop()
+		// Cancel the context to unblock the pending tasks.
+		inventory.axfrReqCancel()
+	}
 	// Set the flag to false to indicate that the workers are not active anymore.
 	inventory.axfrWorkersActive = false
 }
 
 // Returns a flag indicating if the AXFR workers are active.
-func (inventory *zoneInventory) isAXFRWorkersActive() bool {
+func (inventory *zoneInventoryImpl) isAXFRWorkersActive() bool {
 	return inventory.axfrWorkersActive
 }
 
 // This function waits for the asynchronous operations to complete.
-func (inventory *zoneInventory) awaitBackgroundTasks() {
+func (inventory *zoneInventoryImpl) awaitBackgroundTasks() {
 	inventory.wg.Wait()
 }
 
+// Starts the zone inventory and its workers.
+func (inventory *zoneInventoryImpl) start() {
+	// Start the workers performing AXFR requests.
+	inventory.axfrPool = storkutil.NewPausablePool(runtime.GOMAXPROCS(0) * 2)
+	ctx, cancel := context.WithCancel(context.Background())
+	inventory.startAXFRWorkers(ctx)
+	inventory.axfrReqCancel = cancel
+}
+
 // Stops the zone inventory and waits for the background tasks to complete.
-func (inventory *zoneInventory) stop() {
+func (inventory *zoneInventoryImpl) stop() {
 	inventory.stopAXFRWorkers()
 	inventory.awaitBackgroundTasks()
 }
@@ -1266,19 +1313,23 @@ func (vio *viewIO) recreateView(view *bind9stats.View) (err error) {
 }
 
 // Creates a zone information file in the view directory.
-func (vio *viewIO) createZone(zone *bind9stats.Zone) error {
-	zoneDataFilePath := path.Join(vio.viewLocation, getViewIOZoneFileName(zone.Name()))
-	zoneDataFile, err := os.OpenFile(zoneDataFilePath, os.O_RDWR|os.O_CREATE|os.O_TRUNC, 0o640)
+func (vio *viewIO) createZone(zone *bind9stats.Zone) (err error) {
+	zoneDataFilePath := filepath.Join(vio.viewLocation, getViewIOZoneFileName(zone.Name()))
+	zoneDataFile, err := os.OpenFile(zoneDataFilePath, os.O_WRONLY|os.O_CREATE|os.O_TRUNC, 0o640)
 	if err != nil {
-		return errors.Wrapf(err, "failed to save data for zone %s due to an error while opening the file %s", zone.Name(), zoneDataFilePath)
+		err = errors.Wrapf(err, "failed to save data for zone %s due to an error while opening the file %s", zone.Name(), zoneDataFilePath)
+		return
 	}
-	defer zoneDataFile.Close()
+	defer func() {
+		if closeErr := zoneDataFile.Close(); closeErr != nil && err == nil {
+			err = errors.Wrapf(closeErr, "failed to close the zone data file %s", zoneDataFilePath)
+		}
+	}()
 	encoder := json.NewEncoder(zoneDataFile)
-	err = encoder.Encode(zone)
-	if err != nil {
-		return errors.Wrapf(err, "failed to encode the zone data into file %s", zoneDataFilePath)
+	if err = encoder.Encode(zone); err != nil {
+		err = errors.Wrapf(err, "failed to encode the zone data into file %s", zoneDataFilePath)
 	}
-	return nil
+	return
 }
 
 // Reads the view and the corresponding zones from a file.
