@@ -17,10 +17,10 @@ import (
 	"github.com/stretchr/testify/require"
 	gomock "go.uber.org/mock/gomock"
 	"gopkg.in/h2non/gock.v1"
-	bind9config "isc.org/stork/appcfg/bind9"
-	pdnsconfig "isc.org/stork/appcfg/pdns"
-	"isc.org/stork/appdata/bind9stats"
-	pdnsdata "isc.org/stork/appdata/pdns"
+	bind9config "isc.org/stork/daemoncfg/bind9"
+	pdnsconfig "isc.org/stork/daemoncfg/pdns"
+	"isc.org/stork/daemondata/bind9stats"
+	pdnsdata "isc.org/stork/daemondata/pdns"
 	"isc.org/stork/testutil"
 	storkutil "isc.org/stork/util"
 )
@@ -139,7 +139,15 @@ func setBenchmarkGetViewsResponseOK(b *testing.B, response map[string]any) (*bin
 
 // Parse the default BIND9 config file.
 func parseDefaultBind9Config(t *testing.T) *bind9config.Config {
-	bind9Config, err := bind9config.NewParser().ParseFile("testdata/named.conf")
+	bind9Config, err := bind9config.NewParser().ParseFile("testdata/named.conf", "")
+	require.NoError(t, err)
+	require.NotNil(t, bind9Config)
+	return bind9Config
+}
+
+// Parse the default BIND9 RNDC key config file.
+func parseDefaultBind9RNDCKeyConfig(t *testing.T) *bind9config.Config {
+	bind9Config, err := bind9config.NewParser().ParseFile("testdata/rndc.key", "")
 	require.NoError(t, err)
 	require.NotNil(t, bind9Config)
 	return bind9Config
@@ -155,7 +163,7 @@ func parseDefaultPDNSConfig(t *testing.T) *pdnsconfig.Config {
 
 // Parse the default BIND9 config file for benchmark.
 func parseBenchmarkDefaultBind9Config(b *testing.B) *bind9config.Config {
-	bind9Config, err := bind9config.NewParser().ParseFile("testdata/named.conf")
+	bind9Config, err := bind9config.NewParser().ParseFile("testdata/named.conf", "")
 	if err != nil {
 		b.Fatal(err)
 	}
@@ -703,6 +711,7 @@ func TestZoneInventoryPopulateMemoryDisk(t *testing.T) {
 	require.NoError(t, err)
 	config := parseDefaultBind9Config(t)
 	inventory := newZoneInventory(storage, config, bind9StatsClient, "localhost", 5380)
+	inventory.start()
 	defer inventory.stop()
 
 	// Populate the zones into inventory.
@@ -750,6 +759,7 @@ func TestZoneInventoryPopulateMemory(t *testing.T) {
 	// Create zone inventory with memory only.
 	config := parseDefaultBind9Config(t)
 	inventory := newZoneInventory(newZoneInventoryStorageMemory(), config, bind9StatsClient, "localhost", 5380)
+	inventory.start()
 	defer inventory.stop()
 
 	// Populate the zones into inventory/memory.
@@ -790,6 +800,7 @@ func TestZoneInventoryPopulateDisk(t *testing.T) {
 	require.NoError(t, err)
 	config := parseDefaultBind9Config(t)
 	inventory := newZoneInventory(storage, config, bind9StatsClient, "localhost", 5380)
+	inventory.start()
 	defer inventory.stop()
 
 	// Populate the zones into inventory/disk.
@@ -839,6 +850,7 @@ func TestZoneInventoryPopulateLongLastingTaskConflict(t *testing.T) {
 	require.NoError(t, err)
 	config := parseDefaultBind9Config(t)
 	inventory := newZoneInventory(storage, config, bind9StatsClient, "localhost", 5380)
+	inventory.start()
 	defer inventory.stop()
 
 	// Populate the zones into inventory/disk.
@@ -899,6 +911,7 @@ func TestZoneInventoryPopulateAwaitBackgroundTasks(t *testing.T) {
 	require.NoError(t, err)
 	config := parseDefaultBind9Config(t)
 	inventory := newZoneInventory(storage, config, bind9StatsClient, "localhost", 5380)
+	inventory.start()
 
 	// Populate the zones into inventory/disk.
 	done, err := inventory.populate(true)
@@ -926,6 +939,28 @@ func TestZoneInventoryPopulateAwaitBackgroundTasks(t *testing.T) {
 	require.Equal(t, zoneInventoryStatePopulated, inventory.getCurrentState().name)
 }
 
+// Test that the error is returned if the inventory is not started.
+func TestZoneInventoryPopulateInventoryNotStarted(t *testing.T) {
+	// Arrange
+	sandbox := testutil.NewSandbox()
+	defer sandbox.Close()
+
+	bind9StatsClient, off := setGetViewsResponseOK(t, map[string]any{})
+	defer off()
+
+	config := parseDefaultBind9Config(t)
+	storage, _ := newZoneInventoryStorageMemoryDisk(sandbox.BasePath)
+	inventory := newZoneInventory(storage, config, bind9StatsClient, "localhost", 5380)
+
+	// Act
+	done, err := inventory.populate(true)
+
+	// Assert
+	require.ErrorContains(t, err, "zone inventory is not started")
+	require.Nil(t, done)
+	require.Equal(t, zoneInventoryStatePopulatingErred, inventory.getCurrentState().name)
+}
+
 // Test loading the inventory from disk to memory.
 func TestZoneInventoryLoadMemoryDisk(t *testing.T) {
 	// Setup server response.
@@ -952,6 +987,7 @@ func TestZoneInventoryLoadMemoryDisk(t *testing.T) {
 	require.NoError(t, err)
 	config := parseDefaultBind9Config(t)
 	inventory := newZoneInventory(storage, config, bind9StatsClient, "localhost", 5380)
+	inventory.start()
 	defer inventory.stop()
 
 	// Populate the zones into disk.
@@ -1011,6 +1047,7 @@ func TestZoneInventoryLoadMemory(t *testing.T) {
 	storage := newZoneInventoryStorageMemory()
 	config := parseDefaultBind9Config(t)
 	inventory := newZoneInventory(storage, config, bind9StatsClient, "localhost", 5380)
+	inventory.start()
 	defer inventory.stop()
 
 	// Populate the views/zones from DNS server to memory.
@@ -1061,6 +1098,7 @@ func TestZoneInventoryLoadDisk(t *testing.T) {
 	require.NoError(t, err)
 	config := parseDefaultBind9Config(t)
 	inventory := newZoneInventory(storage, config, bind9StatsClient, "localhost", 5380)
+	inventory.start()
 	defer inventory.stop()
 
 	// Populate the views/zones from DNS server to disk.
@@ -1129,6 +1167,8 @@ func TestZoneInventoryLoadAwaitBackgroundTasks(t *testing.T) {
 	require.NoError(t, err)
 	config := parseDefaultBind9Config(t)
 	inventory := newZoneInventory(storage, config, bind9StatsClient, "localhost", 5380)
+	inventory.start()
+	defer inventory.stop()
 
 	// Populate the zones into inventory/disk.
 	done, err := inventory.populate(false)
@@ -1189,6 +1229,7 @@ func TestGetZoneInViewDuringLongLastingTask(t *testing.T) {
 	require.NoError(t, err)
 	config := parseDefaultBind9Config(t)
 	inventory := newZoneInventory(storage, config, bind9StatsClient, "localhost", 5380)
+	inventory.start()
 	defer inventory.stop()
 
 	// Populate the views/zones from DNS server to disk.
@@ -1261,6 +1302,7 @@ func TestZoneInventoryReceiveZonesMemoryStorage(t *testing.T) {
 	require.NoError(t, err)
 	config := parseDefaultBind9Config(t)
 	inventory := newZoneInventory(storage, config, bind9StatsClient, "localhost", 5380)
+	inventory.start()
 	defer inventory.stop()
 
 	// Populate the zones from the DNS server to the inventory.
@@ -1411,6 +1453,7 @@ func TestZoneInventoryReceiveZonesDiskStorage(t *testing.T) {
 	require.NoError(t, err)
 	config := parseDefaultBind9Config(t)
 	inventory := newZoneInventory(storage, config, bind9StatsClient, "localhost", 5380)
+	inventory.start()
 	defer inventory.stop()
 
 	// Populate the zones from the DNS server to the inventory.
@@ -1560,6 +1603,7 @@ func TestZoneInventoryReceiveZonesRPZ(t *testing.T) {
 	require.NoError(t, err)
 	config := parseDefaultBind9Config(t)
 	inventory := newZoneInventory(storage, config, bind9StatsClient, "localhost", 5380)
+	inventory.start()
 	defer inventory.stop()
 
 	// Populate the zones from the DNS server to the inventory.
@@ -1615,6 +1659,7 @@ func TestZoneInventoryReceiveZonesCancel(t *testing.T) {
 	require.NoError(t, err)
 	config := parseDefaultBind9Config(t)
 	inventory := newZoneInventory(storage, config, bind9StatsClient, "localhost", 5380)
+	inventory.start()
 	defer inventory.stop()
 
 	// Populate the zones from the DNS server to the inventory.
@@ -1694,6 +1739,7 @@ func TestZoneInventoryGetZoneInViewMemory(t *testing.T) {
 	// Create zone inventory in memory.
 	config := parseDefaultBind9Config(t)
 	inventory := newZoneInventory(newZoneInventoryStorageMemory(), config, bind9StatsClient, "localhost", 5380)
+	inventory.start()
 	defer inventory.stop()
 
 	// Populate the zones from the DNS server to the inventory.
@@ -1749,6 +1795,7 @@ func TestZoneInventoryGetZoneInViewDisk(t *testing.T) {
 	require.NoError(t, err)
 	config := parseDefaultBind9Config(t)
 	inventory := newZoneInventory(storage, config, bind9StatsClient, "localhost", 5380)
+	inventory.start()
 	defer inventory.stop()
 
 	// Populate the zones from the DNS server to the inventory.
@@ -1805,6 +1852,7 @@ func TestZoneInventoryRequestAXFR(t *testing.T) {
 	storage := newZoneInventoryStorageMemory()
 	config := parseDefaultBind9Config(t)
 	inventory := newZoneInventory(storage, config, bind9StatsClient, "localhost", 5380)
+	inventory.start()
 	defer inventory.stop()
 
 	// Mock the zone transfer execution.
@@ -1889,6 +1937,7 @@ func TestZoneInventoryRequestAXFREnvelopeError(t *testing.T) {
 	storage := newZoneInventoryStorageMemory()
 	config := parseDefaultBind9Config(t)
 	inventory := newZoneInventory(storage, config, bind9StatsClient, "localhost", 5380)
+	inventory.start()
 	defer inventory.stop()
 
 	// Mock the zone transfer execution.
@@ -1960,6 +2009,7 @@ func TestZoneInventoryRequestAXFRResponseError(t *testing.T) {
 	storage := newZoneInventoryStorageMemory()
 	config := parseDefaultBind9Config(t)
 	inventory := newZoneInventory(storage, config, bind9StatsClient, "localhost", 5380)
+	inventory.start()
 	defer inventory.stop()
 
 	// Mock the zone transfer execution.
