@@ -26,3 +26,12 @@ func UnmarshalNamedStatsResponse(response string, parsed interface{}) error {
 	}
 	return nil
 }
+
+// Represents the type of a BIND 9 configuration file returned by the
+// agent when getting the raw configuration.
+type Bind9ConfigFileType string
+
+const (
+	Bind9ConfigFileTypeConfig  Bind9ConfigFileType = "config"
+	Bind9ConfigFileTypeRndcKey Bind9ConfigFileType = "rndc-key"
+)
