@@ -9,8 +9,9 @@ import (
 	"github.com/pkg/errors"
 	"github.com/stretchr/testify/require"
 	gomock "go.uber.org/mock/gomock"
-	keaconfig "isc.org/stork/appcfg/kea"
-	keactrl "isc.org/stork/appctrl/kea"
+	keaconfig "isc.org/stork/daemoncfg/kea"
+	keactrl "isc.org/stork/daemonctrl/kea"
+	"isc.org/stork/datamodel/daemonname"
 	"isc.org/stork/server/agentcomm"
 	"isc.org/stork/server/config"
 	"isc.org/stork/server/configmigrator"
@@ -71,17 +72,17 @@ func TestMigrate(t *testing.T) {
 
 	expectForwardToKeaOverHTTP := func(daemon *dbmodel.Daemon, cmds []keactrl.SerializableCommand, err mockErrors) *gomock.Call {
 		return agentMock.EXPECT().ForwardToKeaOverHTTP(
-			gomock.Any(),          // Context.
-			gomock.Eq(daemon.App), // App.
-			gomock.Eq(cmds),       // Commands.
-			gomock.Any(),          // Responses.
-		).Do(func(ctx context.Context, app *dbmodel.App, cmds []keactrl.SerializableCommand, cmdResponses ...any) {
+			gomock.Any(),      // Context.
+			gomock.Eq(daemon), // Daemon.
+			gomock.Eq(cmds),   // Commands.
+			gomock.Any(),      // Responses.
+		).Do(func(ctx context.Context, daemon *dbmodel.Daemon, cmds []keactrl.SerializableCommand, cmdResponses ...any) {
 			for i := range cmdResponses {
 				if i >= len(err.executionErrs) || err.executionErrs[i] == nil {
 					continue
 				}
 
-				r := cmdResponses[i].(*keactrl.ResponseList)
+				r := cmdResponses[i].(*keactrl.Response)
 				require.Empty(t, *r)
 
 				status := keactrl.ResponseError
@@ -89,12 +90,12 @@ func TestMigrate(t *testing.T) {
 					status = err.executionStatuses[i]
 				}
 
-				(*r) = append(*r, keactrl.Response{
+				(*r) = keactrl.Response{
 					ResponseHeader: keactrl.ResponseHeader{
 						Result: status,
 						Text:   err.executionErrs[i].Error(),
 					},
-				})
+				}
 			}
 		}).Return(&agentcomm.KeaCmdsResult{
 			Error:      err.keaErr,
@@ -200,10 +201,11 @@ func TestMigrate(t *testing.T) {
 
 	createDaemon := func() *dbmodel.Daemon {
 		daemon := &dbmodel.Daemon{
-			ID:     nextDaemonID,
-			Name:   dbmodel.DaemonNameDHCPv4,
-			App:    &dbmodel.App{ID: nextDaemonID},
-			Active: true,
+			ID:      nextDaemonID,
+			Name:    daemonname.DHCPv4,
+			Machine: &dbmodel.Machine{ID: nextDaemonID},
+			Version: "2.3.8",
+			Active:  true,
 		}
 		nextDaemonID++
 		return daemon
@@ -958,6 +960,46 @@ func TestMigrate(t *testing.T) {
 		require.EqualValues(t, getExpectedLabel(errs[0]), errs[0].Label)
 		require.Equal(t, configmigrator.ErrorCauseEntityDaemon, errs[0].CauseEntity)
 	})
+
+	t.Run("unsupported daemon version", func(t *testing.T) {
+		// Arrange
+		daemon := createDaemon()
+		daemon.Version = "1.0.0" // Unsupported version
+		migrator.items = []dbmodel.Host{createHost(daemon)}
+
+		// Act
+		errs := migrator.Migrate()
+
+		// Assert
+		require.Len(t, errs, 1)
+		require.EqualValues(t, daemon.ID, errs[0].ID)
+		require.ErrorContains(t,
+			errs[0].Error,
+			"version 1.0.0 is not supported; at least version 2.3.8 is required",
+		)
+		require.EqualValues(t, getExpectedLabel(errs[0]), errs[0].Label)
+		require.Equal(t, configmigrator.ErrorCauseEntityDaemon, errs[0].CauseEntity)
+	})
+
+	t.Run("corrupt daemon version", func(t *testing.T) {
+		// Arrange
+		daemon := createDaemon()
+		daemon.Version = "<script>alert(1);</script>"
+		migrator.items = []dbmodel.Host{createHost(daemon)}
+
+		// Act
+		errs := migrator.Migrate()
+
+		// Assert
+		require.Len(t, errs, 1)
+		require.EqualValues(t, daemon.ID, errs[0].ID)
+		require.ErrorContains(t,
+			errs[0].Error,
+			"failed to parse the daemon version",
+		)
+		require.EqualValues(t, getExpectedLabel(errs[0]), errs[0].Label)
+		require.Equal(t, configmigrator.ErrorCauseEntityDaemon, errs[0].CauseEntity)
+	})
 }
 
 // Test that the hosts are loaded and counted correctly.
