@@ -1,15 +1,13 @@
 package maintenance
 
 import (
-	"fmt"
-
 	"github.com/go-pg/pg/v10"
 	"github.com/pkg/errors"
 )
 
 // Drops a given table. It doesn't fail if the table doesn't exist.
 func DropTableIfExists(dbi pg.DBI, tableName string) error {
-	if _, err := dbi.Exec(fmt.Sprintf("DROP TABLE IF EXISTS %s", tableName)); err != nil {
+	if _, err := dbi.Exec("DROP TABLE IF EXISTS ?", pg.Ident(tableName)); err != nil {
 		return errors.Wrapf(err, `problem dropping table "%s"`, tableName)
 	}
 	return nil
@@ -17,7 +15,7 @@ func DropTableIfExists(dbi pg.DBI, tableName string) error {
 
 // Drops a given sequence. It doesn't fail if the sequence doesn't exist.
 func DropSequenceIfExists(dbi pg.DBI, tableName string) error {
-	if _, err := dbi.Exec(fmt.Sprintf("DROP SEQUENCE IF EXISTS %s", tableName)); err != nil {
+	if _, err := dbi.Exec("DROP SEQUENCE IF EXISTS ?", pg.Ident(tableName)); err != nil {
 		return errors.Wrapf(err, `problem dropping sequence "%s"`, tableName)
 	}
 	return nil
