@@ -2,6 +2,7 @@ package dbmodel
 
 import (
 	"errors"
+	"fmt"
 	"time"
 
 	"github.com/go-pg/pg/v10"
@@ -45,7 +46,6 @@ const (
 // Relations between the event and other entities.
 type Relations struct {
 	MachineID int64 `json:",omitempty"`
-	AppID     int64 `json:",omitempty"`
 	SubnetID  int64 `json:",omitempty"`
 	DaemonID  int64 `json:",omitempty"`
 	UserID    int64 `json:",omitempty"`
@@ -75,15 +75,13 @@ func AddEvent(db *pg.DB, event *Event) error {
 // limit specify the beginning of the page and the maximum size of the
 // page. Limit has to be greater then 0, otherwise error is returned.
 // The level indicates the lowest level of events that should be
-// returned (0 - info, 1 - warning, 2 - error). daemonType and appType
-// allows selecting events only from given type of app ('kea',
-// 'bind9') or daemon (e.g. 'named' or 'dhcp4'. machineID and userID
-// allows selecting events connected with indicated machine or
-// user. sortField allows indicating sort column in database and
-// sortDir allows selection the order of sorting. If sortField is
-// empty then id is used for sorting. If SortDirAny is used then ASC
-// order is used.
-func GetEventsByPage(db *pg.DB, offset int64, limit int64, level EventLevel, daemonType *string, appType *string, machineID *int64, userID *int64, sortField string, sortDir SortDirEnum) ([]Event, int64, error) {
+// returned (0 - info, 1 - warning, 2 - error). daemon
+// allows for selecting events for a given daemon (e.g. 'named' or 'dhcp4').
+// machineID and userID allows for selecting events pertaining to the specific
+// machine or user. sortField allows for selecting a sort column in database, and
+// sortDir selects the sorting order. If the sortField is empty then id is used
+// for sorting. If SortDirAny is used then ASC order is used.
+func GetEventsByPage(db *pg.DB, offset int64, limit int64, level EventLevel, daemonName *string, machineID *int64, userID *int64, sortField string, sortDir SortDirEnum) ([]Event, int64, error) {
 	if limit == 0 {
 		return nil, 0, pkgerrors.New("limit should be greater than 0")
 	}
@@ -94,13 +92,9 @@ func GetEventsByPage(db *pg.DB, offset int64, limit int64, level EventLevel, dae
 	if level > 0 {
 		q = q.Where("level >= ?", level)
 	}
-	if daemonType != nil {
+	if daemonName != nil {
 		q = q.Join("JOIN daemon ON daemon.id = CAST (relations->>'DaemonID' AS INTEGER)")
-		q = q.Where("daemon.name = ?", daemonType)
-	}
-	if appType != nil {
-		q = q.Join("JOIN app ON app.id = CAST (relations->>'AppID' AS INTEGER)")
-		q = q.Where("app.type = ?", appType)
+		q = q.Where("daemon.name = ?", *daemonName)
 	}
 	if machineID != nil {
 		q = q.Where("CAST (relations->>'MachineID' AS INTEGER) = ?", *machineID)
@@ -110,7 +104,22 @@ func GetEventsByPage(db *pg.DB, offset int64, limit int64, level EventLevel, dae
 	}
 
 	// prepare sorting expression, offset and limit
-	ordExpr := prepareOrderExpr("event", sortField, sortDir)
+	ordExpr, _ := prepareOrderAndDistinctExpr("event", sortField, sortDir, func(sortField, escapedTableName, dirExpr string) (orderExpr, distinctOnExpr string, useCustom bool) {
+		switch sortField {
+		case "level":
+			// When sorting events by level, apply the second sort by created_at field.
+			orderExpr = fmt.Sprintf("%[1]s.level %[2]s, %[1]s.created_at %[2]s", escapedTableName, dirExpr)
+			distinctOnExpr = fmt.Sprintf("%s.level", escapedTableName)
+		case "text":
+			// When sorting events by text, apply the second sort by created_at field.
+			orderExpr = fmt.Sprintf("%[1]s.text %[2]s, %[1]s.created_at %[2]s", escapedTableName, dirExpr)
+			distinctOnExpr = fmt.Sprintf("%s.text", escapedTableName)
+		default:
+			return "", "", false
+		}
+		useCustom = true
+		return
+	})
 	q = q.OrderExpr(ordExpr)
 	q = q.Offset(int(offset))
 	q = q.Limit(int(limit))
