@@ -260,8 +260,8 @@ func GetHost(dbi dbops.DBI, hostID int64) (*Host, error) {
 			return q.Order("ip_reservation.id ASC"), nil
 		}).
 		Relation("Subnet.LocalSubnets").
-		Relation("LocalHosts.Daemon.App.Machine").
-		Relation("LocalHosts.Daemon.App.AccessPoints").
+		Relation("LocalHosts.Daemon.Machine").
+		Relation("LocalHosts.Daemon.AccessPoints").
 		Where("host.id = ?", hostID).
 		Select()
 	if err != nil {
@@ -318,6 +318,9 @@ func GetHostsBySubnetID(dbi dbops.DBI, subnetID int64) ([]Host, error) {
 		Relation("HostIdentifiers", func(q *orm.Query) (*orm.Query, error) {
 			return q.Order("host_identifier.id ASC"), nil
 		}).
+		Relation("LocalHosts", func(q *orm.Query) (*orm.Query, error) {
+			return q.Order("local_host.id ASC"), nil
+		}).
 		Relation("LocalHosts.IPReservations", func(q *orm.Query) (*orm.Query, error) {
 			return q.Order("ip_reservation.id ASC"), nil
 		}).
@@ -376,10 +379,22 @@ func GetHostsByDaemonID(dbi dbops.DBI, daemonID int64, dataSource HostDataSource
 	return hosts, int64(total), err
 }
 
+// Sort field which may be used in GetHostsByPage.
+type HostSortField string
+
+// Valid sort fields.
+const (
+	HostSortFieldSubnet                HostSortField = "subnet"
+	HostSortFieldHostname              HostSortField = "hostname"
+	HostSortFieldIdentifier            HostSortField = "identifier"
+	HostSortFieldReservationAddress    HostSortField = "reservation_address"
+	HostSortFieldReservationIPv6Prefix HostSortField = "reservation_ipv6_prefix"
+)
+
 // Container for values filtering hosts fetched by page.
 //
-// The AppID, if different than 0, is used to fetch hosts whose local hosts belong to
-// the indicated app.
+// The MachineID, if different than 0, is used to fetch hosts whose local hosts belong to
+// the indicated machine.
 //
 // The optional SubnetID is used to fetch hosts belonging to the particular IPv4 or
 // IPv6 subnet. If this value is set to nil all subnets are returned. The value of 0
@@ -399,8 +414,9 @@ func GetHostsByDaemonID(dbi dbops.DBI, daemonID int64, dataSource HostDataSource
 // The DHCPDataDuplicate flag indicates whether to return hosts with
 // duplicated (the same) DHCP data in related Kea configurations.
 type HostsByPageFilters struct {
-	AppID             *int64
+	MachineID         *int64
 	SubnetID          *int64
+	DaemonID          *int64
 	LocalSubnetID     *int64
 	FilterText        *string
 	Global            *bool
@@ -424,25 +440,78 @@ func GetHostsByPage(dbi dbops.DBI, offset, limit int64, filters HostsByPageFilte
 	hosts := []Host{}
 	q := dbi.Model(&hosts)
 
-	// Prepare distinct on expression to include sort field, otherwise distinct
-	// on will fail.
-	distinctOnFields := "host.id"
-	if sortField != "" && sortField != "id" && sortField != "host.id" {
-		distinctOnFields = sortField + ", " + distinctOnFields
+	// REST API is accepting simplified sortField names. Convert it to appropriate field names accepted by DB.
+	var dbSortField string
+	switch HostSortField(sortField) {
+	case HostSortFieldSubnet:
+		dbSortField = "subnet.prefix"
+	case HostSortFieldHostname:
+		dbSortField = "distinct_lh.hostname"
+	case HostSortFieldIdentifier:
+		dbSortField = "distinct_identifier.value"
+	case HostSortFieldReservationAddress:
+		dbSortField = "distinct_reservation.address"
+	case HostSortFieldReservationIPv6Prefix:
+		dbSortField = "distinct_reservation.ipv6_prefix"
+	default:
+		dbSortField = sortField
 	}
+	// Prepare sorting expression and distinct on expression to include sort field,
+	// otherwise distinct on will fail.
+	orderExpr, distinctOnFields := prepareOrderAndDistinctExpr("host", dbSortField, sortDir, nil)
 	q = q.DistinctOn(distinctOnFields)
 
 	// Join to the local host table.
-	if (filters.AppID != nil && *filters.AppID != 0) || (filters.FilterText != nil && len(*filters.FilterText) > 0) {
+	if (filters.MachineID != nil && *filters.MachineID != 0) || (filters.DaemonID != nil && *filters.DaemonID != 0) || (filters.FilterText != nil && len(*filters.FilterText) > 0) {
 		q = q.Join("JOIN local_host").JoinOn("host.id = local_host.host_id")
 	}
 
-	// Filter by app ID.
-	// When filtering by appID we also need the local_host
-	// table as it holds the application identifier.
-	if filters.AppID != nil && *filters.AppID != 0 {
+	// Filter by machine ID.
+	// When filtering by machineID we also need the local_host
+	// table as it holds the machine identifier.
+	if filters.MachineID != nil && *filters.MachineID != 0 {
 		q = q.Join("JOIN daemon").JoinOn("local_host.daemon_id = daemon.id")
-		q = q.Where("daemon.app_id = ?", *filters.AppID)
+		q = q.Where("daemon.machine_id = ?", *filters.MachineID)
+	}
+
+	// Filter by daemon ID.
+	if filters.DaemonID != nil && *filters.DaemonID != 0 {
+		q = q.Where("local_host.daemon_id = ?", *filters.DaemonID)
+	}
+
+	// Sort by hostname.
+	if HostSortField(sortField) == HostSortFieldHostname {
+		sortSubquery := dbi.Model((*LocalHost)(nil)).Column("host_id").ColumnExpr("MIN(hostname) AS hostname").Group("host_id")
+		q = q.Join("INNER JOIN (?) AS distinct_lh", sortSubquery).JoinOn("host.id = distinct_lh.host_id")
+	}
+
+	// Sort by host identifier value.
+	if HostSortField(sortField) == HostSortFieldIdentifier {
+		sortSubquery := dbi.Model((*HostIdentifier)(nil)).Column("host_id").ColumnExpr("array_agg(value ORDER BY value) AS value").Group("host_id")
+		q = q.Join("INNER JOIN (?) AS distinct_identifier", sortSubquery).JoinOn("host.id = distinct_identifier.host_id")
+	}
+
+	// Sort by host reservation address.
+	if HostSortField(sortField) == HostSortFieldReservationAddress {
+		sortSubquery := dbi.Model((*LocalHost)(nil)).
+			Column("host_id").
+			ColumnExpr("array_agg(r.address ORDER BY r.address) AS address").
+			Join("LEFT JOIN ip_reservation AS r").JoinOn("r.local_host_id = local_host.id").
+			Where("family(r.address) = 4").
+			WhereOr("family(r.address) = 6 AND masklen(r.address) = 128").
+			Group("host_id")
+		q = q.Join("LEFT JOIN (?) AS distinct_reservation", sortSubquery).JoinOn("host.id = distinct_reservation.host_id")
+	}
+
+	// Sort by host reservation v6 prefix.
+	if HostSortField(sortField) == HostSortFieldReservationIPv6Prefix {
+		sortSubquery := dbi.Model((*LocalHost)(nil)).
+			Column("host_id").
+			ColumnExpr("array_agg(r.address ORDER BY r.address) AS ipv6_prefix").
+			Join("LEFT JOIN ip_reservation AS r").JoinOn("r.local_host_id = local_host.id").
+			Where("family(r.address) = 6 AND masklen(r.address) < 128").
+			Group("host_id")
+		q = q.Join("LEFT JOIN (?) AS distinct_reservation", sortSubquery).JoinOn("host.id = distinct_reservation.host_id")
 	}
 
 	// Filter by conflict or duplicate.
@@ -602,9 +671,8 @@ func GetHostsByPage(dbi dbops.DBI, offset, limit int64, filters HostsByPageFilte
 		Relation("LocalHosts", func(q *orm.Query) (*orm.Query, error) {
 			return q.Order("local_host.id ASC"), nil
 		}).
-		Relation("LocalHosts.Daemon.App").
-		Relation("LocalHosts.Daemon.App.Machine").
-		Relation("LocalHosts.Daemon.App.AccessPoints")
+		Relation("LocalHosts.Daemon.Machine").
+		Relation("LocalHosts.Daemon.AccessPoints")
 
 	// Only join the subnet if querying all hosts or hosts belonging to a
 	// given subnet.
@@ -612,9 +680,8 @@ func GetHostsByPage(dbi dbops.DBI, offset, limit int64, filters HostsByPageFilte
 		q = q.Relation("Subnet.LocalSubnets")
 	}
 
-	// Prepare sorting expression, offset and limit.
-	ordExpr := prepareOrderExpr("host", sortField, sortDir)
-	q = q.OrderExpr(ordExpr)
+	q = q.OrderExpr(orderExpr)
+	// Prepare offset and limit.
 	q = q.Offset(int(offset))
 	q = q.Limit(int(limit))
 
@@ -729,7 +796,7 @@ func DeleteDaemonsFromHost(dbi dbops.DBI, hostID int64, dataSource HostDataSourc
 	return int64(result.RowsAffected()), nil
 }
 
-// Deletes hosts which are not associated with any apps. Returns deleted host
+// Deletes hosts which are not associated with any daemons. Returns deleted host
 // count and an error.
 func DeleteOrphanedHosts(dbi dbops.DBI) (int64, error) {
 	subquery := dbi.Model(&[]LocalHost{}).
@@ -1209,7 +1276,7 @@ func (host Host) PopulateDaemons(dbi dbops.DBI) error {
 		if lh.DaemonID == 0 {
 			return pkgerrors.Errorf("problem with populating daemons: host %d lacks daemon ID", host.ID)
 		}
-		daemon, err := GetDaemonByID(dbi, lh.DaemonID)
+		daemon, err := GetKeaDaemonByID(dbi, lh.DaemonID)
 		if err != nil {
 			return pkgerrors.WithMessage(err, "problem with populating daemons")
 		}
