@@ -13,7 +13,7 @@ import (
 	"github.com/go-pg/pg/v10"
 	"github.com/go-pg/pg/v10/orm"
 	pkgerrors "github.com/pkg/errors"
-	keaconfig "isc.org/stork/appcfg/kea"
+	keaconfig "isc.org/stork/daemoncfg/kea"
 	dhcpmodel "isc.org/stork/datamodel/dhcp"
 	dbops "isc.org/stork/server/database"
 	storkutil "isc.org/stork/util"
@@ -62,6 +62,21 @@ const (
 	StatNameAssignedOutOfPoolPDs StatName = "assigned-out-of-pool-pds"
 )
 
+// Identifier of the relations between a subnet and other tables.
+type SubnetRelation string
+
+const (
+	SubnetRelationLocalSubnets             SubnetRelation = "LocalSubnets"
+	SubnetRelationLocalSubnetsAddressPools SubnetRelation = "LocalSubnets.AddressPools"
+	SubnetRelationLocalSubnetsPrefixPools  SubnetRelation = "LocalSubnets.PrefixPools"
+	SubnetRelationSharedNetwork            SubnetRelation = "SharedNetwork"
+	SubnetRelationLocalSharedNetworks      SubnetRelation = "SharedNetwork.LocalSharedNetworks"
+	SubnetRelationDaemons                  SubnetRelation = "LocalSubnets.Daemon"
+	SubnetRelationKeaDaemons               SubnetRelation = "LocalSubnets.Daemon.KeaDaemon"
+	SubnetRelationAccessPoints             SubnetRelation = "LocalSubnets.Daemon.AccessPoints"
+	SubnetRelationMachines                 SubnetRelation = "LocalSubnets.Daemon.Machine"
+)
+
 // Custom statistic type to redefine JSON marshalling.
 type Stats map[StatName]any
 
@@ -363,7 +378,7 @@ func (s Subnet) PopulateDaemons(dbi dbops.DBI) error {
 		if ls.DaemonID == 0 {
 			return pkgerrors.Errorf("problem with populating daemons: subnet %d lacks daemon ID", s.ID)
 		}
-		daemon, err := GetDaemonByID(dbi, ls.DaemonID)
+		daemon, err := GetKeaDaemonByID(dbi, ls.DaemonID)
 		if err != nil {
 			return pkgerrors.WithMessage(err, "problem with populating daemons")
 		}
@@ -516,16 +531,16 @@ func AddLocalSubnets(dbi dbops.DBI, subnet *Subnet) error {
 func GetSubnet(dbi dbops.DBI, subnetID int64) (*Subnet, error) {
 	subnet := &Subnet{}
 	err := dbi.Model(subnet).
-		Relation("LocalSubnets.AddressPools", func(q *orm.Query) (*orm.Query, error) {
+		Relation(string(SubnetRelationLocalSubnetsAddressPools), func(q *orm.Query) (*orm.Query, error) {
 			return q.Order("address_pool.id ASC"), nil
 		}).
-		Relation("LocalSubnets.PrefixPools", func(q *orm.Query) (*orm.Query, error) {
+		Relation(string(SubnetRelationLocalSubnetsPrefixPools), func(q *orm.Query) (*orm.Query, error) {
 			return q.Order("prefix_pool.id ASC"), nil
 		}).
-		Relation("LocalSubnets.Daemon.App.AccessPoints").
-		Relation("LocalSubnets.Daemon.App.Machine").
-		Relation("LocalSubnets.Daemon.KeaDaemon").
-		Relation("SharedNetwork.LocalSharedNetworks").
+		Relation(string(SubnetRelationAccessPoints)).
+		Relation(string(SubnetRelationMachines)).
+		Relation(string(SubnetRelationKeaDaemons)).
+		Relation(string(SubnetRelationLocalSharedNetworks)).
 		Where("subnet.id = ?", subnetID).
 		Select()
 	if err != nil {
@@ -544,14 +559,14 @@ func GetSubnetsByDaemonID(dbi dbops.DBI, daemonID int64) ([]Subnet, error) {
 
 	q := dbi.Model(&subnets).
 		Join("INNER JOIN local_subnet AS ls ON ls.subnet_id = subnet.id").
-		Relation("LocalSubnets.AddressPools", func(q *orm.Query) (*orm.Query, error) {
+		Relation(string(SubnetRelationLocalSubnetsAddressPools), func(q *orm.Query) (*orm.Query, error) {
 			return q.Order("address_pool.id ASC"), nil
 		}).
-		Relation("LocalSubnets.PrefixPools", func(q *orm.Query) (*orm.Query, error) {
+		Relation(string(SubnetRelationLocalSubnetsPrefixPools), func(q *orm.Query) (*orm.Query, error) {
 			return q.Order("prefix_pool.id ASC"), nil
 		}).
-		Relation("LocalSubnets.Daemon.App.AccessPoints").
-		Relation("SharedNetwork").
+		Relation(string(SubnetRelationAccessPoints)).
+		Relation(string(SubnetRelationSharedNetwork)).
 		Where("ls.daemon_id = ?", daemonID)
 
 	err := q.Select()
@@ -569,14 +584,14 @@ func GetSubnetsByDaemonID(dbi dbops.DBI, daemonID int64) ([]Subnet, error) {
 func GetSubnetsByPrefix(dbi dbops.DBI, prefix string) ([]Subnet, error) {
 	subnets := []Subnet{}
 	err := dbi.Model(&subnets).
-		Relation("LocalSubnets.AddressPools", func(q *orm.Query) (*orm.Query, error) {
+		Relation(string(SubnetRelationLocalSubnetsAddressPools), func(q *orm.Query) (*orm.Query, error) {
 			return q.Order("address_pool.id ASC"), nil
 		}).
-		Relation("LocalSubnets.PrefixPools", func(q *orm.Query) (*orm.Query, error) {
+		Relation(string(SubnetRelationLocalSubnetsPrefixPools), func(q *orm.Query) (*orm.Query, error) {
 			return q.Order("prefix_pool.id ASC"), nil
 		}).
-		Relation("LocalSubnets.Daemon.App.AccessPoints").
-		Relation("SharedNetwork").
+		Relation(string(SubnetRelationAccessPoints)).
+		Relation(string(SubnetRelationSharedNetwork)).
 		Where("subnet.prefix = ?", prefix).
 		Select()
 	if err != nil {
@@ -594,15 +609,15 @@ func GetSubnetsByPrefix(dbi dbops.DBI, prefix string) ([]Subnet, error) {
 func GetAllSubnets(dbi dbops.DBI, family int) ([]Subnet, error) {
 	subnets := []Subnet{}
 	q := dbi.Model(&subnets).
-		Relation("LocalSubnets.AddressPools", func(q *orm.Query) (*orm.Query, error) {
+		Relation(string(SubnetRelationLocalSubnetsAddressPools), func(q *orm.Query) (*orm.Query, error) {
 			return q.Order("address_pool.id ASC"), nil
 		}).
-		Relation("LocalSubnets.PrefixPools", func(q *orm.Query) (*orm.Query, error) {
+		Relation(string(SubnetRelationLocalSubnetsPrefixPools), func(q *orm.Query) (*orm.Query, error) {
 			return q.Order("prefix_pool.id ASC"), nil
 		}).
-		Relation("LocalSubnets.Daemon.App.AccessPoints").
-		Relation("LocalSubnets.Daemon.App.Machine").
-		Relation("SharedNetwork").
+		Relation(string(SubnetRelationAccessPoints)).
+		Relation(string(SubnetRelationMachines)).
+		Relation(string(SubnetRelationSharedNetwork)).
 		OrderExpr("id ASC")
 
 	// Let's be liberal and allow other values than 0 too. The only special
@@ -626,13 +641,13 @@ func GetAllSubnets(dbi dbops.DBI, family int) ([]Subnet, error) {
 func GetGlobalSubnets(dbi dbops.DBI, family int) ([]Subnet, error) {
 	subnets := []Subnet{}
 	q := dbi.Model(&subnets).
-		Relation("LocalSubnets.AddressPools", func(q *orm.Query) (*orm.Query, error) {
+		Relation(string(SubnetRelationLocalSubnetsAddressPools), func(q *orm.Query) (*orm.Query, error) {
 			return q.Order("address_pool.id ASC"), nil
 		}).
-		Relation("LocalSubnets.PrefixPools", func(q *orm.Query) (*orm.Query, error) {
+		Relation(string(SubnetRelationLocalSubnetsPrefixPools), func(q *orm.Query) (*orm.Query, error) {
 			return q.Order("prefix_pool.id ASC"), nil
 		}).
-		Relation("LocalSubnets.Daemon.App.AccessPoints").
+		Relation(string(SubnetRelationAccessPoints)).
 		OrderExpr("id ASC").
 		Where("subnet.shared_network_id IS NULL")
 
@@ -654,7 +669,9 @@ func GetGlobalSubnets(dbi dbops.DBI, family int) ([]Subnet, error) {
 
 // Container for values filtering subnets fetched by page.
 type SubnetsByPageFilters struct {
-	AppID         *int64
+	// TODO: Code implemented in below line is a temporary solution for virtual applications.
+	MachineID     *int64
+	DaemonID      *int64
 	LocalSubnetID *int64
 	Family        *int64
 	Text          *string
@@ -672,6 +689,50 @@ func (f *SubnetsByPageFilters) SetIPv6Family() {
 	f.Family = &family
 }
 
+// Custom handler for preparing order by and distinct on expressions for
+// sorting subnets and shared networks by statistics.
+func subnetAndSharedNetworkCustomOrderAndDistinct(sortField, escapedTableName, dirExpr string) (orderByExpr, distinctOnExpr string, useCustom bool) {
+	statsExpr := ""
+
+	switch sortField {
+	// Below cases are common for custom sorting in "subnet" and "shared_network" tables.
+	// They compute statsExpr required for sorting by IPv6 prefix delegation related fields.
+	case "total_pds":
+		statsExpr = fmt.Sprintf("(%s.stats->>'total-pds')::numeric", escapedTableName)
+	case "assigned_pds":
+		statsExpr = fmt.Sprintf("(%s.stats->>'assigned-pds')::numeric", escapedTableName)
+	case "pd_utilization":
+		statsExpr = fmt.Sprintf("%s.pd_utilization", escapedTableName)
+	default:
+		// NO-OP for other custom sort fields - statsExpr is not used.
+	}
+
+	switch sortField {
+	case "total_addresses":
+		// Sort subnets and shared networks by total addresses no matter the IP v4/v6 family.
+		distinctOnExpr = fmt.Sprintf("COALESCE(%[1]s.stats->>'total-nas', %[1]s.stats->>'total-addresses')::numeric", escapedTableName)
+		orderByExpr = distinctOnExpr
+	case "assigned_addresses":
+		// Sort subnets and shared networks by assigned addresses no matter the IP v4/v6 family.
+		distinctOnExpr = fmt.Sprintf("COALESCE(%[1]s.stats->>'assigned-nas', %[1]s.stats->>'assigned-addresses')::numeric", escapedTableName)
+		orderByExpr = distinctOnExpr
+	case "total_pds", "assigned_pds", "pd_utilization":
+		// When sorting subnets and shared networks by IPv6 prefix delegation related statistics, sort by the IP v4/v6 family first.
+		// This will handle IPv4 records depending on sorting order in a similar way to ASC NULLS FIRST/DESC NULLS LAST common sorting rule.
+		familyExpr := fmt.Sprintf("%s.inet_family", escapedTableName)
+		if strings.Contains(escapedTableName, "subnet") {
+			familyExpr = fmt.Sprintf("family(%s.prefix)", escapedTableName)
+		}
+		orderByExpr = fmt.Sprintf("%s %s, %s", familyExpr, dirExpr, statsExpr)
+		distinctOnExpr = fmt.Sprintf("%s, %s", familyExpr, statsExpr)
+	default:
+		return "", "", false
+	}
+	useCustom = true
+	orderByExpr += " " + dirExpr
+	return
+}
+
 // Fetches a collection of subnets from the database. The offset and
 // limit specify the beginning of the page and the maximum size of the
 // page. The filters object is used to filter subnets. The nil value disables
@@ -686,39 +747,74 @@ func GetSubnetsByPage(dbi dbops.DBI, offset, limit int64, filters *SubnetsByPage
 	}
 
 	subnets := []Subnet{}
-	q := dbi.Model(&subnets).Distinct()
+	q := dbi.Model(&subnets)
 
-	if filters.AppID != nil || filters.LocalSubnetID != nil || filters.Text != nil {
-		q = q.Join("INNER JOIN local_subnet AS ls ON subnet.id = ls.subnet_id")
+	// REST API is accepting simplified sortField names. Convert it to appropriate field names accepted by DB.
+	var dbSortField string
+	switch sortField {
+	case "shared_network":
+		dbSortField = "shared_network.name"
+	case "name":
+		dbSortField = "distinct_ls.name"
+	case "kea_subnet_id":
+		dbSortField = "distinct_ls.kea_subnet_id"
+	default:
+		dbSortField = sortField
 	}
-	// When filtering by appID we also need the local_subnet table as it holds the
-	// application identifier.
-	if filters.AppID != nil {
-		q = q.Join("INNER JOIN daemon AS d ON ls.daemon_id = d.id")
+
+	// prepare order by and distinct on expression to include sort field, otherwise distinct on will fail
+	orderExpr, distinctOnFields := prepareOrderAndDistinctExpr("subnet", dbSortField, sortDir, subnetAndSharedNetworkCustomOrderAndDistinct)
+	q = q.DistinctOn(distinctOnFields)
+
+	if filters.DaemonID != nil || filters.LocalSubnetID != nil || filters.Text != nil ||
+		// TODO: Code implemented in below line is a temporary solution for virtual applications.
+		filters.MachineID != nil {
+		q = q.Join("INNER JOIN local_subnet AS ls ON subnet.id = ls.subnet_id")
+		// TODO: Code implemented in below block is a temporary solution for virtual applications.
+		if filters.MachineID != nil {
+			q = q.Join("INNER JOIN daemon AS d ON ls.daemon_id = d.id")
+			q = q.Where("d.machine_id = ?", *filters.MachineID)
+		}
 	}
 	// Pools are also required when trying to filter by text.
 	if filters.Text != nil {
 		q = q.Join("LEFT JOIN address_pool AS ap ON ls.id = ap.local_subnet_id")
 	}
+	// Sort by subnet name.
+	if sortField == "name" {
+		sortSubquery := dbi.Model((*LocalSubnet)(nil)).
+			Column("subnet_id").
+			ColumnExpr("array_agg(user_context->'subnet-name' ORDER BY user_context->'subnet-name') AS name").
+			Where("user_context->'subnet-name' IS NOT NULL").
+			Group("subnet_id")
+		q = q.Join("LEFT JOIN (?) AS distinct_ls", sortSubquery).JoinOn("subnet.id = distinct_ls.subnet_id")
+	}
+	// Sort by Kea subnet id.
+	if sortField == "kea_subnet_id" {
+		sortSubquery := dbi.Model((*LocalSubnet)(nil)).Column("subnet_id").ColumnExpr("MIN(local_subnet_id) AS kea_subnet_id").Group("subnet_id")
+		q = q.Join("INNER JOIN (?) AS distinct_ls", sortSubquery).JoinOn("subnet.id = distinct_ls.subnet_id")
+	}
 	// Include pools, shared network the subnets belong to, local subnet info
-	// and the associated apps in the results.
-	q = q.Relation("SharedNetwork").
-		Relation("LocalSubnets.AddressPools", func(q *orm.Query) (*orm.Query, error) {
+	// and the associated daemons in the results.
+	q = q.Relation(string(SubnetRelationSharedNetwork)).
+		Relation(string(SubnetRelationLocalSubnetsAddressPools), func(q *orm.Query) (*orm.Query, error) {
 			return q.Order("address_pool.id ASC"), nil
 		}).
-		Relation("LocalSubnets.PrefixPools", func(q *orm.Query) (*orm.Query, error) {
+		Relation(string(SubnetRelationLocalSubnetsPrefixPools), func(q *orm.Query) (*orm.Query, error) {
 			return q.Order("prefix_pool.id ASC"), nil
 		}).
-		Relation("LocalSubnets.Daemon.App.Machine")
+		Relation(string(SubnetRelationAccessPoints)).
+		// TODO: Code implemented in below line is a temporary solution for virtual applications.
+		Relation(string(SubnetRelationMachines))
 
 	// Applicable family values are 4 and 6.
 	if filters.Family != nil {
 		q = q.Where("family(subnet.prefix) = ?", *filters.Family)
 	}
 
-	// Filter by appID.
-	if filters.AppID != nil {
-		q = q.Where("d.app_id = ?", *filters.AppID)
+	// Filter by daemonID.
+	if filters.DaemonID != nil {
+		q = q.Where("ls.daemon_id = ?", *filters.DaemonID)
 	}
 
 	// Filter by local subnet ID.
@@ -740,9 +836,8 @@ func GetSubnetsByPage(dbi dbops.DBI, offset, limit int64, filters *SubnetsByPage
 		})
 	}
 
-	// prepare sorting expression, offset and limit
-	ordExpr := prepareOrderExpr("subnet", sortField, sortDir)
-	q = q.OrderExpr(ordExpr)
+	// append sorting expression, offset and limit
+	q = q.OrderExpr(orderExpr)
 	q = q.Offset(int(offset))
 	q = q.Limit(int(limit))
 
@@ -763,9 +858,9 @@ func GetSubnetsWithLocalSubnets(dbi dbops.DBI) ([]*Subnet, error) {
 	q := dbi.Model(&subnets)
 	// only selected columns are returned for performance reasons
 	q = q.Column("id", "shared_network_id", "prefix")
-	q = q.Relation("LocalSubnets")
-	q = q.Relation("LocalSubnets.AddressPools")
-	q = q.Relation("LocalSubnets.PrefixPools")
+	q = q.Relation(string(SubnetRelationLocalSubnets))
+	q = q.Relation(string(SubnetRelationLocalSubnetsAddressPools))
+	q = q.Relation(string(SubnetRelationLocalSubnetsPrefixPools))
 	q = q.Order("shared_network_id ASC")
 
 	err := q.Select()
@@ -872,17 +967,6 @@ func DeleteDaemonFromSubnets(dbi dbops.DBI, daemonID int64) (int64, error) {
 	return int64(result.RowsAffected()), nil
 }
 
-// Finds and returns an app associated with a subnet having the specified id.
-func (s *Subnet) GetApp(appID int64) *App {
-	for _, s := range s.LocalSubnets {
-		daemon := s.Daemon
-		if daemon.App != nil && daemon.App.ID == appID {
-			return daemon.App
-		}
-	}
-	return nil
-}
-
 // Iterates over the provided slice of subnets and stores them in the database
 // if they are not there yet. In addition, it associates the subnets with the
 // specified Kea application. Returns a list of added subnets.
@@ -987,8 +1071,8 @@ func CommitNetworksIntoDB(dbi dbops.DBI, networks []SharedNetwork, subnets []Sub
 	return
 }
 
-// Fetch all local subnets for indicated app.
-func GetAppLocalSubnets(dbi dbops.DBI, appID int64) ([]*LocalSubnet, error) {
+// Fetch all local subnets for indicated daemon.
+func GetDaemonLocalSubnets(dbi dbops.DBI, daemonID int64) ([]*LocalSubnet, error) {
 	subnets := []*LocalSubnet{}
 	q := dbi.Model(&subnets)
 	q = q.Join("INNER JOIN daemon AS d ON local_subnet.daemon_id = d.id")
@@ -997,15 +1081,14 @@ func GetAppLocalSubnets(dbi dbops.DBI, appID int64) ([]*LocalSubnet, error) {
 	q = q.Relation("Subnet")
 	q = q.Relation("AddressPools")
 	q = q.Relation("PrefixPools")
-	q = q.Relation("Daemon.App")
-	q = q.Where("d.app_id = ?", appID)
+	q = q.Where("d.id = ?", daemonID)
 
 	err := q.Select()
 	if err != nil {
 		if errors.Is(err, pg.ErrNoRows) {
 			return nil, nil
 		}
-		err = pkgerrors.Wrapf(err, "problem getting all local subnets for app %d", appID)
+		err = pkgerrors.Wrapf(err, "problem getting all local subnets for daemon %d", daemonID)
 		return nil, err
 	}
 	return subnets, nil
@@ -1057,7 +1140,7 @@ func (s *Subnet) UpdateStatistics(dbi dbops.DBI, statistics utilizationStats) er
 	return err
 }
 
-// Deletes subnets which are not associated with any apps. Returns deleted subnet
+// Deletes subnets which are not associated with any daemons. Returns deleted subnet
 // count and an error.
 func DeleteOrphanedSubnets(dbi dbops.DBI) (int64, error) {
 	subquery := dbi.Model(&[]LocalSubnet{}).
