@@ -13,7 +13,9 @@ import (
 	"github.com/go-pg/pg/v10"
 	"github.com/pkg/errors"
 	log "github.com/sirupsen/logrus"
-	"isc.org/stork/appcfg/dnsconfig"
+	agentapi "isc.org/stork/api"
+	bind9config "isc.org/stork/daemoncfg/bind9"
+	"isc.org/stork/daemoncfg/dnsconfig"
 	agentcomm "isc.org/stork/server/agentcomm"
 	dbmodel "isc.org/stork/server/database/model"
 	storkutil "isc.org/stork/util"
@@ -52,6 +54,20 @@ func (error *ManagerRRsAlreadyRequestedError) Error() string {
 	return fmt.Sprintf("zone transfer for view %s, zone %s has been already requested by another user", error.viewName, error.zoneName)
 }
 
+// An error returned upon concurrent attempts to transfer the same zone
+// for the same view and daemon.
+type ManagerBind9FormattedConfigAlreadyRequestedError struct{}
+
+// Instantiates a new ManagerRRsAlreadyRequestedError.
+func NewManagerBind9FormattedConfigAlreadyRequestedError() *ManagerBind9FormattedConfigAlreadyRequestedError {
+	return &ManagerBind9FormattedConfigAlreadyRequestedError{}
+}
+
+// Returns the error as text.
+func (error *ManagerBind9FormattedConfigAlreadyRequestedError) Error() string {
+	return "BIND 9 configuration for the specified daemon has been already requested by another user"
+}
+
 // This interface must be implemented by the instance owning the Manager.
 // The server.StorkServer instance implements this interface because it
 // owns the instance of the Manager. The Manager uses it to get access to
@@ -83,10 +99,20 @@ type Manager interface {
 	FetchZones(poolSize, batchSize int, block bool) (chan ManagerDoneNotify, error)
 	// Checks if the DNS Manager is currently fetching the zones and returns progress.
 	// The first boolean flag indicates whether or not fetch is in progress. The int
-	// parameters indicate the number of apps from which the zones are fetched and the
-	// number of apps from which the zones have been fetched already.
+	// parameters indicate the number of daemons from which the zones are fetched and
+	// the number of daemons from which the zones have been fetched already.
 	GetFetchZonesProgress() (bool, int, int)
+	// Returns the RRs for the specified zone, daemon and view name.
+	// Using the options, the caller can request to force zone transfer even when RRs
+	// are cached in the database.
 	GetZoneRRs(zoneID int64, daemonID int64, viewName string, options ...GetZoneRRsOption) iter.Seq[*RRResponse]
+	// Returns the BIND 9 configuration for the specified daemon with filtering
+	// and file type selection. If the filter is nil, all configuration elements are
+	// returned. Otherwise,only the configuration elements explicitly enabled in the
+	// filter are returned. Similarly, if the file selector is nil, all configuration
+	// files are returned. Otherwise, only the configuration files explicitly enabled
+	// in the file selector are returned.
+	GetBind9FormattedConfig(ctx context.Context, daemonID int64, fileSelector *bind9config.FileTypeSelector, filter *bind9config.Filter) iter.Seq[*Bind9FormattedConfigResponse]
 	Shutdown()
 }
 
@@ -101,10 +127,10 @@ const (
 // A zones fetching state including the flag whether or not the fetch
 // is in progress, and other data useful to track fetch progress.
 type fetchingState struct {
-	fetching           bool
-	completedAppsCount int
-	appsCount          int
-	mutex              sync.RWMutex
+	fetching              bool
+	completedDaemonsCount int
+	daemonsCount          int
+	mutex                 sync.RWMutex
 }
 
 // Attempts to start fetching the zones. If we're already in the fetching state
@@ -117,8 +143,8 @@ func (state *fetchingState) startFetching() bool {
 		return false
 	}
 	state.fetching = true
-	state.appsCount = 0
-	state.completedAppsCount = 0
+	state.daemonsCount = 0
+	state.completedDaemonsCount = 0
 	return true
 }
 
@@ -130,33 +156,31 @@ func (state *fetchingState) stopFetching() {
 }
 
 // Returns the underlying state values: a flag indicating whether or not
-// fetch is in progress, the number of apps from which the zones are being
-// fetched, and the number of apps from which the zones have been fetched
+// fetch is in progress, the number of daemons from which the zones are being
+// fetched, and the number of daemons from which the zones have been fetched
 // already.
 func (state *fetchingState) getFetchZonesProgress() (bool, int, int) {
 	state.mutex.RLock()
 	defer state.mutex.RUnlock()
-	return state.fetching, state.appsCount, state.completedAppsCount
+	return state.fetching, state.daemonsCount, state.completedDaemonsCount
 }
 
-// Sets the total number of apps from which the zones are to be fetched.
-func (state *fetchingState) setAppsCount(appsCount int) {
+// Sets the total number of daemons from which the zones are to be fetched.
+func (state *fetchingState) setDaemonsCount(daemonsCount int) {
 	state.mutex.Lock()
 	defer state.mutex.Unlock()
-	state.appsCount = appsCount
+	state.daemonsCount = daemonsCount
 }
 
-// Increases a counter of apps from which the zones have been fetched.
-func (state *fetchingState) increaseCompletedAppsCount() {
+// Increases a counter of daemons from which the zones have been fetched.
+func (state *fetchingState) increaseCompletedDaemonsCount() {
 	state.mutex.Lock()
 	defer state.mutex.Unlock()
-	state.completedAppsCount += 1
+	state.completedDaemonsCount += 1
 }
 
 // A state of requests sent to the agents to fetch RRs.
 type rrsRequestingState struct {
-	pool        *storkutil.PausablePool
-	cancel      context.CancelFunc
 	requestChan chan *rrsRequest
 	requests    map[uint64]bool
 	mutex       sync.Mutex
@@ -165,7 +189,7 @@ type rrsRequestingState struct {
 // A structure holding a request to fetch RRs for a zone.
 type rrsRequest struct {
 	ctx       context.Context
-	app       *dbmodel.App
+	daemon    *dbmodel.Daemon
 	zoneName  string
 	viewName  string
 	respChan  chan *RRResponse
@@ -174,10 +198,10 @@ type rrsRequest struct {
 }
 
 // Instantiates a new RRs request.
-func newRRsRequest(ctx context.Context, key uint64, app *dbmodel.App, zoneName string, viewName string) *rrsRequest {
+func newRRsRequest(ctx context.Context, key uint64, daemon *dbmodel.Daemon, zoneName string, viewName string) *rrsRequest {
 	return &rrsRequest{
 		ctx:       ctx,
-		app:       app,
+		daemon:    daemon,
 		zoneName:  zoneName,
 		viewName:  viewName,
 		respChan:  make(chan *RRResponse),
@@ -221,6 +245,77 @@ func NewCacheRRResponse(rrs []*dnsconfig.RR, transferAt time.Time) *RRResponse {
 	}
 }
 
+// A state of requests sent to the agents to receive BIND 9 configuration.
+type bind9FormattedConfigRequestingState struct {
+	requestChan chan *bind9FormattedConfigRequest
+	requests    map[int64]bool
+	mutex       sync.Mutex
+}
+
+// A structure holding a request to receive BIND 9 configuration from the agent.
+type bind9FormattedConfigRequest struct {
+	ctx          context.Context
+	daemon       *dbmodel.Daemon
+	fileSelector *bind9config.FileTypeSelector
+	filter       *bind9config.Filter
+	respChan     chan *Bind9FormattedConfigResponse
+	closeOnce    sync.Once
+	daemonID     int64
+}
+
+// Instantiates a new BIND 9 configuration request.
+func newBind9FormattedConfigRequest(ctx context.Context, daemonID int64, daemon *dbmodel.Daemon, fileSelector *bind9config.FileTypeSelector, filter *bind9config.Filter) *bind9FormattedConfigRequest {
+	return &bind9FormattedConfigRequest{
+		ctx:          ctx,
+		daemon:       daemon,
+		fileSelector: fileSelector,
+		filter:       filter,
+		respChan:     make(chan *Bind9FormattedConfigResponse),
+		closeOnce:    sync.Once{},
+		daemonID:     daemonID,
+	}
+}
+
+// A structure representing a chunk of the BIND 9 configuration received
+// over the stream.
+type Bind9FormattedConfigResponse struct {
+	// The BIND 9 configuration file preamble.
+	File *agentapi.ReceiveBind9ConfigFile
+	// The BIND 9 configuration file contents chunk.
+	Contents *string
+	// The error returned when receiving the BIND 9 configuration.
+	Err error
+}
+
+// Creates a new Bind9FormattedConfigResponse representing a BIND 9 configuration
+// file preamble.
+func NewBind9FormattedConfigResponseFile(file *agentapi.ReceiveBind9ConfigFile) *Bind9FormattedConfigResponse {
+	return &Bind9FormattedConfigResponse{
+		File:     file,
+		Contents: nil,
+		Err:      nil,
+	}
+}
+
+// Creates a new Bind9FormattedConfigResponse representing a BIND 9 configuration
+// contents chunk.
+func NewBind9FormattedConfigResponseContents(contents *string) *Bind9FormattedConfigResponse {
+	return &Bind9FormattedConfigResponse{
+		File:     nil,
+		Contents: contents,
+		Err:      nil,
+	}
+}
+
+// Creates a new Bind9FormattedConfigResponse representing an error.
+func NewBind9FormattedConfigResponseError(err error) *Bind9FormattedConfigResponse {
+	return &Bind9FormattedConfigResponse{
+		File:     nil,
+		Contents: nil,
+		Err:      err,
+	}
+}
+
 // DNS Manager implementation. The Manager is responsible for coordinating all
 // operations pertaining to DNS in Stork server.
 type managerImpl struct {
@@ -232,6 +327,12 @@ type managerImpl struct {
 	fetchingState *fetchingState
 	// A state of RRs requests.
 	rrsReqsState *rrsRequestingState
+	// A state of BIND 9 configuration requests.
+	bind9FormattedConfigReqsState *bind9FormattedConfigRequestingState
+	// A pool of workers for RRs requests and BIND 9 configuration requests.
+	pool *storkutil.PausablePool
+	// A cancel function for the pool.
+	cancel context.CancelFunc
 }
 
 // A structure returned over the channel when Manager completes asynchronous task.
@@ -251,10 +352,14 @@ func NewManager(owner ManagerAccessors) (Manager, error) {
 		rrsReqsState: &rrsRequestingState{
 			requestChan: make(chan *rrsRequest),
 			requests:    make(map[uint64]bool),
-			cancel:      cancel,
 		},
+		bind9FormattedConfigReqsState: &bind9FormattedConfigRequestingState{
+			requestChan: make(chan *bind9FormattedConfigRequest),
+			requests:    make(map[int64]bool),
+		},
+		cancel: cancel,
 	}
-	impl.startRRsRequestWorkers(ctx)
+	impl.startAsyncRequestWorkers(ctx)
 	return impl, nil
 }
 
@@ -268,12 +373,12 @@ func (manager *managerImpl) FetchZones(poolSize, batchSize int, block bool) (cha
 	// Get the list of monitored DNS servers. We're going to communicate
 	// with the zone inventories created for these servers to fetch the
 	// list of zones.
-	apps, err := dbmodel.GetAppsByType(manager.db, dbmodel.AppTypeBind9, dbmodel.AppTypePDNS)
+	daemons, err := dbmodel.GetDNSDaemons(manager.db)
 	if err != nil {
 		manager.fetchingState.stopFetching()
 		return nil, err
 	}
-	manager.fetchingState.setAppsCount(len(apps))
+	manager.fetchingState.setDaemonsCount(len(daemons))
 	// Use the channel to communicate when the fetch has finished.
 	// By default the channel is non-blocking in case the caller doesn't
 	// want to wait for the completion. The buffer length of 1 ensures
@@ -292,27 +397,27 @@ func (manager *managerImpl) FetchZones(poolSize, batchSize int, block bool) (cha
 		// communicates with one machine. Concurrently fetching from
 		// multiple machines should significantly decrease the time to
 		// populate all zones.
-		appsChan := make(chan dbmodel.App, len(apps))
-		for _, app := range apps {
-			appsChan <- app
+		daemonsChan := make(chan dbmodel.Daemon, len(daemons))
+		for _, daemon := range daemons {
+			daemonsChan <- daemon
 		}
-		close(appsChan)
+		close(daemonsChan)
 		// Wait group is used to ensure we wait for zone fetch from all DNS servers.
 		wg := sync.WaitGroup{}
-		wg.Add(len(apps))
+		wg.Add(len(daemons))
 		// Mutex protects the results map from concurrent write. The results hold
 		// a map of errors for respective daemons.
 		mutex := sync.Mutex{}
 		results := make(map[int64]*dbmodel.ZoneInventoryStateDetails)
 		// Create worker goroutines. The goroutines will read from the common
-		// channel and fetch the zones from the apps listed in the channel.
+		// channel and fetch the zones from the daemons listed in the channel.
 		for i := 0; i < poolSize; i++ {
-			go func(appsChan <-chan dbmodel.App) {
-				// Read next app from the channel.
-				for app := range appsChan {
-					manager.fetchZonesFromDNSServer(&app, batchSize, &wg, &mutex, results)
+			go func(daemonsChan <-chan dbmodel.Daemon) {
+				// Read next daemon from the channel.
+				for daemon := range daemonsChan {
+					manager.fetchZonesFromDNSServer(&daemon, batchSize, &wg, &mutex, results)
 				}
-			}(appsChan)
+			}(daemonsChan)
 		}
 		// Wait for all the go-routines to complete.
 		wg.Wait()
@@ -330,8 +435,8 @@ func (manager *managerImpl) FetchZones(poolSize, batchSize int, block bool) (cha
 			}
 		}
 		log.WithFields(log.Fields{
-			"appCount":  len(results),
-			"zoneCount": zoneCount,
+			"daemonCount": len(results),
+			"zoneCount":   zoneCount,
 		}).Info("Completed fetching the zones from the agents")
 
 		// Return the result.
@@ -343,14 +448,14 @@ func (manager *managerImpl) FetchZones(poolSize, batchSize int, block bool) (cha
 	return notifyChannel, nil
 }
 
-// Contacts a specified DNS server and fetches zones from it. The app
+// Contacts a specified DNS server and fetches zones from it. The daemon
 // indicates the DNS server to contact. The batchSize parameter controls
 // the size of the batch of zones to be inserted into the database in a single
 // SQL INSERT. The wg and mutex parameters are used to synchronize the work
 // between multiple goroutines. The results parameter is a map of errors for
 // respective daemons. This function can merely be called from the
 // FetchZones function.
-func (manager *managerImpl) fetchZonesFromDNSServer(app *dbmodel.App, batchSize int, wg *sync.WaitGroup, mutex *sync.Mutex, results map[int64]*dbmodel.ZoneInventoryStateDetails) {
+func (manager *managerImpl) fetchZonesFromDNSServer(daemon *dbmodel.Daemon, batchSize int, wg *sync.WaitGroup, mutex *sync.Mutex, results map[int64]*dbmodel.ZoneInventoryStateDetails) {
 	defer wg.Done()
 	var (
 		// Track views. We need to flush the batch when the view changes.
@@ -366,7 +471,7 @@ func (manager *managerImpl) fetchZonesFromDNSServer(app *dbmodel.App, batchSize
 	// performance for large number of zones.
 	batch := dbmodel.NewBatch(manager.db, batchSize, dbmodel.AddZones)
 	state := dbmodel.NewZoneInventoryStateDetails()
-	for zone, err := range manager.agents.ReceiveZones(context.Background(), app, nil) {
+	for zone, err := range manager.agents.ReceiveZones(context.Background(), daemon, nil) {
 		if err != nil {
 			// Returned status depends on the returned error type. Some
 			// errors require special handling.
@@ -392,7 +497,7 @@ func (manager *managerImpl) fetchZonesFromDNSServer(app *dbmodel.App, batchSize
 		if isFirst {
 			// Delete the local zones.
 			isFirst = false
-			err = dbmodel.DeleteLocalZones(manager.db, app.Daemons[0].ID)
+			err = dbmodel.DeleteLocalZones(manager.db, daemon.ID)
 			if err != nil {
 				state.SetStatus(dbmodel.ZoneInventoryStatusErred, err)
 				break
@@ -404,7 +509,7 @@ func (manager *managerImpl) fetchZonesFromDNSServer(app *dbmodel.App, batchSize
 			Name: zone.Name(),
 			LocalZones: []*dbmodel.LocalZone{
 				{
-					DaemonID: app.Daemons[0].ID,
+					DaemonID: daemon.ID,
 					View:     zone.ViewName,
 					Class:    zone.Class,
 					Serial:   zone.Serial,
@@ -438,7 +543,7 @@ func (manager *managerImpl) fetchZonesFromDNSServer(app *dbmodel.App, batchSize
 			state.SetStatus(dbmodel.ZoneInventoryStatusErred, err)
 		}
 	}
-	zoneCountStats, err := dbmodel.GetZoneCountStatsByDaemon(manager.db, app.Daemons[0].ID)
+	zoneCountStats, err := dbmodel.GetZoneCountStatsByDaemon(manager.db, daemon.ID)
 	if err != nil {
 		state.SetStatus(dbmodel.ZoneInventoryStatusErred, err)
 	} else {
@@ -446,19 +551,19 @@ func (manager *managerImpl) fetchZonesFromDNSServer(app *dbmodel.App, batchSize
 		state.SetBuiltinZoneCount(zoneCountStats.BuiltinZones)
 	}
 	// Add the zone inventory state into the database for this DNS server.
-	if err := dbmodel.AddZoneInventoryState(manager.db, dbmodel.NewZoneInventoryState(app.Daemons[0].ID, state)); err != nil {
+	if err := dbmodel.AddZoneInventoryState(manager.db, dbmodel.NewZoneInventoryState(daemon.ID, state)); err != nil {
 		// This is an exceptional situation and normally shouldn't happen.
 		// Let's communicate this issue to the caller and log it. The
 		// zone inventory state won't be available for this server.
 		state.SetStatus(dbmodel.ZoneInventoryStatusErred, err)
 		log.WithFields(log.Fields{
-			"app": app.Name,
+			"daemon": daemon.Name,
 		}).WithError(err).Error("Failed to save the zone inventory status in the database")
 	}
 	// Store the inventory state in the common map, so it can be returned
 	// to a caller.
-	storeResult(mutex, results, app.Daemons[0].ID, state)
-	manager.fetchingState.increaseCompletedAppsCount()
+	storeResult(mutex, results, daemon.ID, state)
+	manager.fetchingState.increaseCompletedDaemonsCount()
 }
 
 // Checks if the DNS Manager is currently fetching the zones.
@@ -475,7 +580,7 @@ func (manager *managerImpl) runRRsRequest(request *rrsRequest) {
 		defer manager.rrsReqsState.mutex.Unlock()
 		delete(manager.rrsReqsState.requests, request.key)
 	}()
-	for rr, err := range manager.agents.ReceiveZoneRRs(context.Background(), request.app, request.zoneName, request.viewName) {
+	for rr, err := range manager.agents.ReceiveZoneRRs(context.Background(), request.daemon, request.zoneName, request.viewName) {
 		var response *RRResponse
 		if err != nil {
 			response = NewErrorRRResponse(err)
@@ -496,7 +601,7 @@ func (manager *managerImpl) runRRsRequest(request *rrsRequest) {
 // over the response channel. The specified key should uniquely identify a
 // zone, view and daemon for which the RRs are requested. If there is an
 // ongoing request for the same key, the function returns an error.
-func (manager *managerImpl) requestZoneRRs(ctx context.Context, key uint64, app *dbmodel.App, zoneName string, viewName string) (chan *RRResponse, error) {
+func (manager *managerImpl) requestZoneRRs(ctx context.Context, key uint64, daemon *dbmodel.Daemon, zoneName string, viewName string) (chan *RRResponse, error) {
 	// Try to mark the request as ongoing. If the request is already present
 	// under the same key, return an error.
 	manager.rrsReqsState.mutex.Lock()
@@ -508,17 +613,79 @@ func (manager *managerImpl) requestZoneRRs(ctx context.Context, key uint64, app
 	manager.rrsReqsState.mutex.Unlock()
 
 	// Create a new request and send it to the channel.
-	request := newRRsRequest(ctx, key, app, zoneName, viewName)
+	request := newRRsRequest(ctx, key, daemon, zoneName, viewName)
 	manager.rrsReqsState.requestChan <- request
 	// Return the response channel, so the caller can receive the RRs.
 	return request.respChan, nil
 }
 
-// Starts a pool of workers that fetch RRs for the requested zones from the
-// agents' zone inventories.
-func (manager *managerImpl) startRRsRequestWorkers(ctx context.Context) {
+// Requests BIND 9 configuration from the agent and returns it over the
+// response channel.
+func (manager *managerImpl) runBind9FormattedConfigRequest(request *bind9FormattedConfigRequest) {
+	defer func() {
+		close(request.respChan)
+		manager.bind9FormattedConfigReqsState.mutex.Lock()
+		defer manager.bind9FormattedConfigReqsState.mutex.Unlock()
+		delete(manager.bind9FormattedConfigReqsState.requests, request.daemonID)
+	}()
+	for rsp, err := range manager.agents.ReceiveBind9FormattedConfig(request.ctx, request.daemon, request.fileSelector, request.filter) {
+		var response *Bind9FormattedConfigResponse
+		switch {
+		case err != nil:
+			response = NewBind9FormattedConfigResponseError(err)
+		case rsp == nil:
+			response = NewBind9FormattedConfigResponseError(errors.Errorf("unexpected nil response while getting BIND 9 configuration from the agent"))
+		default:
+			// Non-error response.
+			switch r := rsp.Response.(type) {
+			case *agentapi.ReceiveBind9ConfigRsp_File:
+				// A BIND 9 configuration file preamble.
+				response = NewBind9FormattedConfigResponseFile(r.File)
+			case *agentapi.ReceiveBind9ConfigRsp_Line:
+				// The BIND 9 configuration file contents.
+				response = NewBind9FormattedConfigResponseContents(&r.Line)
+			default:
+				// Unexpected response type.
+				response = NewBind9FormattedConfigResponseError(errors.Errorf("unexpected response type: %T", rsp.Response))
+			}
+		}
+		select {
+		case <-request.ctx.Done():
+			// The client stopped reading the BIND 9 configuration.
+			return
+		case request.respChan <- response:
+			// Get more requests.
+		}
+	}
+}
+
+// Requests BIND 9 configuration from the agent and returns it over the response channel.
+// This function prevents concurrent requests for the same daemon. If another
+// request for the same daemon is in progress, the function returns an error.
+func (manager *managerImpl) requestBind9FormattedConfig(ctx context.Context, daemonID int64, daemon *dbmodel.Daemon, fileSelector *bind9config.FileTypeSelector, filter *bind9config.Filter) (chan *Bind9FormattedConfigResponse, error) {
+	// Try to mark the request as ongoing. If the request is already present
+	// for the same daemon, return an error.
+	manager.bind9FormattedConfigReqsState.mutex.Lock()
+	if _, ok := manager.bind9FormattedConfigReqsState.requests[daemonID]; ok {
+		manager.bind9FormattedConfigReqsState.mutex.Unlock()
+		return nil, NewManagerBind9FormattedConfigAlreadyRequestedError()
+	}
+	manager.bind9FormattedConfigReqsState.requests[daemonID] = true
+	manager.bind9FormattedConfigReqsState.mutex.Unlock()
+
+	// Create a new request and send it to the channel.
+	request := newBind9FormattedConfigRequest(ctx, daemonID, daemon, fileSelector, filter)
+	manager.bind9FormattedConfigReqsState.requestChan <- request
+	// Return the response channel, so the caller can receive the BIND 9 configuration.
+	return request.respChan, nil
+}
+
+// Starts a pool of workers that fetch RRs for the requested zones and
+// BIND 9 configurations from the agents. Using the worker pool limits
+// the number of concurrent requests to the agents.
+func (manager *managerImpl) startAsyncRequestWorkers(ctx context.Context) {
 	pool := storkutil.NewPausablePool(runtime.GOMAXPROCS(0) * 2)
-	manager.rrsReqsState.pool = pool
+	manager.pool = pool
 	go func(ctx context.Context) {
 		defer func() {
 			// When the worker pool is stopped, we need to close the request channel.
@@ -539,15 +706,26 @@ func (manager *managerImpl) startRRsRequestWorkers(ctx context.Context) {
 					log.WithError(err).Error("Failed to submit RRs request to the worker pool")
 					request.respChan <- NewErrorRRResponse(err)
 				}
+			case request, ok := <-manager.bind9FormattedConfigReqsState.requestChan:
+				if !ok {
+					return
+				}
+				err := pool.Submit(func() {
+					manager.runBind9FormattedConfigRequest(request)
+				})
+				if err != nil {
+					log.WithError(err).Error("Failed to submit BIND 9 configuration request to the worker pool")
+					request.respChan <- NewBind9FormattedConfigResponseError(err)
+				}
 			}
 		}
 	}(ctx)
 }
 
-// Stops the worker pool for fetching RRs.
+// Stops the worker pool for receiving RRs and BIND 9 configuration.
 func (manager *managerImpl) stopRRsRequestWorkers() {
-	manager.rrsReqsState.pool.Stop()
-	manager.rrsReqsState.cancel()
+	manager.pool.Stop()
+	manager.cancel()
 }
 
 // Shuts down the DNS manager by stopping background tasks.
@@ -562,11 +740,11 @@ func (manager *managerImpl) Shutdown() {
 // override the cached RRs.
 func (manager *managerImpl) GetZoneRRs(zoneID int64, daemonID int64, viewName string, options ...GetZoneRRsOption) iter.Seq[*RRResponse] {
 	return func(yield func(*RRResponse) bool) {
-		// We need an app associated with the daemon.
-		daemon, err := dbmodel.GetDaemonByID(manager.db, daemonID)
+		// We need a daemon associated with the zone.
+		daemon, err := dbmodel.GetDNSDaemonByID(manager.db, daemonID)
 		if err != nil {
 			// This is unexpected and we can't proceed because we
-			// don't have the app instance.
+			// don't have the daemon instance.
 			_ = yield(NewErrorRRResponse(err))
 			return
 		}
@@ -575,7 +753,6 @@ func (manager *managerImpl) GetZoneRRs(zoneID int64, daemonID int64, viewName st
 			_ = yield(NewErrorRRResponse(errors.Errorf("daemon with the ID of %d not found", daemonID)))
 			return
 		}
-		app := daemon.App
 
 		// We need a zone name, so let's get it from the database.
 		zone, err := dbmodel.GetZoneByID(manager.db, zoneID, dbmodel.ZoneRelationLocalZones)
@@ -617,7 +794,7 @@ func (manager *managerImpl) GetZoneRRs(zoneID int64, daemonID int64, viewName st
 		key := h.Sum64()
 		ctx, cancel := context.WithCancel(context.Background())
 		defer cancel()
-		ch, err := manager.requestZoneRRs(ctx, key, app, zone.Name, viewName)
+		ch, err := manager.requestZoneRRs(ctx, key, daemon, zone.Name, viewName)
 		if err != nil {
 			// The zone inventory is most likely busy.
 			_ = yield(NewErrorRRResponse(err))
@@ -684,6 +861,44 @@ func (manager *managerImpl) GetZoneRRs(zoneID int64, daemonID int64, viewName st
 	}
 }
 
+// Returns the BIND 9 configuration for the specified daemon with filtering.
+// If the filter is nil, all configuration elements are returned. Otherwise,
+// only the configuration elements explicitly enabled in the filter are returned.
+// The specified daemon ID must point to a BIND 9 daemon. If it points to a different
+// daemon type, the function returns an error without attempting to contact the agent.
+// The returned configuration may contain multiple files. Typically, it contains the
+// main configuration file and the rndc.key file.
+func (manager *managerImpl) GetBind9FormattedConfig(ctx context.Context, daemonID int64, fileSelector *bind9config.FileTypeSelector, filter *bind9config.Filter) iter.Seq[*Bind9FormattedConfigResponse] {
+	return func(yield func(*Bind9FormattedConfigResponse) bool) {
+		// The daemon must be present in the database. We're going to use the
+		// connection parameters associated with the daemon to contact the agent.
+		daemon, err := dbmodel.GetDaemonByID(manager.db, daemonID)
+		if err != nil {
+			_ = yield(NewBind9FormattedConfigResponseError(err))
+			return
+		}
+		if daemon == nil {
+			_ = yield(NewBind9FormattedConfigResponseError(errors.Errorf("unable to get BIND 9 configuration from non-existent daemon with the ID %d", daemonID)))
+			return
+		}
+		ctx, cancel := context.WithCancel(ctx)
+		defer cancel()
+		ch, err := manager.requestBind9FormattedConfig(ctx, daemonID, daemon, fileSelector, filter)
+		if err != nil {
+			// The zone inventory is most likely busy.
+			_ = yield(NewBind9FormattedConfigResponseError(err))
+			return
+		}
+		for r := range ch {
+			if !yield(r) || r.Err != nil {
+				// Stop reading the BIND 9 configuration if the caller is done reading
+				// or there was an error reading from the channel.
+				return
+			}
+		}
+	}
+}
+
 // Convenience function storing a value in a map with mutex protection.
 func storeResult[K comparable, T any](mutex *sync.Mutex, results map[K]T, key K, value T) {
 	mutex.Lock()
