@@ -26,12 +26,12 @@ import (
 	log "github.com/sirupsen/logrus"
 	"golang.org/x/net/netutil"
 
-	keaconfig "isc.org/stork/appcfg/kea"
+	keaconfig "isc.org/stork/daemoncfg/kea"
 	"isc.org/stork/server/agentcomm"
-	"isc.org/stork/server/apps"
 	"isc.org/stork/server/config"
 	"isc.org/stork/server/configmigrator"
 	"isc.org/stork/server/configreview"
+	"isc.org/stork/server/daemons"
 	dbops "isc.org/stork/server/database"
 	dbsession "isc.org/stork/server/database/session"
 	"isc.org/stork/server/dnsop"
@@ -73,7 +73,7 @@ type RestAPI struct {
 	DB                         *dbops.PgDB
 	SessionManager             *dbsession.SessionMgr
 	EventCenter                eventcenter.EventCenter
-	Pullers                    *apps.Pullers
+	Pullers                    *daemons.Pullers
 	ReviewDispatcher           configreview.Dispatcher
 	MetricsCollector           metrics.Collector
 	ConfigManager              config.Manager
@@ -203,8 +203,8 @@ func NewRestAPI(args ...interface{}) (*RestAPI, error) {
 			api.DB = arg.(*pg.DB)
 			continue
 		}
-		if argType.AssignableTo(reflect.TypeOf((*apps.Pullers)(nil))) {
-			api.Pullers = arg.(*apps.Pullers)
+		if argType.AssignableTo(reflect.TypeOf((*daemons.Pullers)(nil))) {
+			api.Pullers = arg.(*daemons.Pullers)
 			continue
 		}
 		if argType.AssignableTo(reflect.TypeOf((*RestAPISettings)(nil))) {
@@ -608,7 +608,7 @@ func (r *RestAPI) Shutdown() {
 
 		r.HTTPServer.SetKeepAlivesEnabled(false)
 		if err := r.HTTPServer.Shutdown(ctx); err != nil {
-			log.Warnf("Could not gracefully shut down the server: %v\n", err)
+			log.WithError(err).Warn("Could not gracefully shut down the server")
 		}
 	}
 	log.Printf("Stopped RESTful API Service")
