@@ -9,9 +9,10 @@ import (
 	"github.com/go-openapi/runtime/middleware"
 	"github.com/pkg/errors"
 	log "github.com/sirupsen/logrus"
-	keaconfig "isc.org/stork/appcfg/kea"
-	"isc.org/stork/server/apps/kea"
+	keaconfig "isc.org/stork/daemoncfg/kea"
+	"isc.org/stork/datamodel/daemonname"
 	"isc.org/stork/server/config"
+	"isc.org/stork/server/daemons/kea"
 	dbmodel "isc.org/stork/server/database/model"
 	"isc.org/stork/server/gen/models"
 	dhcp "isc.org/stork/server/gen/restapi/operations/d_h_c_p"
@@ -118,10 +119,11 @@ func (r *RestAPI) convertSharedNetworkToRestAPI(sn *dbmodel.SharedNetwork) *mode
 	}
 
 	for _, lsn := range sn.LocalSharedNetworks {
+		app := lsn.Daemon.GetVirtualApp()
 		localSharedNetwork := &models.LocalSharedNetwork{
-			AppID:    lsn.Daemon.App.ID,
+			AppID:    app.ID,
 			DaemonID: lsn.Daemon.ID,
-			AppName:  lsn.Daemon.App.Name,
+			AppName:  app.Name,
 		}
 		keaParameters := lsn.KeaParameters
 		if keaParameters != nil {
@@ -268,8 +270,42 @@ func (r *RestAPI) convertSharedNetworkFromRestAPI(restSharedNetwork *models.Shar
 
 // Get the list of shared networks for the given set of parameters.
 func (r *RestAPI) getSharedNetworks(offset, limit, appID, family int64, filterText *string, sortField string, sortDir dbmodel.SortDirEnum) (*models.SharedNetworks, error) {
+	var daemonID int64
+	if appID != 0 {
+		// Fetch daemons.
+		daemons, err := dbmodel.GetDaemonsByVirtualAppID(r.DB, appID)
+		if err != nil {
+			return nil, errors.WithMessage(err, "cannot get daemons from db")
+		}
+		// Select daemon matching the specified family.
+		var daemon *dbmodel.Daemon
+	DAEMON_LOOP:
+		for _, d := range daemons {
+			switch family {
+			case 0:
+				if d.Name == daemonname.DHCPv4 || d.Name == daemonname.DHCPv6 {
+					daemon = d
+				}
+			case 4:
+				if d.Name == daemonname.DHCPv4 {
+					daemon = d
+					break DAEMON_LOOP
+				}
+			case 6:
+				if d.Name == daemonname.DHCPv6 {
+					daemon = d
+					break DAEMON_LOOP
+				}
+			}
+		}
+		if daemon == nil {
+			return &models.SharedNetworks{}, nil
+		}
+		daemonID = daemon.ID
+	}
+
 	// get shared networks from db
-	dbSharedNetworks, total, err := dbmodel.GetSharedNetworksByPage(r.DB, offset, limit, appID, family, filterText, sortField, sortDir)
+	dbSharedNetworks, total, err := dbmodel.GetSharedNetworksByPage(r.DB, offset, limit, daemonID, family, filterText, sortField, sortDir)
 	if err != nil {
 		return nil, err
 	}
@@ -299,6 +335,16 @@ func (r *RestAPI) GetSharedNetworks(ctx context.Context, params dhcp.GetSharedNe
 		limit = *params.Limit
 	}
 
+	sortField := ""
+	if params.SortField != nil {
+		sortField = *params.SortField
+	}
+
+	sortDir := dbmodel.SortDirAny
+	if params.SortDir != nil {
+		sortDir = dbmodel.SortDirEnum(*params.SortDir)
+	}
+
 	var appID int64
 	if params.AppID != nil {
 		appID = *params.AppID
@@ -310,10 +356,10 @@ func (r *RestAPI) GetSharedNetworks(ctx context.Context, params dhcp.GetSharedNe
 	}
 
 	// get shared networks from db
-	sharedNetworks, err := r.getSharedNetworks(start, limit, appID, dhcpVer, params.Text, "", dbmodel.SortDirAsc)
+	sharedNetworks, err := r.getSharedNetworks(start, limit, appID, dhcpVer, params.Text, sortField, sortDir)
 	if err != nil {
 		msg := "Cannot get shared network from db"
-		log.Error(err)
+		log.WithError(err).Error(msg)
 		rsp := dhcp.NewGetSharedNetworksDefault(http.StatusInternalServerError).WithPayload(&models.APIError{
 			Message: &msg,
 		})
@@ -562,7 +608,7 @@ func (r *RestAPI) UpdateSharedNetworkBegin(ctx context.Context, params dhcp.Upda
 		case errors.As(err, &sharedNetworkNotFound):
 			// Failed to find shared network.
 			msg := fmt.Sprintf("Unable to edit the shared network with ID %d because it cannot be found", params.SharedNetworkID)
-			log.Error(msg)
+			log.WithError(err).Error(msg)
 			rsp := dhcp.NewUpdateSharedNetworkBeginDefault(http.StatusBadRequest).WithPayload(&models.APIError{
 				Message: &msg,
 			})
@@ -578,7 +624,7 @@ func (r *RestAPI) UpdateSharedNetworkBegin(ctx context.Context, params dhcp.Upda
 		case errors.As(err, &hooksNotConfigured):
 			// Lack of the libdhcp_subnet_cmds hook.
 			msg := "Unable to update shared network configuration because some daemons lack libdhcp_subnet_cmds hook library"
-			log.Error(msg)
+			log.WithError(err).Error(msg)
 			rsp := dhcp.NewUpdateSharedNetworkBeginDefault(http.StatusBadRequest).WithPayload(&models.APIError{
 				Message: &msg,
 			})
@@ -686,7 +732,7 @@ func (r *RestAPI) DeleteSharedNetwork(ctx context.Context, params dhcp.DeleteSha
 	cctx, err := r.ConfigManager.CreateContext(int64(user.ID))
 	if err != nil {
 		msg := "Problem with creating transaction context for deleting the shared network"
-		log.WithError(err).Error(err)
+		log.WithError(err).Error(msg)
 		rsp := dhcp.NewDeleteSharedNetworkDefault(http.StatusInternalServerError).WithPayload(&models.APIError{
 			Message: &msg,
 		})
