@@ -257,7 +257,17 @@ func (r *RestAPI) GetUsers(ctx context.Context, params users.GetUsersParams) mid
 		limit = *params.Limit
 	}
 
-	usersRsp, err := r.getUsers(start, limit, params.Text, "", dbmodel.SortDirAny)
+	sortField := ""
+	if params.SortField != nil {
+		sortField = *params.SortField
+	}
+
+	sortDir := dbmodel.SortDirAny
+	if params.SortDir != nil {
+		sortDir = dbmodel.SortDirEnum(*params.SortDir)
+	}
+
+	usersRsp, err := r.getUsers(start, limit, params.Text, sortField, sortDir)
 	if err != nil {
 		log.WithFields(log.Fields{
 			"start": start,
@@ -669,7 +679,7 @@ func (r *RestAPI) GetGroups(ctx context.Context, params users.GetGroupsParams) m
 
 	groups, err := r.getGroups(start, limit, params.Text, "", dbmodel.SortDirAny)
 	if err != nil {
-		log.Errorf("Failed to get groups from the database with error: %s", err.Error())
+		log.WithError(err).Error("Failed to get groups from the database")
 
 		msg := "Failed to get groups from the database"
 		rspErr := models.APIError{
