@@ -10,6 +10,7 @@ import (
 	"github.com/go-openapi/strfmt"
 	"github.com/pkg/errors"
 	log "github.com/sirupsen/logrus"
+	"isc.org/stork/datamodel/daemonname"
 	"isc.org/stork/server/agentcomm"
 	dbmodel "isc.org/stork/server/database/model"
 	"isc.org/stork/server/dnsop"
@@ -21,7 +22,7 @@ import (
 // Returns a single DNS zone.
 func (r *RestAPI) GetZone(ctx context.Context, params dns.GetZoneParams) middleware.Responder {
 	// Find the zone in the database.
-	dbZone, err := dbmodel.GetZoneByID(r.DB, params.ZoneID, dbmodel.ZoneRelationLocalZonesApp)
+	dbZone, err := dbmodel.GetZoneByID(r.DB, params.ZoneID, dbmodel.ZoneRelationLocalZonesDaemon, dbmodel.ZoneRelationLocalZonesMachine)
 	if err != nil {
 		// Error while communicating with the database.
 		msg := fmt.Sprintf("Problem fetching DNS zone with ID %d from db", params.ZoneID)
@@ -42,9 +43,10 @@ func (r *RestAPI) GetZone(ctx context.Context, params dns.GetZoneParams) middlew
 	// Zone found. Convert it to the format used in REST API.
 	var restLocalZones []*models.LocalZone
 	for _, localZone := range dbZone.LocalZones {
+		virtualApp := localZone.Daemon.GetVirtualApp()
 		restLocalZones = append(restLocalZones, &models.LocalZone{
-			AppID:    localZone.Daemon.App.ID,
-			AppName:  localZone.Daemon.App.Name,
+			AppID:    virtualApp.ID,
+			AppName:  virtualApp.Name,
 			Class:    localZone.Class,
 			DaemonID: localZone.DaemonID,
 			LoadedAt: strfmt.DateTime(localZone.LoadedAt),
@@ -75,22 +77,72 @@ func (r *RestAPI) GetZones(ctx context.Context, params dns.GetZonesParams) middl
 	if params.Limit != nil {
 		limit = int(*params.Limit)
 	}
+	sortField := "rname"
+	if params.SortField != nil {
+		sortField = *params.SortField
+	}
+	sortDir := dbmodel.SortDirAsc
+	if params.SortDir != nil {
+		sortDir = dbmodel.SortDirEnum(*params.SortDir)
+	}
+
+	var daemonName *daemonname.Name
+	if params.AppType != nil {
+		switch *params.AppType {
+		case "bind9":
+			daemonName = storkutil.Ptr(daemonname.Bind9)
+		case "pdns":
+			daemonName = storkutil.Ptr(daemonname.PDNS)
+		default:
+			// Unknown app type, return empty result.
+			payload := models.Zones{
+				Items: []*models.Zone{},
+				Total: 0,
+			}
+			rsp := dns.NewGetZonesOK().WithPayload(&payload)
+			return rsp
+		}
+	}
+
+	var machineIDPtr *int64
+	if params.AppID != nil {
+		machineID, err := dbmodel.GetMachineIDByVirtualAppID(r.DB, *params.AppID)
+		if err != nil {
+			msg := "Failed to get machine ID by virtual app ID"
+			log.WithError(err).Error(msg)
+			rsp := dns.NewGetZonesDefault(http.StatusInternalServerError).WithPayload(&models.APIError{
+				Message: &msg,
+			})
+			return rsp
+		}
+		if machineID == 0 {
+			// No machine with the specified virtual app ID exist, return empty result.
+			payload := models.Zones{
+				Items: []*models.Zone{},
+				Total: 0,
+			}
+			rsp := dns.NewGetZonesOK().WithPayload(&payload)
+			return rsp
+		}
+		machineIDPtr = &machineID
+	}
+
 	// Apply paging parameters and zone-specific filters.
 	filter := &dbmodel.GetZonesFilter{
-		AppID:   params.AppID,
-		AppType: params.AppType,
-		Class:   params.Class,
-		RPZ:     params.Rpz,
-		Serial:  params.Serial,
-		Text:    params.Text,
-		Offset:  storkutil.Ptr(offset),
-		Limit:   storkutil.Ptr(limit),
+		MachineID:  machineIDPtr,
+		DaemonName: daemonName,
+		Class:      params.Class,
+		RPZ:        params.Rpz,
+		Serial:     params.Serial,
+		Text:       params.Text,
+		Offset:     storkutil.Ptr(offset),
+		Limit:      storkutil.Ptr(limit),
 	}
 	for _, zoneType := range params.ZoneType {
 		filter.EnableZoneType(dbmodel.ZoneType(zoneType))
 	}
 	// Get the zones from the database.
-	zones, total, err := dbmodel.GetZones(r.DB, filter, dbmodel.ZoneRelationLocalZonesApp)
+	zones, total, err := dbmodel.GetZones(r.DB, filter, sortField, sortDir, dbmodel.ZoneRelationLocalZonesDaemon, dbmodel.ZoneRelationLocalZonesAccessPoints, dbmodel.ZoneRelationLocalZonesMachine)
 	if err != nil {
 		msg := "Failed to get zones from the database"
 		log.WithError(err).Error(msg)
@@ -105,9 +157,10 @@ func (r *RestAPI) GetZones(ctx context.Context, params dns.GetZonesParams) middl
 	for _, zone := range zones {
 		var restLocalZones []*models.LocalZone
 		for _, localZone := range zone.LocalZones {
+			app := localZone.Daemon.GetVirtualApp()
 			restLocalZones = append(restLocalZones, &models.LocalZone{
-				AppID:    localZone.Daemon.App.ID,
-				AppName:  localZone.Daemon.App.Name,
+				AppID:    app.ID,
+				AppName:  app.Name,
 				Class:    localZone.Class,
 				DaemonID: localZone.DaemonID,
 				LoadedAt: strfmt.DateTime(localZone.LoadedAt),
@@ -144,7 +197,7 @@ func (r *RestAPI) GetZonesFetch(ctx context.Context, params dns.GetZonesFetchPar
 		rsp := dns.NewGetZonesFetchAccepted().WithPayload(&payload)
 		return rsp
 	}
-	states, count, err := dbmodel.GetZoneInventoryStates(r.DB, dbmodel.ZoneInventoryStateRelationApp)
+	states, count, err := dbmodel.GetZoneInventoryStates(r.DB, dbmodel.ZoneInventoryStateRelationDaemon, dbmodel.ZoneInventoryStateRelationAccessPoints, dbmodel.ZoneInventoryStateRelationMachine)
 	if err != nil {
 		msg := "Failed to get zones fetch states from the database"
 		log.WithError(err).Error(msg)
@@ -159,9 +212,10 @@ func (r *RestAPI) GetZonesFetch(ctx context.Context, params dns.GetZonesFetchPar
 	}
 	var restStates []*models.ZoneInventoryState
 	for _, state := range states {
+		app := state.Daemon.GetVirtualApp()
 		restStates = append(restStates, &models.ZoneInventoryState{
-			AppID:              state.Daemon.AppID,
-			AppName:            state.Daemon.App.Name,
+			AppID:              app.ID,
+			AppName:            app.Name,
 			CreatedAt:          strfmt.DateTime(state.CreatedAt),
 			DaemonID:           state.DaemonID,
 			Error:              state.State.Error,
@@ -214,7 +268,7 @@ func (r *RestAPI) GetZoneRRs(ctx context.Context, params dns.GetZoneRRsParams) m
 	)
 	for rrResponse := range r.DNSManager.GetZoneRRs(params.ZoneID, params.DaemonID, params.ViewName) {
 		if rrResponse.Err != nil {
-			msg := "Failed to get zone contents from the database"
+			msg := "Failed to get zone contents"
 			log.WithError(rrResponse.Err).Error(msg)
 			switch {
 			case errors.As(rrResponse.Err, &alreadyRequestedError):
