@@ -9,16 +9,16 @@ import (
 	log "github.com/sirupsen/logrus"
 
 	"isc.org/stork"
-	keaconfig "isc.org/stork/appcfg/kea"
+	keaconfig "isc.org/stork/daemoncfg/kea"
 	"isc.org/stork/hooks"
 	"isc.org/stork/server/agentcomm"
-	"isc.org/stork/server/apps"
-	"isc.org/stork/server/apps/bind9"
-	"isc.org/stork/server/apps/kea"
 	"isc.org/stork/server/certs"
 	"isc.org/stork/server/config"
 	"isc.org/stork/server/configmigrator"
 	"isc.org/stork/server/configreview"
+	"isc.org/stork/server/daemons"
+	"isc.org/stork/server/daemons/bind9"
+	"isc.org/stork/server/daemons/kea"
 	dbops "isc.org/stork/server/database"
 	dbmodel "isc.org/stork/server/database/model"
 	"isc.org/stork/server/dnsop"
@@ -41,7 +41,7 @@ type StorkServer struct {
 
 	GeneralSettings GeneralSettings
 
-	Pullers *apps.Pullers
+	Pullers *daemons.Pullers
 
 	MetricsCollector metrics.Collector
 
@@ -162,7 +162,7 @@ func (ss *StorkServer) Bootstrap(reload bool) (err error) {
 		return err
 	}
 
-	ss.Pullers = &apps.Pullers{}
+	ss.Pullers = &daemons.Pullers{}
 
 	// This instance provides functions to search for option definitions, both in the
 	// database and among the standard options. It is required by the config manager.
@@ -172,8 +172,8 @@ func (ss *StorkServer) Bootstrap(reload bool) (err error) {
 	// is global for the server so various parts of the application can use it.
 	ss.DaemonLocker = config.NewDaemonLocker()
 
-	// setup apps state puller
-	ss.Pullers.AppsStatePuller, err = apps.NewStatePuller(ss.DB, ss.Agents, ss.EventCenter, ss.ReviewDispatcher, ss.DHCPOptionDefinitionLookup)
+	// setup state puller
+	ss.Pullers.StatePuller, err = daemons.NewStatePuller(ss.DB, ss.Agents, ss.EventCenter, ss.ReviewDispatcher, ss.DHCPOptionDefinitionLookup)
 	if err != nil {
 		return err
 	}
@@ -220,7 +220,7 @@ func (ss *StorkServer) Bootstrap(reload bool) (err error) {
 	// important to maintain one instance of the lookup because it applies indexing
 	// on the option definitions it returns. Indexing should be done only once at
 	// server startup.
-	ss.ConfigManager = apps.NewManager(ss)
+	ss.ConfigManager = daemons.NewManager(ss)
 
 	// Check if the machine registration endpoint should be disabled.
 	enableMachineRegistration, err := dbmodel.GetSettingBool(ss.DB, "enable_machine_registration")
@@ -252,7 +252,7 @@ func (ss *StorkServer) Bootstrap(reload bool) (err error) {
 		ss.Pullers.KeaHostsPuller.Shutdown()
 		ss.Pullers.KeaStatsPuller.Shutdown()
 		ss.Pullers.Bind9StatsPuller.Shutdown()
-		ss.Pullers.AppsStatePuller.Shutdown()
+		ss.Pullers.StatePuller.Shutdown()
 		ss.DNSManager.Shutdown()
 		if ss.MetricsCollector != nil {
 			ss.MetricsCollector.Shutdown()
@@ -300,7 +300,7 @@ func (ss *StorkServer) Shutdown(reload bool) {
 		ss.Pullers.KeaHostsPuller.Shutdown()
 		ss.Pullers.KeaStatsPuller.Shutdown()
 		ss.Pullers.Bind9StatsPuller.Shutdown()
-		ss.Pullers.AppsStatePuller.Shutdown()
+		ss.Pullers.StatePuller.Shutdown()
 		ss.DNSManager.Shutdown()
 		ss.Agents.Shutdown()
 		ss.EventCenter.Shutdown()
