@@ -1,3 +1,4 @@
+from datetime import datetime, timedelta
 import os.path
 from pathlib import Path
 import shutil
@@ -7,7 +8,6 @@ import pytest
 from core.compose_factory import create_docker_compose
 from core import wrappers
 from core.utils import setup_logger
-from core import lease_generators
 from core import performance_chart
 from core.version import parse_version_info
 
@@ -81,9 +81,10 @@ def kea_parametrize(
     )
 
 
-def ha_pair_parametrize(
+def ha_parametrize(
     first_service_name="agent-kea-ha1",
     second_service_name="agent-kea-ha2",
+    third_service_name="agent-kea-ha3",
     min_version=None,
     suppress_registration=False,
 ):
@@ -96,6 +97,9 @@ def ha_pair_parametrize(
         Name of docker-compose service of the first Kea instance, by default "agent-kea-ha1"
     second_service_name : str, optional
         Name of docker-compose service of the second Kea instance, by default "agent-kea-ha2"
+    third_service_name : str, optional
+        Name of docker-compose service of the third Kea instance, by default "agent-kea-ha3"
+        Set to None if the third instance should not be started.
     suppress_registration : bool, optional
         Suppress the Stork Agent registration in a server, by default False
     min_version : str, optional
@@ -109,11 +113,12 @@ def ha_pair_parametrize(
         the Pytest decorator ready to use
     """
     return pytest.mark.parametrize(
-        "ha_pair_service",
+        "ha_service",
         [
             {
                 "first_service_name": first_service_name,
                 "second_service_name": second_service_name,
+                "third_service_name": third_service_name,
                 "suppress_registration": suppress_registration,
                 "min_kea_version": min_version,
             }
@@ -251,9 +256,9 @@ def kea_service(request):
 
 
 @pytest.fixture
-def ha_pair_service(request):
+def ha_service(request):
     """
-    A fixture setting up the Kea High-Availability pair services and
+    A fixture setting up the Kea High-Availability services and
     guarantees that they are operational.
 
     Parameters
@@ -268,11 +273,12 @@ def ha_pair_service(request):
 
     Notes
     -----
-    You can use the ha_pair_parametrize helper for configure the service.
+    You can use the ha_parametrize helper for configure the service.
     """
     param = {
         "first_service_name": "agent-kea-ha1",
         "second_service_name": "agent-kea-ha2",
+        "third_service_name": "agent-kea-ha3",
         "suppress_registration": False,
         "min_kea_version": None,
     }
@@ -293,7 +299,17 @@ def ha_pair_service(request):
         config_dirname="kea-ha2",
         min_kea_version=param["min_kea_version"],
     )
-    return first_wrapper, second_wrapper
+    if param["third_service_name"] is not None:
+        third_wrapper = _prepare_kea_wrapper(
+            request=request,
+            service_name=param["third_service_name"],
+            suppress_registration=param["suppress_registration"],
+            config_dirname="kea-ha3",
+            min_kea_version=param["min_kea_version"],
+        )
+    else:
+        third_wrapper = None
+    return first_wrapper, second_wrapper, third_wrapper
 
 
 def _prepare_kea_wrapper(
@@ -359,12 +375,7 @@ def _prepare_kea_wrapper(
         server_service_instance = request.getfixturevalue("server_service")
 
     # Re-generate the lease files
-    config_dir = os.path.join(os.path.dirname(__file__), "../config", config_dirname)
-    with open(os.path.join(config_dir, "kea-leases4.csv"), "wt", encoding="utf-8") as f:
-        lease_generators.gen_dhcp4_lease_file(f)
-
-    with open(os.path.join(config_dir, "kea-leases6.csv"), "wt", encoding="utf-8") as f:
-        lease_generators.gen_dhcp6_lease_file(f)
+    _regenerate_lease_files(config_dirname)
 
     compose.bootstrap(service_name)
     compose.wait_for_operational(service_name)
@@ -376,6 +387,28 @@ def _prepare_kea_wrapper(
     return wrapper
 
 
+def _regenerate_lease_files(config_dirname: str):
+    """
+    Regenerates the Kea lease files with a new expiration time.
+    This is used to ensure that the lease files are always up-to-date.
+
+    Parameters
+    ----------
+    config_dirname : str
+        The name of the directory where the lease files are stored.
+    """
+    config_dir = os.path.join(os.path.dirname(__file__), "../config", config_dirname)
+    lease_expire = str(int((datetime.now() + timedelta(minutes=10)).timestamp()))
+
+    for template_path in Path(config_dir).glob("kea-leases*.csv.template"):
+        with open(template_path, "rt", encoding="utf-8") as f:
+            lease_template = f.read()
+        lease_file_content = lease_template.format(expire=lease_expire)
+        lease_path = template_path.with_suffix("")
+        with open(lease_path, "w+t", encoding="utf-8") as f:
+            f.write(lease_file_content)
+
+
 @pytest.fixture
 def bind9_service(request):
     """
@@ -537,6 +570,13 @@ def collect_logs(test_dir: Path):
 
     # Write logs
     with open(test_dir / "stdout.log", "wt", encoding="utf-8") as f:
+        # Stdout log is in format: container_name | timestamp message
+        # Sort by timestamp.
+        stdout = "\n".join(
+            sorted(
+                stdout.splitlines(), key=lambda x: x.split("|")[1] if "|" in x else x
+            )
+        )
         f.write(stdout)
 
     with open(test_dir / "stderr.log", "wt", encoding="utf-8") as f:
@@ -549,10 +589,23 @@ def collect_logs(test_dir: Path):
             continue
         has_non_operational_service = True
         inspect_stdout = compose.inspect_raw(service_name)
-        filename = f"inspect-{service_name}.json"
-        with open(test_dir / filename, "wt", encoding="utf-8") as f:
+        with open(
+            test_dir / f"inspect-{service_name}.json", "wt", encoding="utf-8"
+        ) as f:
             f.write(inspect_stdout)
 
+        # Collect supervisor status
+        _, stdout, _ = compose.exec(
+            service_name, ["supervisorctl", "status"], check=False
+        )
+        if stdout.strip() != "":
+            with open(
+                test_dir / f"supervisorctl-status-{service_name}.out",
+                "wt",
+                encoding="utf-8",
+            ) as f:
+                f.write(stdout)
+
     if has_non_operational_service:
         # Collect service statuses
         ps_stdout = compose.ps()
@@ -616,11 +669,14 @@ def finish(request):
 
         # If the test fails due to non-assertion error then the call status is
         # unavailable.
-        if hasattr(request.node, "rep_call") and request.node.rep_call.failed:
+        # The rep_call attribute is unavailable if an error occurs in the
+        # fixture setup phase.
+        if not hasattr(request.node, "rep_call") or request.node.rep_call.failed:
             collect_logs(test_dir)
 
         # Down all containers
         compose = create_docker_compose()
+        compose.kill()
         compose.down()
 
     request.addfinalizer(collect_logs_and_down_all)
