@@ -1,3 +1,4 @@
+import functools
 import re
 from contextlib import contextmanager
 from datetime import datetime, timezone, timedelta
@@ -126,11 +127,30 @@ class Server(ComposeServiceWrapper):  # pylint: disable=too-many-public-methods)
     # Authentication
 
     def log_in(
-        self, username: str, password: str, authentication_method_id="internal"
+        self,
+        username: str,
+        password: str,
+        authentication_method_id: str = "internal",
+        suppress_client_side_validation: bool = False,
     ) -> User:
-        """Logs in a user. Returns the user info."""
+        """
+        Logs in a user. Returns the user info.
+        It accepts a flag to suppress client-side validation. It is useful when
+        testing malicious payloads.
+        """
         api_instance = UsersApi(self._api_client)
-        http_info = api_instance.create_session_with_http_info(
+        create_session_method = api_instance.create_session_with_http_info
+
+        if suppress_client_side_validation:
+            # The OpenAPI client wraps the method calls with decorators that
+            # perform client-side validation. We need to unwrap the original
+            # method and call it directly. The raw_function needs to have self
+            # parameter bound to the api_instance (original caller object).
+            create_session_method = functools.partial(
+                create_session_method.raw_function, api_instance
+            )
+
+        http_info = create_session_method(
             credentials={
                 "identifier": username,
                 "secret": password,
@@ -663,7 +683,11 @@ class Server(ComposeServiceWrapper):  # pylint: disable=too-many-public-methods)
         until the end of writing all application-related entries (subnets,
         shared networks, hosts) to the database.
         """
-        self._wait_for_puller("apps_state_puller_interval", start)
+        try:
+            self._wait_for_puller("state_puller_interval", start)
+        except openapi_client.exceptions.NotFoundException:
+            # Applicable only when the server is running in the older version.
+            self._wait_for_puller("apps_state_puller_interval", start)
 
     @wait_for_success(
         DatabaseDeadlockError, wait_msg="Waiting to fetch next machine state..."
@@ -768,8 +792,21 @@ class Server(ComposeServiceWrapper):  # pylint: disable=too-many-public-methods)
     def wait_for_fetch_zones(self) -> ZoneInventoryStates:
         """Waits for finishing the zones fetch."""
         zones_states = self.get_zone_inventory_states()
-        if zones_states is None:
-            raise NoSuccessException("zones have not been fetched yet")
+        if zones_states is None or zones_states.total == 0:
+            # The zone fetching has not been started yet. Trigger fetching.
+            self.fetch_zones()
+            raise NoSuccessException("zones have not been available yet")
+        for state in zones_states.items:
+            if state.status == "uninitialized":
+                # Re-trigger fetching. Maybe this time the inventory will be
+                # ready.
+                self.fetch_zones()
+                raise NoSuccessException("zone inventory not initialized yet")
+            if state.status == "busy":
+                # Fetching is in progress. Wait until it is finished.
+                raise NoSuccessException("zones fetching is in progress")
+            if state.status == "erred":
+                raise Exception("zone inventory is in erred state")
         return zones_states
 
     @wait_for_success(
