@@ -9,12 +9,12 @@ import { AppsVersions, GeneralService, ServicesService, Settings, SettingsServic
 import { provideHttpClientTesting } from '@angular/common/http/testing'
 import { MessageService } from 'primeng/api'
 import { GlobalSearchComponent } from './global-search/global-search.component'
-import { OverlayPanelModule } from 'primeng/overlaypanel'
+import { PopoverModule } from 'primeng/popover'
 import { NoopAnimationsModule } from '@angular/platform-browser/animations'
 import { FormsModule } from '@angular/forms'
 import { PriorityErrorsPanelComponent } from './priority-errors-panel/priority-errors-panel.component'
 import { ServerSentEventsService, ServerSentEventsTestingService } from './server-sent-events.service'
-import { MessagesModule } from 'primeng/messages'
+import { MessageModule } from 'primeng/message'
 import { ToggleButtonModule } from 'primeng/togglebutton'
 import { SettingService } from './setting.service'
 import { of } from 'rxjs'
@@ -49,10 +49,10 @@ describe('AppComponent', () => {
                 SplitButtonModule,
                 ProgressSpinnerModule,
                 ToastModule,
-                OverlayPanelModule,
+                PopoverModule,
                 NoopAnimationsModule,
                 FormsModule,
-                MessagesModule,
+                MessageModule,
                 ToggleButtonModule,
             ],
             providers: [
@@ -112,7 +112,8 @@ describe('AppComponent', () => {
             'Subnets',
             'Shared Networks',
             'Services',
-            'Apps',
+            'Kea Apps',
+            'DNS Apps',
             'Machines',
             'Grafana',
             'Monitoring',
@@ -130,7 +131,7 @@ describe('AppComponent', () => {
 
         // List of menu items that are expected to be hidden. Unless listed here, the test expects
         // the menu item to be visible.
-        const expHiddenItems = ['DHCP', 'DNS', 'Apps', 'Grafana', 'Users']
+        const expHiddenItems = ['DHCP', 'Kea Apps', 'DNS Apps', 'Grafana', 'Users']
 
         for (const name of expMenuItems) {
             // Check if the menu item is there
@@ -183,11 +184,9 @@ describe('AppComponent', () => {
         expect(badges.length).toEqual(2)
 
         // Check if error severity badge is shown in top bar menu.
-        expect(badges[0].classes.hasOwnProperty('p-menuitem-badge')).toBeTrue()
         expect(badges[0].classes.hasOwnProperty('p-badge-dot')).toBeTrue()
         expect(badges[0].classes.hasOwnProperty('p-badge-danger')).toBeTrue()
 
-        expect(badges[1].classes.hasOwnProperty('p-menuitem-badge')).toBeTrue()
         expect(badges[1].classes.hasOwnProperty('p-badge-dot')).toBeTrue()
         expect(badges[1].classes.hasOwnProperty('p-badge-danger')).toBeTrue()
     })
