@@ -67,7 +67,6 @@ import { AuthInterceptor } from './auth-interceptor'
 import { LoginScreenComponent } from './login-screen/login-screen.component'
 import { DashboardComponent } from './dashboard/dashboard.component'
 import { HostsTableComponent } from './hosts-table/hosts-table.component'
-import { SwaggerUiComponent } from './swagger-ui/swagger-ui.component'
 import { MachinesPageComponent } from './machines-page/machines-page.component'
 import { UsersPageComponent } from './users-page/users-page.component'
 import { AppsPageComponent } from './apps-page/apps-page.component'
@@ -168,6 +167,9 @@ import { PdnsDaemonComponent } from './pdns-daemon/pdns-daemon.component'
 import { TabViewComponent } from './tab-view/tab-view.component'
 import { TriStateCheckboxComponent } from './tri-state-checkbox/tri-state-checkbox.component'
 import { UserFormComponent } from './user-form/user-form.component'
+import { TextFileViewerComponent } from './text-file-viewer/text-file-viewer.component'
+import { Bind9ConfigPreviewComponent } from './bind9-config-preview/bind9-config-preview.component'
+import { Bind9DaemonControlsComponent } from './bind9-daemon-controls/bind9-daemon-controls.component'
 
 /** Create the OpenAPI client configuration. */
 export function cfgFactory() {
@@ -311,7 +313,6 @@ const AuraBluePreset = definePreset(Aura, {
         LoginScreenComponent,
         DashboardComponent,
         HostsTableComponent,
-        SwaggerUiComponent,
         MachinesPageComponent,
         LocaltimePipe,
         UsersPageComponent,
@@ -464,6 +465,9 @@ const AuraBluePreset = definePreset(Aura, {
         InputIcon,
         TriStateCheckboxComponent,
         UserFormComponent,
+        TextFileViewerComponent,
+        Bind9ConfigPreviewComponent,
+        Bind9DaemonControlsComponent,
     ],
     providers: [
         {
