@@ -14,7 +14,7 @@ import { ConfigReports, ServicesService } from '../backend'
 import { LocaltimePipe } from '../pipes/localtime.pipe'
 import { TableModule } from 'primeng/table'
 import { ChipModule } from 'primeng/chip'
-import { OverlayPanelModule } from 'primeng/overlaypanel'
+import { PopoverModule } from 'primeng/popover'
 import { ConfigCheckerPreferenceUpdaterComponent } from '../config-checker-preference-updater/config-checker-preference-updater.component'
 import { ConfigCheckerPreferencePickerComponent } from '../config-checker-preference-picker/config-checker-preference-picker.component'
 import { HelpTipComponent } from '../help-tip/help-tip.component'
@@ -49,7 +49,7 @@ describe('ConfigReviewPanelComponent', () => {
                 TableModule,
                 ChipModule,
                 FormsModule,
-                OverlayPanelModule,
+                PopoverModule,
                 ToggleButtonModule,
                 DataViewModule,
                 ManagedAccessDirective,
@@ -485,14 +485,14 @@ describe('ConfigReviewPanelComponent', () => {
 
     it('should open the config review checkers panel with minimal layout on click the button', () => {
         // The button should exist.
-        const buttonElement = fixture.debugElement.query(By.css('button[label=Checkers]'))
+        const buttonElement = fixture.debugElement.query(By.css('p-button[label=Checkers] button'))
         expect(buttonElement).not.toBeNull()
 
         // The checker updater shouldn't exist yet.
         let pickerElement = fixture.debugElement.query(By.directive(ConfigCheckerPreferenceUpdaterComponent))
         expect(pickerElement).toBeNull()
 
-        buttonElement.triggerEventHandler('click', new Event('click'))
+        buttonElement.nativeElement.click()
         fixture.detectChanges()
 
         // The checker picker should be presented.
