@@ -1,13 +1,16 @@
 import { Meta, StoryObj, moduleMetadata } from '@storybook/angular'
 import { DelegatedPrefixBarComponent } from './delegated-prefix-bar.component'
 import { DelegatedPrefixPool } from '../backend'
+import { TooltipModule } from 'primeng/tooltip'
+import { UtilizationBarComponent } from '../utilization-bar/utilization-bar.component'
 
 export default {
     title: 'App/DelegatedPrefixBar',
     component: DelegatedPrefixBarComponent,
     decorators: [
         moduleMetadata({
-            imports: [],
+            imports: [TooltipModule],
+            declarations: [UtilizationBarComponent],
         }),
     ],
 } as Meta
@@ -16,7 +19,7 @@ type Story = StoryObj<DelegatedPrefixBarComponent>
 
 export const StandardPrefix: Story = {
     args: {
-        prefix: {
+        pool: {
             prefix: '3001:42::/64',
             delegatedLength: 80,
         } as DelegatedPrefixPool,
@@ -25,7 +28,7 @@ export const StandardPrefix: Story = {
 
 export const ExcludedPrefix: Story = {
     args: {
-        prefix: {
+        pool: {
             prefix: '2001:db8:1:8000::/48',
             delegatedLength: 64,
             excludedPrefix: '2001:db8:1:8000:cafe:80::/72',
