@@ -9,9 +9,9 @@ import {
 import { By } from '@angular/platform-browser'
 import { NoopAnimationsModule } from '@angular/platform-browser/animations'
 import { CheckboxModule } from 'primeng/checkbox'
-import { DropdownModule } from 'primeng/dropdown'
+import { SelectModule } from 'primeng/select'
 import { InputNumberModule } from 'primeng/inputnumber'
-import { OverlayPanelModule } from 'primeng/overlaypanel'
+import { PopoverModule } from 'primeng/popover'
 import { ToggleButtonModule } from 'primeng/togglebutton'
 import { SplitButtonModule } from 'primeng/splitbutton'
 import { createDefaultDhcpOptionFormGroup } from '../forms/dhcp-option-form'
@@ -22,6 +22,8 @@ import { IPType } from '../iptype'
 import { DhcpOptionFieldFormGroup, DhcpOptionFieldType } from '../forms/dhcp-option-field'
 import { DividerModule } from 'primeng/divider'
 import { FloatLabelModule } from 'primeng/floatlabel'
+import { ButtonModule } from 'primeng/button'
+import { TagModule } from 'primeng/tag'
 
 describe('DhcpOptionFormComponent', () => {
     let component: DhcpOptionFormComponent
@@ -32,16 +34,18 @@ describe('DhcpOptionFormComponent', () => {
             providers: [UntypedFormBuilder],
             imports: [
                 CheckboxModule,
-                DropdownModule,
+                SelectModule,
                 FormsModule,
                 InputNumberModule,
                 NoopAnimationsModule,
-                OverlayPanelModule,
+                PopoverModule,
                 ReactiveFormsModule,
                 SplitButtonModule,
                 ToggleButtonModule,
                 DividerModule,
                 FloatLabelModule,
+                ButtonModule,
+                TagModule,
             ],
             declarations: [DhcpOptionFormComponent, DhcpOptionSetFormComponent, HelpTipComponent],
         }).compileComponents()
@@ -64,7 +68,7 @@ describe('DhcpOptionFormComponent', () => {
 
     it('should display DHCPv4 options selection', () => {
         // By default, the component should display a dropdown with option codes.
-        const dropdownEl = fixture.debugElement.query(By.css('p-dropdown'))
+        const dropdownEl = fixture.debugElement.query(By.css('p-select'))
         expect(dropdownEl).toBeTruthy()
 
         // The dropdown should include a placeholder informing about its purpose.
@@ -85,7 +89,7 @@ describe('DhcpOptionFormComponent', () => {
         fixture.detectChanges()
 
         // There should be a dropdown.
-        const dropdownEl = fixture.debugElement.query(By.css('p-dropdown'))
+        const dropdownEl = fixture.debugElement.query(By.css('p-select'))
         expect(dropdownEl).toBeTruthy()
 
         // The dropdown should include a placeholder informing about its purpose.
@@ -132,12 +136,12 @@ describe('DhcpOptionFormComponent', () => {
         expect(fixture.debugElement.query(By.css('[title="Help for binary Option Field"]'))).toBeTruthy()
 
         // Find the last button. It should delete the option field.
-        const allBtns = fixture.debugElement.queryAll(By.css('button'))
+        const allBtns = fixture.debugElement.queryAll(By.css('button.p-button-danger'))
         const deleteFieldBtn = allBtns[allBtns.length - 1]
         expect(deleteFieldBtn).toBeTruthy()
 
         // Click the button to delete the field.
-        deleteFieldBtn.nativeElement.dispatchEvent(new Event('click'))
+        deleteFieldBtn.nativeElement.click()
         fixture.detectChanges()
 
         // There should be no option fields and the tag should be back.
@@ -199,11 +203,11 @@ describe('DhcpOptionFormComponent', () => {
         let numberFieldEls = containerEl.queryAll(By.css('p-inputNumber'))
         expect(numberFieldEls.length).toBe(9)
 
-        const deleteBtns = containerEl.queryAll(By.css('button'))
+        const deleteBtns = containerEl.queryAll(By.css('button.p-button-danger'))
         expect(deleteBtns.length).toBe(14)
 
         // Delete uint32 option field.
-        deleteBtns[5].nativeElement.dispatchEvent(new Event('click'))
+        deleteBtns[5].nativeElement.click()
         fixture.detectChanges()
 
         // Make sure it is gone.
@@ -245,7 +249,7 @@ describe('DhcpOptionFormComponent', () => {
         expect(suboptionEl.query(By.css('textarea'))).toBeTruthy()
 
         // Simulate deleting the suboption.
-        deleteBtn.parent.nativeElement.dispatchEvent(new Event('click'))
+        deleteBtn.parent.nativeElement.click()
         fixture.detectChanges()
 
         // It should be gone.
