@@ -7,7 +7,7 @@ import { ConfirmationService, MessageService } from 'primeng/api'
 import { ChipModule } from 'primeng/chip'
 import { ConfirmDialogModule } from 'primeng/confirmdialog'
 import { FieldsetModule } from 'primeng/fieldset'
-import { OverlayPanelModule } from 'primeng/overlaypanel'
+import { PopoverModule } from 'primeng/popover'
 import { ProgressSpinnerModule } from 'primeng/progressspinner'
 import { TableModule } from 'primeng/table'
 import { TagModule } from 'primeng/tag'
@@ -17,14 +17,15 @@ import { TreeModule } from 'primeng/tree'
 import { DhcpClientClassSetViewComponent } from '../dhcp-client-class-set-view/dhcp-client-class-set-view.component'
 import { DhcpOptionSetViewComponent } from '../dhcp-option-set-view/dhcp-option-set-view.component'
 import { HelpTipComponent } from '../help-tip/help-tip.component'
-import { HostTabComponent } from '../host-tab/host-tab.component'
+import { HostTabComponent } from './host-tab.component'
 import { IdentifierComponent } from '../identifier/identifier.component'
 import { toastDecorator } from '../utils-stories'
 import { EntityLinkComponent } from '../entity-link/entity-link.component'
 import { ByteCharacterComponent } from '../byte-character/byte-character.component'
 import { provideHttpClientTesting } from '@angular/common/http/testing'
-import { MessagesModule } from 'primeng/messages'
 import { HostDataSourceLabelComponent } from '../host-data-source-label/host-data-source-label.component'
+import { ButtonModule } from 'primeng/button'
+import { MessageModule } from 'primeng/message'
 
 export default {
     title: 'App/HostTab',
@@ -45,12 +46,13 @@ export default {
         }),
         moduleMetadata({
             imports: [
+                ButtonModule,
                 ChipModule,
                 ConfirmDialogModule,
                 FieldsetModule,
                 FormsModule,
-                MessagesModule,
-                OverlayPanelModule,
+                MessageModule,
+                PopoverModule,
                 ProgressSpinnerModule,
                 TableModule,
                 RouterModule,
