@@ -7,6 +7,7 @@ import { RenameAppDialogComponent } from './rename-app-dialog.component'
 import { DialogModule } from 'primeng/dialog'
 import { SharedModule } from 'primeng/api'
 import { NoopAnimationsModule } from '@angular/platform-browser/animations'
+import { ButtonModule } from 'primeng/button'
 
 describe('RenameAppDialogComponent', () => {
     let component: RenameAppDialogComponent
@@ -14,7 +15,7 @@ describe('RenameAppDialogComponent', () => {
 
     beforeEach(waitForAsync(() => {
         TestBed.configureTestingModule({
-            imports: [FormsModule, DialogModule, SharedModule, NoopAnimationsModule],
+            imports: [FormsModule, DialogModule, SharedModule, NoopAnimationsModule, ButtonModule],
             declarations: [RenameAppDialogComponent],
         }).compileComponents()
     }))
@@ -182,9 +183,8 @@ describe('RenameAppDialogComponent', () => {
         expect(appNameInputError.nativeElement.innerText).toBe('An app with this name already exists.')
 
         // Ensure that the submit button is disabled.
-        const renameButton = fixture.debugElement.query(By.css('#rename-button'))
-        expect(renameButton.properties.hasOwnProperty('disabled')).toBeTrue()
-        expect(renameButton.properties.disabled).toBeTrue()
+        const renameButton = fixture.debugElement.query(By.css('#rename-button button[disabled]'))
+        expect(renameButton).toBeTruthy()
     })
 
     it('should reject a name referencing non-existing machine', () => {
