@@ -8,6 +8,7 @@ import { getErrorMessage } from '../utils'
 
 @Component({
     selector: 'app-shared-network-tab',
+    standalone: false,
     templateUrl: './shared-network-tab.component.html',
     styleUrls: ['./shared-network-tab.component.sass'],
 })
@@ -113,11 +114,17 @@ export class SharedNetworkTabComponent implements OnInit {
     /**
      * Displays a dialog to confirm shared network deletion.
      */
-    confirmDeleteSharedNetwork() {
+    confirmDeleteSharedNetwork(id: string) {
         this.confirmService.confirm({
+            key: id,
             message: 'Are you sure that you want to permanently delete this shared network and its subnets?',
             header: 'Delete Shared Network',
             icon: 'pi pi-exclamation-triangle',
+            rejectButtonProps: { text: true, icon: 'pi pi-times', label: 'Cancel' },
+            acceptButtonProps: {
+                icon: 'pi pi-check',
+                label: 'Delete',
+            },
             accept: () => {
                 this.deleteSharedNetwork()
             },
