@@ -282,19 +282,21 @@ describe('SharedNetworksTableComponent', () => {
         ]
         getNetworksSpy = spyOn(dhcpService, 'getSharedNetworks')
         // Prepare response when no filtering is applied.
-        getNetworksSpy.withArgs(0, 10, null, null, null).and.returnValue(of(fakeResponses[0]))
+        getNetworksSpy.withArgs(0, 10, null, null, null, null, null).and.returnValue(of(fakeResponses[0]))
+        getNetworksSpy.withArgs(0, 10, null, '4', null, null, null).and.returnValue(of(fakeResponses[0]))
+        getNetworksSpy.withArgs(0, 10, null, '6', null, null, null).and.returnValue(of(fakeResponses[0]))
         // Prepare response when shared networks are filtered by text to get an item without stats.
-        getNetworksSpy.withArgs(0, 10, null, null, 'frog-no-stats').and.returnValue(of(fakeResponses[1]))
+        getNetworksSpy.withArgs(0, 10, null, null, 'frog-no-stats', null, null).and.returnValue(of(fakeResponses[1]))
         // Prepare response when shared networks are filtered by text to get an item with 4 subnets.
-        getNetworksSpy.withArgs(0, 10, null, null, 'cat').and.returnValue(of(fakeResponses[2]))
+        getNetworksSpy.withArgs(0, 10, null, null, 'cat', null, null).and.returnValue(of(fakeResponses[2]))
         // Prepare responses for table filtering tests.
-        getNetworksSpy.withArgs(0, 10, 5, null, 'cat').and.returnValue(of(fakeResponses[2]))
-        getNetworksSpy.withArgs(0, 10, 5, 6, 'cat').and.returnValue(of(fakeResponses[2]))
-        getNetworksSpy.withArgs(0, 10, 1, null, null).and.returnValue(of(fakeResponses[2]))
+        getNetworksSpy.withArgs(0, 10, 5, null, 'cat', null, null).and.returnValue(of(fakeResponses[2]))
+        getNetworksSpy.withArgs(0, 10, 5, 6, 'cat', null, null).and.returnValue(of(fakeResponses[2]))
+        getNetworksSpy.withArgs(0, 10, 1, null, null, null, null).and.returnValue(of(fakeResponses[2]))
 
         fixture.detectChanges()
 
-        component.clearTableState()
+        component.table?.clear()
         fixture.detectChanges()
     })
 
@@ -310,7 +312,7 @@ describe('SharedNetworksTableComponent', () => {
         fixture.detectChanges()
 
         // Data loading should be done.
-        expect(getNetworksSpy).toHaveBeenCalledOnceWith(0, 10, null, null, null)
+        expect(getNetworksSpy).toHaveBeenCalledOnceWith(0, 10, null, null, null, null, null)
         expect(component.dataLoading).toBeFalse()
         // Records count should be updated.
         expect(component.totalRecords).toBe(10496)
@@ -322,7 +324,7 @@ describe('SharedNetworksTableComponent', () => {
         fixture.detectChanges()
 
         // Assert
-        expect(getNetworksSpy).toHaveBeenCalledOnceWith(0, 10, null, null, null)
+        expect(getNetworksSpy).toHaveBeenCalledOnceWith(0, 10, null, null, null, null, null)
         const stats: { [key: string]: BigInt } = component.dataCollection[0].stats as any
         expect(stats['assigned-addresses']).toBe(
             BigInt('12345678901234567890123456789012345678901234567890123456789012345678901234567890')
@@ -341,7 +343,7 @@ describe('SharedNetworksTableComponent', () => {
         fixture.detectChanges()
 
         // Assert
-        expect(getNetworksSpy).toHaveBeenCalledOnceWith(0, 10, null, null, null)
+        expect(getNetworksSpy).toHaveBeenCalledOnceWith(0, 10, null, null, null, null, null)
         const stats: { [key: string]: BigInt } = component.dataCollection[0].subnets[0].stats as any
         expect(stats['assigned-addresses']).toBe(BigInt('42'))
         expect(stats['total-addresses']).toBe(
@@ -360,25 +362,34 @@ describe('SharedNetworksTableComponent', () => {
         fixture.detectChanges()
 
         // Assert
-        expect(getNetworksSpy).toHaveBeenCalledWith(0, 10, null, null, 'frog-no-stats')
+        expect(getNetworksSpy).toHaveBeenCalledWith(0, 10, null, null, 'frog-no-stats', null, null)
         expect(component.dataCollection[0].stats).toBeUndefined()
         // No throw
     })
 
-    it('should detect IPv6 subnets', () => {
-        const networks: SharedNetwork[] = [
-            {
-                subnets: [{ subnet: '10.0.0.0/8' }, { subnet: '192.168.0.0/16' }],
+    it('should detect IPv6 subnets included in filter', () => {
+        expect(component.ipV6SubnetsFilterIncluded()).toBeTrue()
+        expect(component.emptyMessageColspan()).toEqual(9)
+
+        component.loadData({
+            first: 0,
+            rows: 10,
+            filters: {
+                dhcpVersion: { value: '4' },
             },
-        ]
-
-        component.dataCollection = networks
-        expect(component.isAnyIPv6SubnetVisible).toBeFalse()
-
-        networks.push({
-            subnets: [{ subnet: 'fe80::/64' }],
         })
-        expect(component.isAnyIPv6SubnetVisible).toBeTrue()
+        expect(component.ipV6SubnetsFilterIncluded()).toBeFalse()
+        expect(component.emptyMessageColspan()).toEqual(6)
+
+        component.loadData({
+            first: 0,
+            rows: 10,
+            filters: {
+                dhcpVersion: { value: '6' },
+            },
+        })
+        expect(component.ipV6SubnetsFilterIncluded()).toBeTrue()
+        expect(component.emptyMessageColspan()).toEqual(9)
     })
 
     it('should display proper utilization bars', async () => {
@@ -389,7 +400,7 @@ describe('SharedNetworksTableComponent', () => {
         await fixture.whenStable()
         fixture.detectChanges()
 
-        expect(getNetworksSpy).toHaveBeenCalledWith(0, 10, null, null, 'cat')
+        expect(getNetworksSpy).toHaveBeenCalledWith(0, 10, null, null, 'cat', null, null)
         expect(component.dataCollection.length).toBe(1)
         expect(component.dataCollection[0].subnets.length).toBe(4)
 
@@ -499,4 +510,68 @@ describe('SharedNetworksTableComponent', () => {
         // Assert
         expect(component.filterTable).toHaveBeenCalledOnceWith(1, component.table.filters['appId'] as FilterMetadata)
     }))
+
+    it('should get total delegated prefixes for v6 network', () => {
+        // Arrange
+        const network: SharedNetwork = {
+            universe: 6,
+            stats: {
+                'total-pds': 10,
+            },
+        }
+
+        // Act
+        const result = component.getTotalDelegatedPrefixes(network)
+
+        // Assert
+        expect(result).toEqual(10)
+    })
+
+    it('should get assigned delegated prefixes for v6 network', () => {
+        // Arrange
+        const network: SharedNetwork = {
+            universe: 6,
+            stats: {
+                'assigned-pds': 10,
+            },
+        }
+
+        // Act
+        const result = component.getAssignedDelegatedPrefixes(network)
+
+        // Assert
+        expect(result).toEqual(10)
+    })
+
+    it('should not get total delegated prefixes for v4 network', () => {
+        // Arrange
+        const network: SharedNetwork = {
+            universe: 4,
+            stats: {
+                'total-addresses': 10,
+            },
+        }
+
+        // Act
+        const result = component.getTotalDelegatedPrefixes(network)
+
+        // Assert
+        expect(result).toBeUndefined()
+    })
+
+    it('should not get assigned delegated prefixes for v4 network', () => {
+        // Arrange
+        const network: SharedNetwork = {
+            universe: 4,
+            stats: {
+                'assigned-addresses': 10,
+            },
+        }
+
+        // Act
+        const result = component.getAssignedDelegatedPrefixes(network)
+
+        // Assert
+        expect(result).toBeUndefined()
+    })
 })
