@@ -6,7 +6,7 @@ import { DhcpClientClassSetFormComponent } from '../dhcp-client-class-set-form/d
 import { ButtonModule } from 'primeng/button'
 import { CheckboxModule } from 'primeng/checkbox'
 import { AutoComplete, AutoCompleteModule } from 'primeng/autocomplete'
-import { DropdownModule } from 'primeng/dropdown'
+import { SelectModule } from 'primeng/select'
 import {
     FormControl,
     FormGroup,
@@ -19,14 +19,14 @@ import { InputNumberModule } from 'primeng/inputnumber'
 import { NoopAnimationsModule } from '@angular/platform-browser/animations'
 import { TableModule } from 'primeng/table'
 import { TagModule } from 'primeng/tag'
-import { TriStateCheckboxModule } from 'primeng/tristatecheckbox'
-import { OverlayPanelModule } from 'primeng/overlaypanel'
+import { PopoverModule } from 'primeng/popover'
 import { StorkValidators } from '../validators'
 import { By } from '@angular/platform-browser'
 import { ArrayValueSetFormComponent } from '../array-value-set-form/array-value-set-form.component'
 import { HelpTipComponent } from '../help-tip/help-tip.component'
 import { MultiSelectModule } from 'primeng/multiselect'
 import { InputTextModule } from 'primeng/inputtext'
+import { TriStateCheckboxComponent } from '../tri-state-checkbox/tri-state-checkbox.component'
 
 /**
  * Intrface to the form used in the unit tests.
@@ -59,17 +59,17 @@ describe('SharedParametersFormComponent', () => {
                 ButtonModule,
                 CheckboxModule,
                 AutoCompleteModule,
-                DropdownModule,
+                SelectModule,
                 FormsModule,
                 InputNumberModule,
                 MultiSelectModule,
                 NoopAnimationsModule,
                 TableModule,
                 TagModule,
-                TriStateCheckboxModule,
-                OverlayPanelModule,
+                PopoverModule,
                 ReactiveFormsModule,
                 InputTextModule,
+                TriStateCheckboxComponent,
             ],
         }).compileComponents()
 
@@ -184,7 +184,7 @@ describe('SharedParametersFormComponent', () => {
         let labelDiv = allWrapperDivs[0].queryAll(By.css('div.font-semibold'))
         expect(labelDiv.length).toBe(1)
         expect(labelDiv[0].nativeElement.innerText).toBe('Allocator')
-        let controls = allWrapperDivs[0].queryAll(By.css('p-dropdown'))
+        let controls = allWrapperDivs[0].queryAll(By.css('p-select'))
         expect(controls.length).toBe(2)
         let tags = allWrapperDivs[0].queryAll(By.css('p-tag'))
         expect(tags.length).toBe(2)
@@ -252,7 +252,7 @@ describe('SharedParametersFormComponent', () => {
         labelDiv = allWrapperDivs[4].queryAll(By.css('div.font-semibold'))
         expect(labelDiv.length).toBe(1)
         expect(labelDiv[0].nativeElement.innerText).toBe('DDNS Override Client Update')
-        controls = allWrapperDivs[4].queryAll(By.css('p-triStateCheckbox'))
+        controls = allWrapperDivs[4].queryAll(By.css('div:not(.hidden) > p-checkbox'))
         expect(controls.length).toBe(1)
         tags = allWrapperDivs[4].queryAll(By.css('p-tag'))
         expect(tags.length).toBe(0)
@@ -367,7 +367,7 @@ describe('SharedParametersFormComponent', () => {
         fixture.detectChanges()
 
         // The expanded list should contain the client classes.
-        const classSpans = divs[1].queryAll(By.css('ul.p-autocomplete-items li span'))
+        const classSpans = divs[1].queryAll(By.css('ul.p-autocomplete-list li span'))
         expect(classSpans).toBeTruthy()
         expect(classSpans.length).toBe(2)
         expect(classSpans.map((de) => de.nativeElement.innerText)).toEqual(jasmine.arrayContaining(['foo', 'bar']))
