@@ -24,6 +24,7 @@ import { lastValueFrom } from 'rxjs'
  */
 @Component({
     selector: 'app-subnet-form',
+    standalone: false,
     templateUrl: './subnet-form.component.html',
     styleUrls: ['./subnet-form.component.sass'],
 })
@@ -107,11 +108,22 @@ export class SubnetFormComponent implements OnInit, OnDestroy {
         // edits.
         if (this.state.preserved) {
             this.state.loaded = true
+            if (!this.state.transactionID) {
+                this._beginTransaction()
+            }
+
             return
         }
 
-        // We currently only support updating a subnet. In this case the subnet
-        // id must be provided.
+        // Begin transaction.
+        this._beginTransaction()
+    }
+
+    /**
+     * Begins either create or update transaction.
+     * @private
+     */
+    private _beginTransaction() {
         if (this.subnetId) {
             // Send POST to /subnets/{id}/transaction/new.
             this.updateSubnetBegin()
@@ -147,7 +159,7 @@ export class SubnetFormComponent implements OnInit, OnDestroy {
 
         // The server should return new transaction id and a current list of
         // daemons to select.
-        this.state.transactionId = response.id
+        this.state.transactionID = response.id
         this.state.allDaemons = []
         for (let d of response.daemons) {
             this.state.allDaemons.push({
@@ -536,7 +548,7 @@ export class SubnetFormComponent implements OnInit, OnDestroy {
             }
             subnet.id = this.subnetId
             this.dhcpApi
-                .updateSubnetSubmit(this.subnetId, this.state.transactionId, subnet)
+                .updateSubnetSubmit(this.subnetId, this.state.transactionID, subnet)
                 .toPromise()
                 .then(() => {
                     this.messageService.add({
@@ -562,7 +574,7 @@ export class SubnetFormComponent implements OnInit, OnDestroy {
         }
         // Creating a new subnet.
         this.dhcpApi
-            .createSubnetSubmit(this.state.transactionId, subnet)
+            .createSubnetSubmit(this.state.transactionID, subnet)
             .toPromise()
             .then((data) => {
                 this.messageService.add({
