@@ -12,6 +12,7 @@ import { lastValueFrom } from 'rxjs'
  */
 @Component({
     selector: 'app-subnet-tab',
+    standalone: false,
     templateUrl: './subnet-tab.component.html',
     styleUrls: ['./subnet-tab.component.sass'],
 })
@@ -189,11 +190,16 @@ export class SubnetTabComponent implements OnInit {
     /**
      * Displays a dialog to confirm subnet deletion.
      */
-    confirmDeleteSubnet() {
+    confirmDeleteSubnet(id: string): void {
         this.confirmService.confirm({
+            key: id,
             message: 'Are you sure that you want to permanently delete this subnet?',
             header: 'Delete Subnet',
             icon: 'pi pi-exclamation-triangle',
+            rejectButtonProps: { text: true, icon: 'pi pi-times' },
+            acceptButtonProps: {
+                icon: 'pi pi-check',
+            },
             accept: () => {
                 this.deleteSubnet()
             },
