@@ -1,6 +1,6 @@
-import { Component, Input, OnDestroy, OnInit, ViewChild } from '@angular/core'
-import { tableFiltersToQueryParams, tableHasFilter } from '../table'
-import { DHCPService, Subnet } from '../backend'
+import { Component, computed, Input, OnDestroy, OnInit, signal, ViewChild } from '@angular/core'
+import { convertSortingFields, tableFiltersToQueryParams, tableHasFilter } from '../table'
+import { DHCPService, Subnet, SubnetSortField } from '../backend'
 import { Table, TableLazyLoadEvent } from 'primeng/table'
 import { Router } from '@angular/router'
 import { MessageService, TableState } from 'primeng/api'
@@ -58,6 +58,16 @@ export class SubnetsTableComponent implements OnInit, OnDestroy {
      */
     totalRecords: number = 0
 
+    /**
+     * Returns true if the table filtering does not exclude IPv6 subnets.
+     */
+    ipV6SubnetsFilterIncluded = signal<boolean>(true)
+
+    /**
+     * Keeps value for colspan attribute for the table "empty message" placeholder.
+     */
+    emptyMessageColspan = computed<number>(() => (this.ipV6SubnetsFilterIncluded() ? 12 : 9))
+
     /**
      * RxJS Subscription holding all subscriptions to Observables, so that they can be all unsubscribed
      * at once onDestroy.
@@ -84,6 +94,8 @@ export class SubnetsTableComponent implements OnInit, OnDestroy {
         // The goal is to send to backend something as simple as:
         // this.someApi.getSubnets(JSON.stringify(event))
 
+        this.ipV6SubnetsFilterIncluded.set((<FilterMetadata>event.filters['dhcpVersion'])?.value !== '4')
+
         lastValueFrom(
             this.dhcpApi
                 .getSubnets(
@@ -92,7 +104,8 @@ export class SubnetsTableComponent implements OnInit, OnDestroy {
                     (event.filters['appId'] as FilterMetadata)?.value ?? null,
                     (event.filters['subnetId'] as FilterMetadata)?.value ?? null,
                     (event.filters['dhcpVersion'] as FilterMetadata)?.value ?? null,
-                    (event.filters['text'] as FilterMetadata)?.value || null
+                    (event.filters['text'] as FilterMetadata)?.value || null,
+                    ...convertSortingFields<SubnetSortField>(event)
                 )
                 // Custom parsing for statistics
                 .pipe(
@@ -260,10 +273,11 @@ export class SubnetsTableComponent implements OnInit, OnDestroy {
     protected readonly tableHasFilter = tableHasFilter
 
     /**
-     * Clears the PrimeNG table state (filtering, pagination are reset).
+     * Clears the PrimeNG table filtering. As a result, table pagination is also reset.
+     * It doesn't reset the table sorting, if any was applied.
      */
-    clearTableState() {
-        this.table?.clear()
+    clearTableFiltering() {
+        this.table?.clearFilterValues()
         this.router.navigate([])
     }
 
@@ -329,4 +343,10 @@ export class SubnetsTableComponent implements OnInit, OnDestroy {
             this.rows = state.rows ?? 10
         }
     }
+
+    /**
+     * Reference to an enum so it could be used in the HTML template.
+     * @protected
+     */
+    protected readonly SubnetSortField = SubnetSortField
 }
