@@ -1,5 +1,6 @@
-import { Table } from 'primeng/table'
+import { Table, TableLazyLoadEvent } from 'primeng/table'
 import { FilterMetadata } from 'primeng/api/filtermetadata'
+import { SortDir } from './backend'
 
 /**
  * Checks if given PrimeNG table filters contain any non-blank value.
@@ -65,3 +66,22 @@ export function tableFiltersToQueryParams(table: Table) {
     const entries = Object.entries(table.filters).map((entry) => [entry[0], (<FilterMetadata>entry[1]).value])
     return Object.fromEntries(entries)
 }
+
+/**
+ * Function converting PrimeNG table sorting related metadata to REST API
+ * sorting fields format.
+ * @template TSortField type of possible sorting field values
+ * @param event table lazy load event
+ * @returns an array of sorting related fields in REST API format
+ */
+export function convertSortingFields<TSortField>(event: TableLazyLoadEvent): [TSortField, SortDir] {
+    if (!event || !event.sortField) {
+        return [null, null]
+    }
+
+    if (event.sortOrder === null || event.sortOrder === undefined) {
+        return [<TSortField>event.sortField, null]
+    }
+
+    return [<TSortField>event.sortField, event.sortOrder === -1 ? SortDir.Desc : SortDir.Asc]
+}
