@@ -1,8 +1,8 @@
 import { Component, Input, OnDestroy, OnInit } from '@angular/core'
 import { AppType, Severity, VersionFeedback, VersionService } from '../version.service'
-import { Message, MessageService } from 'primeng/api'
+import { ToastMessageOptions, MessageService } from 'primeng/api'
 import { first, Subscription } from 'rxjs'
-import { getErrorMessage } from '../utils'
+import { getErrorMessage, getIconBySeverity } from '../utils'
 import { map } from 'rxjs/operators'
 
 /**
@@ -16,6 +16,7 @@ import { map } from 'rxjs/operators'
  */
 @Component({
     selector: 'app-version-status',
+    standalone: false,
     templateUrl: './version-status.component.html',
     styleUrl: './version-status.component.sass',
 })
@@ -68,7 +69,7 @@ export class VersionStatusComponent implements OnInit, OnDestroy {
     /**
      * Holds PrimeNG Message value for the block message.
      */
-    messages: Message[] | undefined
+    messages: ToastMessageOptions[] | undefined
 
     /**
      * Full name of the app. This is either 'Kea', 'Bind9' or 'Stork agent'. This is computed based on app field.
@@ -102,15 +103,23 @@ export class VersionStatusComponent implements OnInit, OnDestroy {
      * primary one, and the offline will be a fallback option.
      */
     ngOnInit(): void {
-        // Mute version checks for non-ISC apps.
-        if (!this.iscApp) {
-            return
+        switch (this.app) {
+            case 'bind9':
+                this._appName = 'BIND9'
+                break
+            case 'pdns':
+                this._appName = 'PowerDNS'
+                break
+            case 'stork':
+                this._appName = 'Stork agent'
+                break
+            default:
+                this._appName = this.app[0].toUpperCase() + this.app.slice(1)
+                break
         }
-        this._appName = this.app === 'bind9' ? this.app.toUpperCase() : this.app[0].toUpperCase() + this.app.slice(1)
-        this._appName += this.app === 'stork' ? ' agent' : ''
-        if (!this.version) {
-            // Version is a mandatory input. In case it is false (undefined, null, empty string),
-            // simply return. No feedback will be displayed.
+        // Mute version checks for non-ISC apps. Version is mandatory. In case it is
+        // false (undefined, null, empty string), simply return. No feedback will be displayed.
+        if (!this.iscApp || !this.version) {
             return
         }
 
@@ -184,7 +193,7 @@ export class VersionStatusComponent implements OnInit, OnDestroy {
     private setSeverity(feedback: VersionFeedback) {
         this.severity = feedback.severity
         this.feedbackMessages = feedback.messages ?? []
-        const m: Message = {
+        const m: ToastMessageOptions = {
             severity: Severity[feedback.severity],
             summary: `${this.appName} ${this.version}`,
             detail: feedback.messages.join('<br><br>'),
@@ -212,4 +221,16 @@ export class VersionStatusComponent implements OnInit, OnDestroy {
                 break
         }
     }
+
+    /**
+     * Reference to the function so it can be used in the html template.
+     * @protected
+     */
+    protected readonly getIconBySeverity = getIconBySeverity
+
+    /**
+     * Reference to the enum so it can be used in the html template.
+     * @protected
+     */
+    protected readonly Severity = Severity
 }
